@@ -1,10 +1,9 @@
-#!/usr/bin/env python
 #
 # Class and helper functions for handling (Hatari) INI style
 # configuration files: loading, saving, setting/getting variables,
 # mapping them to sections, listing changes
 #
-# Copyright (C) 2008-2012 by Eero Tamminen
+# Copyright (C) 2008-2012,2016-2020 by Eero Tamminen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,6 +25,8 @@ from conftypes import conftypes
 
 def value_to_text(key, value):
     "value_to_text(key, value) -> text, convert Python type to string"
+    #if key not in conftypes:
+    #    print("ERROR: key '%s' missing!" % key)
     assert(key in conftypes)
     valtype = type(value)
     if valtype == bool:
@@ -34,11 +35,12 @@ def value_to_text(key, value):
             text = "TRUE"
         else:
             text = "FALSE"
-    elif valtype == int:
-        assert(conftypes[key] == "Int")
+    elif valtype in (float, int):
+        assert(conftypes[key] in ("Float", "Int"))
         text = str(value)
     else:
-        assert(conftypes[key] == "String")
+        # keyboard key name or any string
+        assert(conftypes[key] in ("Key", "String"))
         if value == None:
             text = ""
         else:
@@ -57,6 +59,9 @@ def text_to_value(text):
         try:
             # integer?
             value = int(text)
+            # if not exact, maybe it's float?
+            if text != '%d' % value:
+                value = float(text)
         except ValueError:
             # string
             value = text
@@ -71,8 +76,12 @@ class ConfigStore:
         "ConfigStore(userconfdir, fgfile[,defaults,miss_is_error])"
         self.defaults = defaults
         self.userpath = self._get_full_userpath(confdirs)
+        # this is only about ConfigStore checks i.e. value existing in
+        # current Hatari config, value_to_text() will still fail if
+        # config name isn't even in conftypes (= all Hatari configs)
         self.miss_is_error = miss_is_error
-    
+        self.changed = False
+
     def _get_full_userpath(self, confdirs):
         "get_userpath(leafdir) -> config file default save path from HOME, CWD or their subdir"
         # user's hatari.cfg can be in home or current work dir,
@@ -90,32 +99,33 @@ class ConfigStore:
     def get_filepath(self, filename):
         "get_filepath(filename) -> return correct full path to config file"
         # user config has preference over system one
+        sep = os.path.sep
         for path in (self.userpath, os.getenv("HATARI_SYSTEM_CONFDIR")):
             if path:
-                file = "%s%c%s" % (path, os.path.sep, filename)
-                if os.path.isfile(file):
-                    return file
+                confpath = "%s%c%s" % (path, sep, filename)
+                if os.path.isfile(confpath):
+                    return confpath
         # writing needs path name although it's missing for reading
-        return "%s%c%s" % (self.userpath, os.path.sep, filename)
-    
+        return "%s%c%s" % (self.userpath, sep, filename)
+
     def load(self, path):
-        "load(path) -> load given configuration file"
+        "load(path): load given configuration file -> message on error, None on success"
         if os.path.isfile(path):
             sections = self._read(path)
             if sections:
                 self.sections = sections
             else:
-                print("ERROR: configuration file loading failed!")
-                return
+                return "loading failed"
         else:
-            print("WARNING: configuration file missing!")
-            if self.defaults:
-                print("-> using dummy 'defaults'.")
+            if not self.defaults:
+                return "file missing"
+            print("-> using dummy 'defaults'.")
             self.sections = self.defaults
         self.path = path
         self.cfgfile = os.path.basename(path)
         self.original = self.get_checkpoint()
         self.changed = False
+        return None
 
     def is_loaded(self):
         "is_loaded() -> True if configuration loading succeeded"
@@ -126,7 +136,7 @@ class ConfigStore:
     def get_path(self):
         "get_path() -> configuration file path"
         return self.path
-    
+
     def _read(self, path):
         "_read(path) -> (all keys, section2key mappings)"
         print("Reading configuration file '%s'..." % path)
@@ -148,10 +158,12 @@ class ConfigStore:
                     seckeys = {}
                 name = line
                 continue
-            if line.find('=') < 0:
+            offset = line.find('=')
+            if offset < 0:
                 print("WARNING: line without key=value pair:\n%s" % line)
                 continue
-            key, text = [string.strip() for string in line.split('=')]
+            key = line[:offset].strip()
+            text = line[offset+1:].strip()
             seckeys[key] = text_to_value(text)
         if seckeys:
             sections[name] = seckeys
@@ -163,7 +175,7 @@ class ConfigStore:
         for section in self.sections.keys():
             checkpoint[section] = self.sections[section].copy()
         return checkpoint
-    
+
     def get_checkpoint_changes(self, checkpoint):
         "get_checkpoint_changes() -> list of (key, value) pairs for later changes"
         changed = []
@@ -174,13 +186,13 @@ class ConfigStore:
                 for key, value in self.sections[section].items():
                     changed.append((key, value))
                 continue
-            for key, value in self.sections[section].items():                    
+            for key, value in self.sections[section].items():
                 if (key not in checkpoint[section] or
                 value != checkpoint[section][key]):
                     text = value_to_text(key, value)
                     changed.append(("%s.%s" % (section, key), text))
         return changed
-    
+
     def revert_to_checkpoint(self, checkpoint):
         "revert_to_checkpoint(checkpoint), revert to given checkpoint"
         self.sections = checkpoint
@@ -202,7 +214,7 @@ class ConfigStore:
         elif self.sections[section][key] != value:
             self.changed = True
         self.sections[section][key] = value
-        
+
     def is_changed(self):
         "is_changed() -> True if current configuration is changed"
         return self.changed
@@ -210,7 +222,7 @@ class ConfigStore:
     def get_changes(self):
         "get_changes(), return (key, value) list for each changed config option"
         return self.get_checkpoint_changes(self.original)
-    
+
     def write(self, fileobj):
         "write(fileobj), write current configuration to given file object"
         sections = list(self.sections.keys())
@@ -248,7 +260,7 @@ class ConfigStore:
         print("Saved configuration file:", self.path)
         self.changed = False
         return self.path
-    
+
     def save_as(self, path):
         "save_as(path) -> path, save configuration to given file and select it"
         assert(path)
