@@ -1,8 +1,8 @@
-#!/usr/bin/python2
+#!/usr/bin/python3
 #
-# A Debug UI for the Hatari, part of PyGtk Hatari UI
+# A Debug UI for the Hatari, part of Python Gtk Hatari UI
 #
-# Copyright (C) 2008-2011 by Eero Tamminen
+# Copyright (C) 2008-2025 by Eero Tamminen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,21 +15,21 @@
 # GNU General Public License for more details.
 
 import os
-# use correct version of pygtk/gtk
-import pygtk
-pygtk.require('2.0')
-import gtk
-import pango
+import gi
+# use correct version of gtk
+gi.require_version('Gtk', '3.0')
+from gi.repository import Gtk
+from gi.repository import Gdk
 
 from config import ConfigStore
 from uihelpers import UInfo, create_button, create_toggle, \
      create_table_dialog, table_add_entry_row, table_add_widget_row, \
-     get_save_filename, FselEntry
+     FselEntry
 from dialogs import TodoDialog, ErrorDialog, AskDialog, KillDialog
 
 
 def dialog_apply_cb(widget, dialog):
-    dialog.response(gtk.RESPONSE_APPLY)
+    dialog.response(Gtk.ResponseType.APPLY)
 
 
 # -------------
@@ -38,13 +38,13 @@ def dialog_apply_cb(widget, dialog):
 class SaveDialog:
     def __init__(self, parent):
         table, self.dialog = create_table_dialog(parent, "Save from memory", 3, 2)
-        self.file = FselEntry(self.dialog)
-        table_add_widget_row(table, 0, "File name:", self.file.get_container())
-        self.address = table_add_entry_row(table, 1, "Save address:", 6)
+        self.fsel = FselEntry(self.dialog, "Memory save file name:")
+        table_add_widget_row(table, 0, 0, "File name:", self.fsel.get_container())
+        self.address = table_add_entry_row(table, 1, 0, "Save address:", 6)
         self.address.connect("activate", dialog_apply_cb, self.dialog)
-        self.length = table_add_entry_row(table, 2, "Number of bytes:", 6)
+        self.length = table_add_entry_row(table, 2, 0, "Number of bytes:", 6)
         self.length.connect("activate", dialog_apply_cb, self.dialog)
-    
+
     def run(self, address):
         "run(address) -> (filename,address,length), all as strings"
         if address:
@@ -53,8 +53,8 @@ class SaveDialog:
         filename = length = None
         while 1:
             response = self.dialog.run()
-            if response == gtk.RESPONSE_APPLY:
-                filename = self.file.get_filename()
+            if response == Gtk.ResponseType.APPLY:
+                filename = self.fsel.get_filename()
                 address_txt = self.address.get_text()
                 length_txt = self.length.get_text()
                 if filename and address_txt and length_txt:
@@ -83,12 +83,12 @@ class SaveDialog:
 
 class LoadDialog:
     def __init__(self, parent):
-        chooser = gtk.FileChooserButton('Select a File')
+        chooser = Gtk.FileChooserButton(title="Select memory file to load:")
         chooser.set_local_only(True)  # Hatari cannot access URIs
         chooser.set_width_chars(12)
         table, self.dialog = create_table_dialog(parent, "Load to memory", 2, 2)
-        self.file = table_add_widget_row(table, 0, "File name:", chooser)
-        self.address = table_add_entry_row(table, 1, "Load address:", 6)
+        self.fsel = table_add_widget_row(table, 0, 0, "File name:", chooser)
+        self.address = table_add_entry_row(table, 1, 0, "Load address:", 6)
         self.address.connect("activate", dialog_apply_cb, self.dialog)
 
     def run(self, address):
@@ -99,8 +99,8 @@ class LoadDialog:
         filename = None
         while 1:
             response = self.dialog.run()
-            if response == gtk.RESPONSE_APPLY:
-                filename = self.file.get_filename()
+            if response == Gtk.ResponseType.APPLY:
+                filename = self.fsel.get_filename()
                 address_txt = self.address.get_text()
                 if filename and address_txt:
                     try:
@@ -119,21 +119,25 @@ class LoadDialog:
 
 class OptionsDialog:
     def __init__(self, parent):
-        self.dialog = gtk.Dialog("Debugger UI options", parent,
-            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
-            (gtk.STOCK_APPLY,  gtk.RESPONSE_APPLY,
-             gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE))
+        self.dialog = Gtk.Dialog("Debugger UI options", parent,
+            modal=True, destroy_with_parent=True)
+        self.dialog.add_buttons(
+            Gtk.STOCK_APPLY,  Gtk.ResponseType.APPLY,
+            Gtk.STOCK_CLOSE, Gtk.ResponseType.CLOSE
+        )
+        lines = Gtk.Scale.new_with_range(Gtk.Orientation.HORIZONTAL, 0, 50, 5)
+        lines.set_tooltip_text("Change view mode afterwards to get new (min size) into effect")
+        lines.set_digits(0)
+        self.lines = lines
 
-        self.lines = gtk.Adjustment(0, 5, 50)
-        scale = gtk.HScale(self.lines)
-        scale.set_digits(0)
-        
-        self.follow_pc = gtk.CheckButton("On stop, set address to PC")
+        follow = Gtk.CheckButton(label="On stop, set address to PC")
+        follow.set_tooltip_text("Run-time option, not saved to Debugger config file")
+        self.follow_pc = follow
 
         vbox = self.dialog.vbox
-        vbox.add(gtk.Label("Memdump/disasm lines:"))
-        vbox.add(scale)
-        vbox.add(self.follow_pc)
+        vbox.add(Gtk.Label(label="Memdump/disasm lines:"))
+        vbox.add(lines)
+        vbox.add(follow)
         vbox.show_all()
 
     def run(self, lines, follow_pc):
@@ -142,7 +146,7 @@ class OptionsDialog:
         self.lines.set_value(lines)
         self.dialog.show_all()
         response = self.dialog.run()
-        if response == gtk.RESPONSE_APPLY:
+        if response == Gtk.ResponseType.APPLY:
             lines = int(self.lines.get_value())
             follow_pc = self.follow_pc.get_active()
         self.dialog.hide()
@@ -154,6 +158,7 @@ class OptionsDialog:
 # constants for the other classes
 class Constants:
     # dump modes
+    NODUMP = 0
     DISASM = 1
     MEMDUMP = 2
     REGISTERS = 3
@@ -177,14 +182,14 @@ class MemoryAddress:
         # widgets
         self.entry, self.memory = self.create_widgets()
         # settings
-        self.dumpmode = Constants.REGISTERS
+        self.dumpmode = Constants.NODUMP
         self.follow_pc = True
         self.lines = 12
         # addresses
         self.first = None
         self.second = None
         self.last = None
-        
+
     def clear(self):
         if self.follow_pc:
             # get first address from PC when next stopped
@@ -193,13 +198,12 @@ class MemoryAddress:
         self.last  = None
 
     def create_widgets(self):
-        entry = gtk.Entry(6)
-        entry.set_width_chars(6)
+        entry = Gtk.Entry(max_length=6, width_chars=7)
         entry.connect("activate", self._entry_cb)
-        memory = gtk.Label()
-        mono = pango.FontDescription("monospace")
-        memory.modify_font(mono)
-        entry.modify_font(mono)
+        memory = Gtk.TextView()
+        memory.set_monospace(True)
+        memory.set_cursor_visible(False)
+        memory.set_editable(False)
         return (entry, memory)
 
     def _entry_cb(self, widget):
@@ -212,36 +216,39 @@ class MemoryAddress:
 
     def reset_entry(self):
         self.entry.set_text("%06X" % self.first)
-        
+
     def get(self):
         return self.first
 
-    def get_memory_label(self):
+    def get_memory_view(self):
         return self.memory
-    
+
     def get_address_entry(self):
         return self.entry
 
     def get_follow_pc(self):
         return self.follow_pc
-    
+
     def set_follow_pc(self, follow_pc):
         self.follow_pc = follow_pc
 
     def get_lines(self):
         return self.lines
-    
+
     def set_lines(self, lines):
         self.lines = lines
-    
+
     def set_dumpmode(self, mode):
+        if mode == self.dumpmode:
+            return
         self.dumpmode = mode
         self.dump()
-        
+
     def dump(self, address = None, move_idx = 0):
         if self.dumpmode == Constants.REGISTERS:
             output = self._get_registers()
-            self.memory.set_label("".join(output))
+            text = self.memory.get_buffer()
+            text.set_text("".join(output))
             return
 
         if not address:
@@ -252,7 +259,7 @@ class MemoryAddress:
         if not address:
             print("ERROR: address needed")
             return
-        
+
         if self.dumpmode == Constants.MEMDUMP:
             output = self._get_memdump(address, move_idx)
         elif self.dumpmode == Constants.DISASM:
@@ -260,21 +267,19 @@ class MemoryAddress:
         else:
             print("ERROR: unknown dumpmode:", self.dumpmode)
             return
-        self.memory.set_label("".join(output))
+        text = self.memory.get_buffer()
+        text.set_text("".join(output))
         if move_idx:
             self.reset_entry()
-    
+
     def _get_registers(self):
         self.hatari.debug_command("r")
         output = self.hatari.get_lines(self.debug_output)
         if not self.first:
             # 2nd last line has first PC in 1st column, last line next PC in 2nd column
-            self.second = int(output[-1][output[-1].find(":")+2:], 16)
-            # OldUAE CPU core has ':' in both
-            offset = output[-2].find(":")
-            if offset < 0:
-                # WinUAE CPU core only in one
-                offset = output[-2].find(" ")
+            offset = output[-1].find(":")
+            self.second = int(output[-1][offset+2:], 16)
+            offset = output[-2].find(" ")
             if offset < 0:
                 print("ERROR: unable to parse register dump line:\n\t'%s'", output[-2])
                 return output
@@ -298,7 +303,10 @@ class MemoryAddress:
         output = self.hatari.get_lines(self.debug_output)
         self.second = address + linewidth
         return output
-        
+
+    def _get_line_addr(self, line, offset):
+        return int(line[offset:line.find(' ')], 16)
+
     def _get_disasm(self, address, move_idx):
         # TODO: uses brute force i.e. ask for more lines that user has
         # requested to be sure that the window is filled, assuming
@@ -332,10 +340,16 @@ class MemoryAddress:
                 output = output[-self.lines:]
             else:
                 output = output[:self.lines]
-        # with disasm need to re-get the addresses from the output
-        self.first  = int(output[0][1:output[0].find(":")], 16)
-        self.second = int(output[1][1:output[1].find(":")], 16)
-        self.last   = int(output[-1][1:output[-1].find(":")], 16)
+        # remove symbol lines from the output for address checking
+        addresses = [line for line in output if ' ' in line]
+        # address offset for UAE core and external disassembler output
+        offset = 0
+        if addresses[0][0] == '$':
+            offset = 1
+        # need to re-get the addresses from the disassembly output
+        self.first  = self._get_line_addr(addresses[0], offset)
+        self.second = self._get_line_addr(addresses[1], offset)
+        self.last   = self._get_line_addr(addresses[-1], offset)
         return output
 
     def _set_clamped(self, first, last):
@@ -353,10 +367,11 @@ class MemoryAddress:
 
 # the Hatari debugger UI class and methods
 class HatariDebugUI:
-    
+
     def __init__(self, hatariobj, do_destroy = False):
-        self.address = MemoryAddress(hatariobj)
         self.hatari = hatariobj
+        self.address = MemoryAddress(hatariobj)
+        self.address.set_dumpmode(Constants.REGISTERS)
         # set when needed/created
         self.dialog_load = None
         self.dialog_save = None
@@ -369,66 +384,65 @@ class HatariDebugUI:
         self.load_options()
         # UI initialization/creation
         self.window = self.create_ui("Hatari Debug UI", do_destroy)
-        
+
     def create_ui(self, title, do_destroy):
         # buttons at top
-        hbox1 = gtk.HBox()
+        hbox1 = Gtk.HBox()
         self.create_top_buttons(hbox1)
 
         # disasm/memory dump at the middle
-        align = gtk.Alignment()
-        # top, bottom, left, right padding
-        align.set_padding(8, 0, 8, 8)
-        align.add(self.address.get_memory_label())
+        addr = self.address.get_memory_view()
 
         # buttons at bottom
-        hbox2 = gtk.HBox()
+        hbox2 = Gtk.HBox()
         self.create_bottom_buttons(hbox2)
 
         # their container
-        vbox = gtk.VBox()
-        vbox.pack_start(hbox1, False)
-        vbox.pack_start(align, True, True)
-        vbox.pack_start(hbox2, False)
-        
+        vbox = Gtk.VBox()
+        vbox.pack_start(hbox1, False, True, 0)
+        vbox.pack_start(addr, True, True, 0)
+        vbox.pack_start(hbox2, False, True, 0)
+
         # and the window for all of this
-        window = gtk.Window(gtk.WINDOW_TOPLEVEL)
-        window.set_events(gtk.gdk.KEY_RELEASE_MASK)
+        window = Gtk.Window(type=Gtk.WindowType.TOPLEVEL)
+        window.set_events(Gdk.EventMask.KEY_RELEASE_MASK)
         window.connect("key_release_event", self.key_event_cb)
         if do_destroy:
             window.connect("delete_event", self.quit)
         else:
             window.connect("delete_event", self.hide)
-        window.set_icon_from_file(UInfo.icon)
+        info = UInfo()
+        window.set_icon_from_file(info.icon)
         window.set_title(title)
         window.add(vbox)
         return window
-    
+
     def create_top_buttons(self, box):
         self.stop_button = create_toggle("Stop", self.stop_cb)
         box.add(self.stop_button)
 
         monitor = create_button("Monitor...", self.monitor_cb)
         box.add(monitor)
-        
+
         buttons = (
             ("<<<", "Page_Up",  -Constants.MOVE_MAX),
             ("<<",  "Up",       -Constants.MOVE_MED),
-            ("<",   "Left",     -Constants.MOVE_MIN),
-            (">",   "Right",     Constants.MOVE_MIN),
+            ("<",   None,       -Constants.MOVE_MIN),
+            (">",   None,        Constants.MOVE_MIN),
             (">>",  "Down",      Constants.MOVE_MED),
             (">>>", "Page_Down", Constants.MOVE_MAX)
         )
         self.keys = {}
         for label, keyname, offset in buttons:
             button = create_button(label, self.set_address_offset, offset)
-            keyval = gtk.gdk.keyval_from_name(keyname)
-            self.keys[keyval] =  offset
+            if keyname:
+                keyval = Gdk.keyval_from_name(keyname)
+                self.keys[keyval] =  offset
             box.add(button)
 
         # to middle of <<>> buttons
         address_entry = self.address.get_address_entry()
-        box.pack_start(address_entry, False)
+        box.pack_start(address_entry, False, True, 0)
         box.reorder_child(address_entry, 5)
 
     def create_bottom_buttons(self, box):
@@ -439,11 +453,10 @@ class HatariDebugUI:
         )
         group = None
         for label, mode in radios:
-            button = gtk.RadioButton(group, label)
+            button = Gtk.RadioButton(label=label, group=group, can_focus=False)
             if not group:
                 group = button
             button.connect("toggled", self.dumpmode_cb, mode)
-            button.unset_flags(gtk.CAN_FOCUS)
             box.add(button)
         group.set_active(True)
 
@@ -474,7 +487,7 @@ class HatariDebugUI:
 
     def set_address_offset(self, widget, move_idx):
         self.address.dump(None, move_idx)
-    
+
     def monitor_cb(self, widget):
         TodoDialog(self.window).run("add register / memory address range monitor window.")
 
@@ -491,46 +504,46 @@ class HatariDebugUI:
         (filename, address, length) = self.dialog_save.run(self.address.get())
         if filename and address and length:
             self.hatari.debug_command("s %s $%06x $%06x" % (filename, address, length))
-        
+
     def options_cb(self, widget):
         if not self.dialog_options:
             self.dialog_options = OptionsDialog(self.window)
-        old_lines = self.config.get("[General]", "nLines")
-        old_follow_pc = self.config.get("[General]", "bFollowPC")
+        old_lines = self.config.get("[Debugger]", "nDisasmLines")
+        old_follow_pc = self.address.get_follow_pc()
         lines, follow_pc = self.dialog_options.run(old_lines, old_follow_pc)
         if lines != old_lines:
-            self.config.set("[General]", "nLines", lines)
+            self.config.set("[Debugger]", "nDisasmLines", lines)
             self.address.set_lines(lines)
         if follow_pc != old_follow_pc:
-            self.config.set("[General]", "bFollowPC", follow_pc)
             self.address.set_follow_pc(follow_pc)
+        self.address.dump()
 
     def load_options(self):
         # TODO: move config to MemoryAddress class?
         # (depends on how monitoring of addresses should work)
         lines = self.address.get_lines()
-        follow_pc = self.address.get_follow_pc()
-        miss_is_error = False # needed for adding windows
+        # ConfigStore does checks and type conversions based on names
+        # of Hatari config sections and keys, so this needs to use
+        # same names to avoid asserts, and it can't e.g. save
+        # follow_pc option value, which will keep as run-time one
         defaults = {
-            "[General]": {
-            	"nLines": lines,
-                "bFollowPC": follow_pc
+            "[Debugger]": {
+                "nDisasmLines": lines,
             }
         }
         userconfdir = ".hatari"
-        config = ConfigStore(userconfdir, defaults, miss_is_error)
-        configpath = config.get_filepath("debugui.cfg")
+        config = ConfigStore(userconfdir, defaults)
+        configpath = config.get_filepath(".debugui.cfg")
         config.load(configpath) # set defaults
         try:
-            self.address.set_lines(config.get("[General]", "nLines"))
-            self.address.set_follow_pc(config.get("[General]", "bFollowPC"))
+            self.address.set_lines(config.get("[Debugger]", "nDisasmLines"))
         except (KeyError, AttributeError):
             ErrorDialog(None).run("Debug UI configuration mismatch!\nTry again after removing: '%s'." % configpath)
         self.config = config
-    
+
     def save_options(self):
         self.config.save()
-    
+
     def show(self):
         self.stop_button.set_active(True)
         self.window.show_all()
@@ -544,7 +557,7 @@ class HatariDebugUI:
 
     def quit(self, widget, arg):
         KillDialog(self.window).run(self.hatari)
-        gtk.main_quit()
+        Gtk.main_quit()
 
 
 def main():
@@ -560,12 +573,11 @@ def main():
         args = None
     hatariobj.run(args)
 
-    info = UInfo()
     debugui = HatariDebugUI(hatariobj, True)
     debugui.window.show_all()
-    gtk.main()
+    Gtk.main()
     debugui.save_options()
 
-    
+
 if __name__ == "__main__":
     main()
