@@ -1,9 +1,8 @@
-#!/usr/bin/env python
 #
 # Classes for Hatari emulator instance and mapping its congfiguration
 # variables with its command line option.
 #
-# Copyright (C) 2008-2016 by Eero Tamminen
+# Copyright (C) 2008-2025 by Eero Tamminen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,7 +15,6 @@
 # GNU General Public License for more details.
 
 import os
-import sys
 import time
 import signal
 import socket
@@ -24,6 +22,11 @@ import select
 from config import ConfigStore
 
 
+def _path_quote(path):
+    "quote spaces in paths as expected by Hatari socket API"
+    return path.replace(" ", "\\ ")
+
+
 # Running Hatari instance
 class Hatari:
     "running hatari instance and methods for communicating with it"
@@ -48,34 +51,30 @@ class Hatari:
 
     def is_compatible(self):
         "check Hatari compatibility and return error string if it's not"
-        error = "Hatari not found or it doesn't support the required --control-socket option!"
+        lines = control = mmu = False
         pipe = os.popen(self.hataribin + " -h")
         for line in pipe.readlines():
             if line.find("--control-socket") >= 0:
-                error = None
-                break
+                control = True
+            elif line.find("--mmu") >= 0:
+                mmu = True
+            lines = True
         try:
             pipe.close()
         except IOError:
             pass
-        return error
-
-    def is_winuae(self):
-        "check whether Hatari has WinUAE CPU core (=more features) or oldUAE one"
-        result = False
-        pipe = os.popen(self.hataribin + " -h")
-        for line in pipe.readlines():
-            if line.find("--mmu") >= 0:
-                result = True
-                break
-        try:
-            pipe.close()
-        except IOError:
-            pass
-        return result
+        if not lines:
+            return "'%s' not found!" % self.hataribin
+        if not control:
+            return "Hatari missing required --control-socket option!"
+        if not mmu:
+            return "Hatari is not the expected (WinUAE CPU core) version!"
+        return None
 
     def save_config(self):
-        os.popen(self.hataribin + " --saveconfig")
+        "ask Hatari to save config.  Return None on success, otherwise Hatari return code"
+        pipe = os.popen(self.hataribin + " --saveconfig")
+        return pipe.close()
 
     def _create_server(self):
         if self.server:
@@ -85,19 +84,19 @@ class Hatari:
             os.unlink(self.controlpath)
         self.server.bind(self.controlpath)
         self.server.listen(1)
-        
+
     def _send_message(self, msg):
         if self.control:
-            self.control.send(msg)
+            self.control.send(bytes(msg, "ASCII"))
             return True
         else:
             print("ERROR: no Hatari (control socket)")
             return False
-        
+
     def change_option(self, option):
         "change_option(option), changes given Hatari cli option"
         return self._send_message("hatari-option %s\n" % option)
-        
+
     def set_path(self, key, path):
         "set_path(key, path), sets path with given key"
         return self._send_message("hatari-path %s %s\n" % (key, path))
@@ -109,7 +108,7 @@ class Hatari:
             return self._send_message("hatari-enable %s\n" % device)
         else:
             return self._send_message("hatari-disable %s\n" % device)
-        
+
     def trigger_shortcut(self, shortcut):
         "trigger_shortcut(shortcut), triggers given Hatari (keyboard) shortcut"
         return self._send_message("hatari-shortcut %s\n" % shortcut)
@@ -129,7 +128,7 @@ class Hatari:
     def unpause(self):
         "unpause(), continues Hatari emulation"
         return self._send_message("hatari-cont\n")
-    
+
     def _open_output_file(self, hataricommand, option, path):
         if os.path.exists(path):
             os.unlink(path)
@@ -137,7 +136,7 @@ class Hatari:
         #       reads only byte at the time and stops after first newline)?
         #os.mkfifo(path)
         #raw_input("attach strace now, then press Enter\n")
-        
+
         # ask Hatari to open/create the requested output file...
         hataricommand("%s %s" % (option, path))
         wait = 0.025
@@ -160,7 +159,7 @@ class Hatari:
     def open_log_output(self):
         "open_trace_output() -> file, opens Hatari debug log file"
         return self._open_output_file(self.change_option, "--log-file", self.logpath)
-    
+
     def get_lines(self, fileobj):
         "get_lines(file) -> list of lines readable from given Hatari output file"
         # wait until data is available, then wait for some more
@@ -179,13 +178,13 @@ class Hatari:
 
     def get_embed_info(self):
         "get_embed_info() -> (width, height), get embedded Hatari window size"
-        width, height = self.control.recv(12).split("x")
+        width, height = self.control.recv(12).split(b"x")
         return (int(width), int(height))
 
     def get_control_socket(self):
         "get_control_socket() -> socket which can be checked for embed ID changes"
         return self.control
-        
+
     def is_running(self):
         "is_running() -> bool, True if Hatari is running, False otherwise"
         if not self.pid:
@@ -200,7 +199,7 @@ class Hatari:
                 self.control = None
             return False
         return True
-    
+
     def run(self, extra_args = None, parent_id = None):
         "run([embedding args][,parent window ID]), runs Hatari"
         # if parent_win given, embed Hatari to it
@@ -237,7 +236,7 @@ class Hatari:
         # it for some reason doesn't listen to any events delivered to that
         # window nor implements XEMBED protocol to get them in a way most
         # friendly to embedder:
-        #   http://standards.freedesktop.org/xembed-spec/latest/
+        #   https://specifications.freedesktop.org/xembed-spec/latest/
         #
         # Instead we tell hatari to reparent itself after creating
         # its own window into this program widget window
@@ -268,34 +267,66 @@ class Hatari:
 # it needs actual method for (each) setting.
 class HatariConfigMapping(ConfigStore):
     _paths = {
-        "memauto": ("[Memory]", "szAutoSaveFileName", "Automatic memory snapshot"),
-        "memsave": ("[Memory]", "szMemoryCaptureFileName", "Manual memory snapshot"),
-        "midiin":  ("[Midi]", "sMidiInFileName", "Midi input"),
-        "midiout": ("[Midi]", "sMidiOutFileName", "Midi output"),
-        "rs232in": ("[RS232]", "szInFileName", "RS232 I/O input"),
-        "rs232out": ("[RS232]", "szOutFileName", "RS232 I/O output"),
-        "printout": ("[Printer]", "szPrintToFileName", "Printer output"),
-        "soundout": ("[Sound]", "szYMCaptureFileName", "Sound output")
+        "memauto":    ("[Memory]", "szAutoSaveFileName", "Memory snapshot, automatic"),
+        "memsave":    ("[Memory]", "szMemoryCaptureFileName", "Memory snapshot, manual"),
+        "midiin":     ("[Midi]", "sMidiInFileName", "Midi input"),
+        "midiout":    ("[Midi]", "sMidiOutFileName", "Midi output"),
+        "rs232in":    ("[RS232]", "szInFileName", "RS232: MFP IO input"),
+        "rs232out":   ("[RS232]", "szOutFileName", "RS232: MFP IO output"),
+        "sccain":     ("[RS232]", "SccAInFileName", "RS232: SCC-A IO input"),
+        "sccaout":    ("[RS232]", "SccAOutFileName", "RS232: SCC-A IO output"),
+        "sccalanin":  ("[RS232]", "SccAInFileName", "RS232: SCC-A Lan IO input"),
+        "sccalanout": ("[RS232]", "SccAOutFileName", "RS232: SCC-A Lan IO output"),
+        "sccbin":     ("[RS232]", "SccBInFileName", "RS232: SCC-B IO input"),
+        "sccbout":    ("[RS232]", "SccBOutFileName", "RS232: SCC-B IO output"),
+        "printout":   ("[Printer]", "szPrintToFileName", "Printer output"),
+        "soundout":   ("[Sound]", "szYMCaptureFileName", "Sound output")
     }
-    has_modeltype = True  # from v2.0 onwards
-    has_keepstres = True  # only with SDL1
+    # enable Hatari v2.5+ options
+    has_opts_2_6 = True
+    has_opts_2_5 = True
+
     "access methods to Hatari configuration file variables and command line options"
     def __init__(self, hatari):
         confdirs = [".config/hatari", ".hatari"]
         ConfigStore.__init__(self, confdirs)
         conffilename = "hatari.cfg"
-        self.load(self.get_filepath(conffilename))
-
+        confpath = self.get_filepath(conffilename)
+        print("confpath: %s" % confpath)
+        error = self.load(confpath)
+        if error:
+            print("WARNING: %s %s!" % (confpath, error))
+        else:
+            print("loaded config: %s" % confpath)
         self._hatari = hatari
         self._lock_updates = False
         self._desktop_w = 0
         self._desktop_h = 0
         self._options = []
-        self._winuae = hatari.is_winuae()
-        # initialize has_* attribs for things that may not be anymore
-        # valid on Hatari config file and/or command line
-        self.get_machine()
-        self.get_desktop_st()
+
+    def init_compat(self):
+        "do config mapping initializations needing config loading to have succeeded"
+        # initialize has_opts_<version> attribs for things that may not
+        # be anymore valid on Hatari config file and/or command line
+        try:
+            # added for Hatari >v2.5
+            self.get("[RS232]", "bCpuDataCache")
+            return
+        except KeyError:
+            pass
+        self.has_opts_2_6 = False
+        try:
+            # added for Hatari v2.5
+            self.get("[RS232]", "EnableSccA")
+            return
+        except KeyError:
+            pass
+        self.has_opts_2_5 = False
+        # drop v2.5 keys and use v2.4 option names
+        print("Hatari v2.5 option(s) missing, reverting to v2.4 ones")
+        for key in ("sccain","sccaout","sccalanin","sccalanout","sccbin"):
+            del(self._paths[key])
+        self._paths["sccbout"] = ("[RS232]", "sSccBOutFileName", "RS232: SCC-B IO output")
 
     def validate(self):
         "exception is thrown if the loaded configuration isn't compatible"
@@ -316,19 +347,25 @@ class HatariConfigMapping(ConfigStore):
             else:
                 getattr(self, method)()
 
-    def _change_option(self, option, quoted = None):
-        "handle option changing, and quote spaces for quoted part of it"
-        if quoted:
-            option = "%s %s" % (option, quoted.replace(" ", "\\ "))
+    def _change_option(self, option, filename = ""):
+        "handle option changing, and handle filenames appropriately"
+        if filename is None:
+            option = "%s none" % option
+        elif filename:
+            if os.path.isfile(filename):
+                option = "%s %s" % (option, _path_quote(filename))
+            else:
+                print("WARN: skipping '%s' option with non-existing filename '%s'" % (option, filename))
+                return
         if self._lock_updates:
             self._options.append(option)
         else:
             self._hatari.change_option(option)
 
     def lock_updates(self):
-        "lock_updates(), collect Hatari configuration changes"
+        "lock_updates(), collect Hatari configuration changes to trigger only single reboot"
         self._lock_updates = True
-    
+
     def flush_updates(self):
         "flush_updates(), apply collected Hatari configuration changes"
         self._lock_updates = False
@@ -343,7 +380,7 @@ class HatariConfigMapping(ConfigStore):
         for key, item in list(self._paths.items()):
             paths.append((key, self.get(item[0], item[1]), item[2]))
         return paths
-    
+
     def set_paths(self, paths):
         for key, path in paths:
             self.set(self._paths[key][0], self._paths[key][1], path)
@@ -373,53 +410,106 @@ class HatariConfigMapping(ConfigStore):
         self.set("[RS232]", "bEnableRS232", value)
         self._hatari.set_device("rs232", value)
 
+    def get_scca(self):
+        if not self.has_opts_2_5:
+            return False
+        return self.get("[RS232]", "EnableSccA")
+
+    def set_scca(self, value):
+        if not self.has_opts_2_5:
+            return
+        self.set("[RS232]", "EnableSccA", value)
+        self._hatari.set_device("scca", value)
+
+    def get_scca_lan(self):
+        if not self.has_opts_2_5:
+            return False
+        return self.get("[RS232]", "EnableSccALan")
+
+    def set_scca_lan(self, value):
+        if not self.has_opts_2_5:
+            return
+        self.set("[RS232]", "EnableSccALan", value)
+        self._hatari.set_device("sccalan", value)
+
+    def get_sccb(self):
+        if not self.has_opts_2_5:
+            return self.get("[RS232]", "bEnableSccB")
+        return self.get("[RS232]", "EnableSccB")
+
+    def set_sccb(self, value):
+        if not self.has_opts_2_5:
+            self.set("[RS232]", "bEnableSccB", value)
+        else:
+            self.set("[RS232]", "EnableSccB", value)
+        self._hatari.set_device("sccb", value)
+
     # ------------ machine ---------------
     def get_machine_types(self):
-        if self.has_modeltype:
-            return ("ST", "MegaST", "STE", "MegaSTE", "TT", "Falcon")
-        else:
-            return ("ST", "STE", "TT", "Falcon")
+        return ("ST", "MegaST", "STE", "MegaSTE", "TT", "Falcon")
 
     def get_machine(self):
-        try:
-            return self.get("[System]", "nModelType")
-        except KeyError:
-            self.has_modeltype = False
-            return self.get("[System]", "nMachineType")
+        return self.get("[System]", "nModelType")
 
     def has_accurate_winsize(self):
-        if self.has_modeltype:
-            return (self.get_machine() < 4)
-        else:
-            return (self.get_machine() < 2)
+        return (self.get_machine() < 4)
 
     def set_machine(self, value):
-        if self.has_modeltype:
-            self.set("[System]", "nModelType", value)
-            self._change_option("--machine %s" % ("st", "megast", "ste", "megaste", "tt", "falcon")[value])
-        else:
-            self.set("[System]", "nMachineType", value)
-            self._change_option("--machine %s" % ("st", "ste", "tt", "falcon")[value])
+        self.set("[System]", "nModelType", value)
+        self._change_option("--machine %s" % ("st", "megast", "ste", "megaste", "tt", "falcon")[value])
 
     # ------------ CPU level ---------------
     def get_cpulevel_types(self):
-        if self._winuae:
-            return ("68000", "68010", "68020", "68E030", "68040", "68060")
-        else:
-            return ("68000", "68010", "68020", "68EC030+FPU", "68040")
+        return ("68000", "68010", "68020", "68030", "68040", "68060")
 
     def get_cpulevel(self):
         return self.get("[System]", "nCpuLevel")
 
     def set_cpulevel(self, value):
-        if value == 5: # WinUAE 060
+        if value == 5: # 060
             value = 6
         self.set("[System]", "nCpuLevel", value)
         self._change_option("--cpulevel %d" % value)
 
+    # ------------ compatible ---------------
+    def get_compatible(self):
+        return self.get("[System]", "bCompatibleCpu")
+
+    def set_compatible(self, value):
+        self.set("[System]", "bCompatibleCpu", value)
+        self._change_option("--compatible %s" % value)
+
+    # ------------ CPU caches ---------------
+    def get_data_cache(self):
+        if not self.has_opts_2_6:
+            return True
+        return self.get("[System]", "bCpuDataCache")
+
+    def set_data_cache(self, value):
+        if not self.has_opts_2_6:
+            return
+        self.set("[System]", "bCpuDataCache", value)
+        self._change_option("--data-cache %s" % value)
+
+    # ------------ CPU exact ---------------
+    def get_cycle_exact(self):
+        return self.get("[System]", "bCycleExactCpu")
+
+    def set_cycle_exact(self, value):
+        self.set("[System]", "bCycleExactCpu", value)
+        self._change_option("--cpu-exact %s" % value)
+
+    # ------------ MMU ---------------
+    def get_mmu(self):
+        return self.get("[System]", "bMMU")
+
+    def set_mmu(self, value):
+        self.set("[System]", "bMMU", value)
+        self._change_option("--mmu %s" % value)
+
     # ------------ CPU clock ---------------
     def get_cpuclock_types(self):
-        return ("8 MHz", "16 MHz", "32 MHz") 
+        return ("8 MHz", "16 MHz", "32 MHz")
 
     def get_cpuclock(self):
         clocks = {8:0, 16: 1, 32:2}
@@ -435,6 +525,33 @@ class HatariConfigMapping(ConfigStore):
         self.set("[System]", "nCpuFreq", value)
         self._change_option("--cpuclock %d" % value)
 
+    # ------------ FPU type ---------------
+    def get_fpu_types(self):
+        return ("None", "68881", "68882", "Internal")
+
+    def get_fpu_type(self):
+        return self.get("[System]", "n_FPUType")
+
+    def set_fpu_type(self, value):
+        self.set("[System]", "n_FPUType", value)
+        self._change_option("--fpu %s" % self.get_fpu_types()[value])
+
+    # ------------ SW FPU --------------
+    def get_fpu_soft(self):
+        return self.get("[System]", "bSoftFloatFPU")
+
+    def set_fpu_soft(self, value):
+        self.set("[System]", "bSoftFloatFPU", value)
+        self._change_option("--fpu-softfloat %s" % value)
+
+    # ------------ ST blitter --------------
+    def get_blitter(self):
+        return self.get("[System]", "bBlitter")
+
+    def set_blitter(self, value):
+        self.set("[System]", "bBlitter", value)
+        self._change_option("--blitter %s" % value)
+
     # ------------ DSP type ---------------
     def get_dsp_types(self):
         return ("None", "Dummy", "Emulated")
@@ -446,21 +563,13 @@ class HatariConfigMapping(ConfigStore):
         self.set("[System]", "nDSPType", value)
         self._change_option("--dsp %s" % ("none", "dummy", "emu")[value])
 
-    # ------------ compatible ---------------
-    def get_compatible(self):
-        return self.get("[System]", "bCompatibleCpu")
-
-    def set_compatible(self, value):
-        self.set("[System]", "bCompatibleCpu", value)
-        self._change_option("--compatible %s" % str(value))
-
     # ------------ Timer-D ---------------
     def get_timerd(self):
         return self.get("[System]", "bPatchTimerD")
 
     def set_timerd(self, value):
         self.set("[System]", "bPatchTimerD", value)
-        self._change_option("--timer-d %s" % str(value))
+        self._change_option("--timer-d %s" % value)
 
     # ------------ fastforward ---------------
     def get_fastforward(self):
@@ -468,15 +577,15 @@ class HatariConfigMapping(ConfigStore):
 
     def set_fastforward(self, value):
         self.set("[System]", "bFastForward", value)
-        self._change_option("--fast-forward %s" % str(value))
-        
+        self._change_option("--fast-forward %s" % value)
+
     # ------------ sound ---------------
     def get_sound_values(self):
-        # 48kHz, 44.1kHz and STE/TT/Falcon DMA 50066Hz divisable values
+        # 48kHz, 44.1kHz and STE/TT/Falcon DMA 50066Hz divisible values
         return ("6000", "6258", "8000", "11025", "12000", "12517",
                 "16000", "22050", "24000", "25033", "32000",
                 "44100", "48000", "50066")
-    
+
     def get_sound(self):
         enabled = self.get("[Sound]", "bEnableSound")
         hz = str(self.get("[Sound]", "nPlaybackFreq"))
@@ -493,46 +602,47 @@ class HatariConfigMapping(ConfigStore):
             self._change_option("--sound %s" % hz)
         else:
             self._change_option("--sound off")
-    
+
     def get_ymmixer_types(self):
-        return ("linear", "table", "model")
-    
+        return ("Linear", "ST table", "Math model")
+
     def get_ymmixer(self):
         # values for types are start from 1, not 0
         return self.get("[Sound]", "YmVolumeMixing")-1
-    
+
     def set_ymmixer(self, value):
+        ymmixer_types = ("linear", "table", "model")
         self.set("[Sound]", "YmVolumeMixing", value+1)
-        self._change_option("--ym-mixing %s" % self.get_ymmixer_types()[value])
-    
+        self._change_option("--ym-mixing %s" % ymmixer_types[value])
+
     def get_bufsize(self):
         return self.get("[Sound]", "nSdlAudioBufferSize")
-    
+
     def set_bufsize(self, value):
         value = int(value)
-        if value < 10: value = 10
-        if value > 100: value = 100
+        if value < 10 or value > 100:
+            value = 0
         self.set("[Sound]", "nSdlAudioBufferSize", value)
         self._change_option("--sound-buffer-size %d" % value)
-    
+
     def get_sync(self):
         return self.get("[Sound]", "bEnableSoundSync")
-    
+
     def set_sync(self, value):
         self.set("[Sound]", "bEnableSoundSync", value)
-        self._change_option("--sound-sync %s" % str(value))
+        self._change_option("--sound-sync %s" % value)
 
     def get_mic(self):
         return self.get("[Sound]", "bEnableMicrophone")
-    
+
     def set_mic(self, value):
         self.set("[Sound]", "bEnableMicrophone", value)
-        self._change_option("--mic %s" % str(value))
+        self._change_option("--mic %s" % value)
 
     # ----------- joystick --------------
     def get_joystick_types(self):
         return ("Disabled", "Real joystick", "Keyboard")
-    
+
     def get_joystick_names(self):
         return (
         "ST Joystick 0",
@@ -556,16 +666,16 @@ class HatariConfigMapping(ConfigStore):
     # ------------ floppy handling ---------------
     def get_floppydir(self):
         return self.get("[Floppy]", "szDiskImageDirectory")
-    
+
     def set_floppydir(self, path):
         return self.set("[Floppy]", "szDiskImageDirectory", path)
 
     def get_floppy(self, drive):
         return self.get("[Floppy]", "szDisk%cFileName" % ("A", "B")[drive])
-    
+
     def set_floppy(self, drive, filename):
         self.set("[Floppy]", "szDisk%cFileName" %  ("A", "B")[drive], filename)
-        self._change_option("--disk-%c" % ("a", "b")[drive], str(filename))
+        self._change_option("--disk-%c" % ("a", "b")[drive], filename)
 
     def get_floppy_drives(self):
         return (self.get("[Floppy]", "EnableDriveA"), self.get("[Floppy]", "EnableDriveB"))
@@ -575,7 +685,7 @@ class HatariConfigMapping(ConfigStore):
         for drive in ("A", "B"):
             value = drives[idx]
             self.set("[Floppy]", "EnableDrive%c" % drive, value)
-            self._change_option("--drive-%c %s" % (drive.lower(), str(value)))
+            self._change_option("--drive-%c %s" % (drive.lower(), value))
             idx += 1
 
     def get_fastfdc(self):
@@ -583,7 +693,7 @@ class HatariConfigMapping(ConfigStore):
 
     def set_fastfdc(self, value):
         self.set("[Floppy]", "FastFloppy", value)
-        self._change_option("--fastfdc %s" % str(value))
+        self._change_option("--fastfdc %s" % value)
 
     def get_doublesided(self):
         driveA = self.get("[Floppy]", "DriveA_NumberOfHeads")
@@ -591,7 +701,7 @@ class HatariConfigMapping(ConfigStore):
         if driveA > 1 or driveB > 1:
             return True
         return False
-    
+
     def set_doublesided(self, value):
         if value: sides = 2
         else:     sides = 1
@@ -648,51 +758,52 @@ class HatariConfigMapping(ConfigStore):
         return self.get("[HardDisk]", "szHardDiskDirectory")
 
     def set_hd_dir(self, dirname):
-        if dirname and os.path.isdir(dirname):
-            self.set("[HardDisk]", "bUseHardDiskDirectory", True)
         self.set("[HardDisk]", "szHardDiskDirectory", dirname)
-        self._change_option("--harddrive", str(dirname))
+        if dirname:
+            if os.path.isdir(dirname):
+                self.set("[HardDisk]", "bUseHardDiskDirectory", True)
+                self._change_option("--harddrive %s" % _path_quote(dirname))
+        else:
+            self._change_option("--harddrive none")
 
     # ------------ ACSI HD (file) ---------------
     def get_acsi_image(self):
-        self.get("[HardDisk]", "bUseHardDiskImage") # for validation
-        return self.get("[HardDisk]", "szHardDiskImage")
+        self.get("[ACSI]", "bUseDevice0")
+        return self.get("[ACSI]", "sDeviceFile0")
 
     def set_acsi_image(self, filename):
         if filename and os.path.isfile(filename):
-            self.set("[HardDisk]", "bUseHardDiskImage", True)
-        self.set("[HardDisk]", "szHardDiskImage", filename)
-        self._change_option("--acsi", str(filename))
+            self.set("[ACSI]", "bUseDevice0", True)
+        self.set("[ACSI]", "sDeviceFile0", filename)
+        self._change_option("--acsi", filename)
 
     # ------------ IDE master (file) ---------------
     def get_idemaster_image(self):
-        self.get("[HardDisk]", "bUseIdeMasterHardDiskImage") # for validation
-        return self.get("[HardDisk]", "szIdeMasterHardDiskImage")
+        return self.get("[IDE]", "sDeviceFile0")
 
     def set_idemaster_image(self, filename):
         if filename and os.path.isfile(filename):
-            self.set("[HardDisk]", "bUseIdeMasterHardDiskImage", True)
-        self.set("[HardDisk]", "szIdeMasterHardDiskImage", filename)
-        self._change_option("--ide-master", str(filename))
+            self.set("[IDE]", "bUseDevice0", True)
+        self.set("[IDE]", "sDeviceFile0", filename)
+        self._change_option("--ide-master", filename)
 
     # ------------ IDE slave (file) ---------------
     def get_ideslave_image(self):
-        self.get("[HardDisk]", "bUseIdeSlaveHardDiskImage") # for validation
-        return self.get("[HardDisk]", "szIdeSlaveHardDiskImage")
+        return self.get("[IDE]", "sDeviceFile1")
 
     def set_ideslave_image(self, filename):
         if filename and os.path.isfile(filename):
-            self.set("[HardDisk]", "bUseIdeSlaveHardDiskImage", True)
-        self.set("[HardDisk]", "szIdeSlaveHardDiskImage", filename)
-        self._change_option("--ide-slave", str(filename))
+            self.set("[IDE]", "bUseDevice1", True)
+        self.set("[IDE]", "sDeviceFile1", filename)
+        self._change_option("--ide-slave", filename)
 
     # ------------ TOS ROM ---------------
     def get_tos(self):
         return self.get("[ROM]", "szTosImageFileName")
-    
+
     def set_tos(self, filename):
         self.set("[ROM]", "szTosImageFileName", filename)
-        self._change_option("--tos", str(filename))
+        self._change_option("--tos", filename)
 
     # ------------ memory ---------------
     def get_memory_names(self):
@@ -703,9 +814,13 @@ class HatariConfigMapping(ConfigStore):
         "return index to what get_memory_names() returns"
         sizemap = (0, 1, 2, 2, 3, 3, 3, 3, 4, 4, 4, 4, 4, 4, 5)
         memsize = self.get("[Memory]", "nMemorySize")
-        if memsize >= 0 and memsize < len(sizemap):
-            return sizemap[memsize]
-        return 1 # default = 1BM
+        if memsize >= 1024 and memsize <= 14*1024:
+            memsize //= 1024
+        elif memsize >= 512:
+            memsize = 0
+        elif memsize < 0 or memsize >= len(sizemap):
+            memsize = 1
+        return sizemap[memsize]
 
     def set_memory(self, idx):
         # map memory item index to memory size
@@ -714,25 +829,29 @@ class HatariConfigMapping(ConfigStore):
             memsize = sizemap[idx]
         else:
             memsize = 1
+        if memsize:
+            memsize *= 1024
+        else:
+            memsize = 512
         self.set("[Memory]", "nMemorySize", memsize)
         self._change_option("--memsize %d" % memsize)
 
     def get_ttram(self):
         return self.get("[Memory]", "nTTRamSize")
 
-    def set_ttram(self, memsize):
-        # guarantee correct type (Gtk float -> config int)
-        memsize = int(memsize)
+    def set_ttram(self, memsize, machine):
+        # enforce 4MB granularity used also by Hatari
+        memsize = (int(memsize)+3) & ~3
         self.set("[Memory]", "nTTRamSize", memsize)
         self._change_option("--ttram %d" % memsize)
-        if memsize:
+        if memsize and machine in ("TT", "Falcon"):
             # TT-RAM need 32-bit addressing (i.e. disable 24-bit)
             self.set("[System]", "bAddressSpace24", False)
             self._change_option("--addr24 off")
         else:
             # switch 24-bit addressing back for compatibility
             self.set("[System]", "bAddressSpace24", True)
-            self._change_option("--addr24 on", False)
+            self._change_option("--addr24 on")
 
     # ------------ monitor ---------------
     def get_monitor_types(self):
@@ -755,13 +874,13 @@ class HatariConfigMapping(ConfigStore):
             "4 frames",
             "Automatic"
         )
-    
+
     def get_frameskip(self):
         fs = self.get("[Screen]", "nFrameSkips")
         if fs < 0 or fs > 5:
             return 5
         return fs
-    
+
     def set_frameskip(self, value):
         value = int(value) # guarantee correct type
         self.set("[Screen]", "nFrameSkips", value)
@@ -770,7 +889,7 @@ class HatariConfigMapping(ConfigStore):
     # ------------ VBL slowdown ---------------
     def get_slowdown_names(self):
         return ("Disabled", "2x", "3x", "4x", "5x", "6x", "8x")
-    
+
     def set_slowdown(self, value):
         value = 1 + int(value)
         self._change_option("--slowdown %d" % value)
@@ -790,74 +909,61 @@ class HatariConfigMapping(ConfigStore):
 
     def set_desktop(self, value):
         self.set("[Screen]", "bKeepResolution", value)
-        self._change_option("--desktop %s" % str(value))
-
-    # --------- keep desktop res - st ------
-    def get_desktop_st(self):
-        try:
-            return self.get("[Screen]", "bKeepResolutionST")
-        except KeyError:
-            self.has_keepstres = False
-            return False
-
-    def set_desktop_st(self, value):
-        if self.has_keepstres:
-            self.set("[Screen]", "bKeepResolutionST", value)
-            self._change_option("--desktop-st %s" % str(value))
+        self._change_option("--desktop %s" % value)
 
     # ------------ force max ---------------
     def get_force_max(self):
         return self.get("[Screen]", "bForceMax")
-    
+
     def set_force_max(self, value):
         self.set("[Screen]", "bForceMax", value)
-        self._change_option("--force-max %s" % str(value))
+        self._change_option("--force-max %s" % value)
 
     # ------------ show borders ---------------
     def get_borders(self):
         return self.get("[Screen]", "bAllowOverscan")
-    
+
     def set_borders(self, value):
         self.set("[Screen]", "bAllowOverscan", value)
-        self._change_option("--borders %s" % str(value))
+        self._change_option("--borders %s" % value)
 
     # ------------ show statusbar ---------------
     def get_statusbar(self):
         return self.get("[Screen]", "bShowStatusbar")
-    
+
     def set_statusbar(self, value):
         self.set("[Screen]", "bShowStatusbar", value)
-        self._change_option("--statusbar %s" % str(value))
+        self._change_option("--statusbar %s" % value)
 
     # ------------ crop statusbar ---------------
     def get_crop(self):
         return self.get("[Screen]", "bCrop")
-    
+
     def set_crop(self, value):
         self.set("[Screen]", "bCrop", value)
-        self._change_option("--crop %s" % str(value))
+        self._change_option("--crop %s" % value)
 
     # ------------ show led ---------------
     def get_led(self):
         return self.get("[Screen]", "bShowDriveLed")
-    
+
     def set_led(self, value):
         self.set("[Screen]", "bShowDriveLed", value)
-        self._change_option("--drive-led %s" % str(value))
+        self._change_option("--drive-led %s" % value)
 
     # ------------ monitor aspect ratio ---------------
     def get_aspectcorrection(self):
         return self.get("[Screen]", "bAspectCorrect")
-    
+
     def set_aspectcorrection(self, value):
         self.set("[Screen]", "bAspectCorrect", value)
-        self._change_option("--aspect %s" % str(value))
+        self._change_option("--aspect %s" % value)
 
     # ------------ max window size ---------------
     def set_desktop_size(self, w, h):
         self._desktop_w = w
         self._desktop_h = h
-        
+
     def get_desktop_size(self):
         return (self._desktop_w, self._desktop_h)
 
@@ -897,7 +1003,7 @@ class HatariConfigMapping(ConfigStore):
             width = self.get("[Screen]", "nVdiWidth")
             height = self.get("[Screen]", "nVdiHeight")
             return (width, height)
-        
+
         # window sizes for other than ST & STE can differ
         if self.has_accurate_winsize():
             videl = False
@@ -929,9 +1035,9 @@ class HatariConfigMapping(ConfigStore):
         # overscan borders?
         if self.get_borders() and not videl:
             # properly aligned borders on top of zooming
-            leftx = (maxw-width)/zoom
-            borderx = 2*(min(48,leftx/2)/16)*16
-            lefty = (maxh-height)/zoom
+            leftx = (maxw-width)//zoom
+            borderx = 2*(min(48,leftx//2)//16)*16
+            lefty = (maxh-height)//zoom
             bordery = min(29+47, lefty)
             width += zoom*borderx
             height += zoom*bordery
