@@ -36,6 +36,7 @@
 #include "newcpu.h"
 #include "cpummu.h"
 #include "debug.h"
+#include "log.h"
 
 #define MMUDUMP 1
 
@@ -142,13 +143,14 @@ void mmu_tt_modified (void)
 /* This dump output makes much more sense than old one */
 
 #ifdef WINUAE_FOR_HATARI
-#define	ULONG	Uint32
+#define	ULONG	uae_u32
 #endif
 
 #define LEVELA_SIZE 7
 #define LEVELB_SIZE 7
-#define LEVELC_SIZE 6
-#define PAGE_SIZE 12 // = 1 << 12 = 4096
+#define LEVELC_SIZE (mmu_pagesize_8k ? 5 : 6)
+#define PAGE_SIZE_4k 12 // = 1 << 12 = 4096
+#define PAGE_SIZE_8k 13 // = 1 << 13 = 8192
 
 #define LEVELA_VAL(x) ((((uae_u32)(x)) >> (32 - (LEVELA_SIZE                            ))) & ((1 << LEVELA_SIZE) - 1))
 #define LEVELB_VAL(x) ((((uae_u32)(x)) >> (32 - (LEVELA_SIZE + LEVELB_SIZE              ))) & ((1 << LEVELB_SIZE) - 1))
@@ -179,25 +181,27 @@ static void mmu_dump_table(const char * label, uaecptr root_ptr)
 	ULONG startaddr;
 	ULONG odesc;
 	ULONG totalpages;
-	ULONG pagemask = (1 << PAGE_SIZE) - 1;
+	ULONG page_size = mmu_pagesize_8k ? PAGE_SIZE_8k : PAGE_SIZE_4k;
+	ULONG pagemask = (1 << page_size) - 1;
+	ULONG descmask = pagemask & ~(0x08 | 0x10); // mask out unused and M bits
 
 	root_ptr &= 0xfffffe00;
-	console_out_f(_T("MMU dump start. Root = %08x\n"), root_ptr);
-	totalpages = 1 << (32 - PAGE_SIZE);
+	console_out_f(_T("MMU dump start. Root = %08x. Page = %d\n"), root_ptr, 1 << page_size);
+	totalpages = 1 << (32 - page_size);
 	startaddr = 0;
 	odesc = getdesc(root_ptr, startaddr);
 	for (i = 0; i <= totalpages; i++) {
-		ULONG addr = i << PAGE_SIZE;
+		ULONG addr = i << page_size;
 		ULONG desc = 0;
 		if (i < totalpages)
 			desc = getdesc(root_ptr, addr);
-		if ((desc & pagemask) != (odesc & pagemask) || i == totalpages) {
+		if ((desc & descmask) != (odesc & descmask) || i == totalpages) {
 			uae_u8 cm, sp;
 			cm = (odesc >> 5) & 3;
 			sp = (odesc >> 7) & 1;
-			console_out_f(_T("%08x - %08x: %08x WP=%d S=%d CM=%d (%08x)\n"),
-				startaddr, addr - 1, odesc & ~((1 << PAGE_SIZE) - 1),
-				(odesc & 4) ? 1 : 0, sp, cm, odesc);
+			console_out_f(_T("%08x - %08x: %08x WP=%d S=%d CM=%d V=%d (%08x)\n"),
+				startaddr, addr - 1, odesc & ~((1 << page_size) - 1),
+				(odesc & 4) ? 1 : 0, sp, cm, (odesc & 3) > 0, odesc);
 			startaddr = addr;
 			odesc = desc;
 		}
@@ -360,8 +364,7 @@ static void flush_shortcut_cache(uaecptr addr, bool super)
 		memset(&atc_data_cache_read, 0xff, sizeof atc_data_cache_read);
 		memset(&atc_data_cache_write, 0xff, sizeof atc_data_cache_write);
 	} else {
-		int i;
-		for (i = 0; i < MMUFASTCACHE_ENTRIES; i++) {
+		for (int i = 0; i < MMUFASTCACHE_ENTRIES; i++) {
 			uae_u32 idx = ((addr & mmu_pagemaski) >> mmu_pageshift1m) | (super ? 1 : 0);
 			if (atc_data_cache_read[i].log == idx)
 				atc_data_cache_read[i].log = 0xffffffff;
@@ -377,6 +380,28 @@ static ALWAYS_INLINE int mmu_get_fc(bool super, bool data)
 	return (super ? 4 : 0) | (data ? 1 : 2);
 }
 
+void mmu_hardware_bus_error(uaecptr addr, uae_u32 v, bool read, bool ins, int size)
+{
+	uae_u32 fc;
+	
+	if (ismoves) {
+		fc = read ? regs.sfc : regs.dfc;
+	} else {
+		fc = (regs.s ? 4 : 0) | (ins ? 2 : 1);
+	}
+	mmu_bus_error(addr, v, fc, !read, size, 0, true);
+}
+
+bool mmu_is_super_access(bool read)
+{
+	if (!ismoves) {
+		return regs.s;
+	} else {
+		uae_u32 fc = read ? regs.sfc : regs.dfc;
+		return (fc & 4) != 0;
+	}
+}
+
 void mmu_bus_error(uaecptr addr, uae_u32 val, int fc, bool write, int size,uae_u32 status060, bool nonmmu)
 {
 	if (currprefs.mmu_model == 68040) {
@@ -384,15 +409,15 @@ void mmu_bus_error(uaecptr addr, uae_u32 val, int fc, bool write, int size,uae_u
 
 		if (ismoves) {
 			// MOVES special behavior
-			int fc2 = write ? regs.dfc : regs.sfc;
-			if (fc2 == 0 || fc2 == 3 || fc2 == 4 || fc2 == 7)
+			int old_fc = fc = write ? regs.dfc : regs.sfc;
+			if ((fc & 3) == 0 || (fc & 3) == 3) {
 				ssw |= MMU_SSW_TT1;
-			if ((fc2 & 3) != 3)
-				fc2 &= ~2;
+			} else if (fc & 2) {
+				fc = (fc & 4) | 1;
+			}
 #if MMUDEBUGMISC > 0
-			write_log (_T("040 MMU MOVES fc=%d -> %d\n"), fc, fc2);
+			write_log (_T("040 MMU MOVES fc=%d -> %d\n"), old_fc, fc);
 #endif
-			fc = fc2;
 		}
 
 		ssw |= fc & MMU_SSW_TM;				/* TM = FC */
@@ -502,6 +527,7 @@ void mmu_bus_error(uaecptr addr, uae_u32 val, int fc, bool write, int size,uae_u
 
 	}
 
+	ismoves = false;
 	rmw_cycle = false;
 	locked_rmw_cycle = false;
 	regs.mmu_fault_addr = addr;
@@ -636,9 +662,14 @@ static void desc_put_long(uaecptr addr, uae_u32 v)
 static uae_u32 mmu_fill_atc(uaecptr addr, bool super, uae_u32 tag, bool write, struct mmu_atc_line *l, uae_u32 *status060)
 {
     uae_u32 desc, desc_addr, wp;
-	uae_u32 status = 0;
+    uae_u32 status = 0;
     int i;
+	int old_s;
     
+    // Use supervisor mode to access descriptors (really is fc = 7)
+    old_s = regs.s;
+    regs.s = 1;
+
     wp = 0;
     desc = super ? regs.srp : regs.urp;
     
@@ -764,8 +795,11 @@ fail:
 #if MMUDEBUG > 0
         write_log(_T("MMU: bus error during table search.\n"));
 #endif
-    } ENDTRY
-    
+    } ENDTRY;
+
+    // Restore original supervisor state
+    regs.s = old_s;
+
 #if MMUDEBUG > 2
     write_log(_T("translate: %x,%u,%u -> %x\n"), addr, super, write, desc);
 #endif
@@ -1057,7 +1091,7 @@ static void REGPARAM2 mmu_put_lrmw_word_unaligned(uaecptr addr, uae_u16 val)
 	SAVE_EXCEPTION;
 	TRY(prb) {
 		mmu_put_user_byte(addr, val >> 8, regs.s != 0, sz_word, true);
-		mmu_put_user_byte(addr + 1, val, regs.s != 0, sz_word, true);
+		mmu_put_user_byte(addr + 1, (uae_u8)val, regs.s != 0, sz_word, true);
 		RESTORE_EXCEPTION;
 	}
 	CATCH(prb) {
@@ -1098,7 +1132,7 @@ void REGPARAM2 mmu_put_word_unaligned(uaecptr addr, uae_u16 val, bool data)
 	SAVE_EXCEPTION;
 	TRY(prb) {
 		mmu_put_byte(addr, val >> 8, data, sz_word);
-		mmu_put_byte(addr + 1, val, data, sz_word);
+		mmu_put_byte(addr + 1, (uae_u8)val, data, sz_word);
 		RESTORE_EXCEPTION;
 	}
 	CATCH(prb) {
@@ -1115,7 +1149,7 @@ uae_u32 REGPARAM2 sfc_get_long(uaecptr addr)
 	uae_u32 res;
 
 	ismoves = true;
-	if (likely(!is_unaligned(addr, 4))) {
+	if (likely(!is_unaligned_page(addr, 4))) {
 		res = mmu_get_user_long(addr, super, false, sz_long, false);
 	} else {
 		if (likely(!(addr & 1))) {
@@ -1157,7 +1191,7 @@ uae_u16 REGPARAM2 sfc_get_word(uaecptr addr)
 	uae_u16 res;
 
 	ismoves = true;
-	if (likely(!is_unaligned(addr, 2))) {
+	if (likely(!is_unaligned_page(addr, 2))) {
 		res = mmu_get_user_word(addr, super, false, sz_word, false);
 	} else {
 		res = (uae_u16)mmu_get_user_byte(addr, super, false, sz_word, false) << 8;
@@ -1194,7 +1228,7 @@ void REGPARAM2 dfc_put_long(uaecptr addr, uae_u32 val)
 	ismoves = true;
 	SAVE_EXCEPTION;
 	TRY(prb) {
-		if (likely(!is_unaligned(addr, 4))) {
+		if (likely(!is_unaligned_page(addr, 4))) {
 			mmu_put_user_long(addr, val, super, sz_long, false);
 		} else if (likely(!(addr & 1))) {
 			mmu_put_user_word(addr, val >> 16, super, sz_long, false);
@@ -1223,11 +1257,11 @@ void REGPARAM2 dfc_put_word(uaecptr addr, uae_u16 val)
 	ismoves = true;
 	SAVE_EXCEPTION;
 	TRY(prb) {
-		if (likely(!is_unaligned(addr, 2))) {
+		if (likely(!is_unaligned_page(addr, 2))) {
 			mmu_put_user_word(addr, val, super, sz_word, false);
 		} else {
 			mmu_put_user_byte(addr, val >> 8, super, sz_word, false);
-			mmu_put_user_byte(addr + 1, val, super, sz_word, false);
+			mmu_put_user_byte(addr + 1, (uae_u8)val, super, sz_word, false);
 		}
 		RESTORE_EXCEPTION;
 	}
@@ -1261,7 +1295,6 @@ void REGPARAM2 dfc_put_byte(uaecptr addr, uae_u8 val)
 void mmu_get_move16(uaecptr addr, uae_u32 *v, bool data, int size)
 {
 	bool super = regs.s != 0;
-	int i;
 
 	addr &= ~15;
 	if ((!mmu_ttr_enabled || mmu_match_ttr(addr,super,data) == TTR_NO_MATCH) && regs.mmu_enabled) {
@@ -1269,7 +1302,7 @@ void mmu_get_move16(uaecptr addr, uae_u32 *v, bool data, int size)
 	}
 	// MOVE16 read and cache miss: do not allocate new cache line
 	mmu_cache_state |= CACHE_DISABLE_ALLOCATE;
-	for (i = 0; i < 4; i++) {
+	for (int i = 0; i < 4; i++) {
 		v[i] = x_phys_get_long(addr + i * 4);
 	}
 }
@@ -1277,7 +1310,6 @@ void mmu_get_move16(uaecptr addr, uae_u32 *v, bool data, int size)
 void mmu_put_move16(uaecptr addr, uae_u32 *val, bool data, int size)
 {
 	bool super = regs.s != 0;
-	int i;
 
 	addr &= ~15;
 	if ((!mmu_ttr_enabled || mmu_match_ttr_write(addr,super,data,val[0],size) == TTR_NO_MATCH) && regs.mmu_enabled) {
@@ -1285,7 +1317,7 @@ void mmu_put_move16(uaecptr addr, uae_u32 *val, bool data, int size)
 	}
 	// MOVE16 write invalidates existing line and also does not allocate new cache lines.
 	mmu_cache_state = CACHE_DISABLE_MMU;
-	for (i = 0; i < 4; i++) {
+	for (int i = 0; i < 4; i++) {
 		x_phys_put_long(addr + i * 4, val[i]);
 	}
 }
@@ -1434,6 +1466,15 @@ void REGPARAM2 mmu_set_funcs(void)
 {
 	if (currprefs.mmu_model != 68040 && currprefs.mmu_model != 68060)
 		return;
+
+	x_phys_get_iword = phys_get_word;
+	x_phys_get_ilong = phys_get_long;
+	x_phys_get_byte = phys_get_byte;
+	x_phys_get_word = phys_get_word;
+	x_phys_get_long = phys_get_long;
+	x_phys_put_byte = phys_put_byte;
+	x_phys_put_word = phys_put_word;
+	x_phys_put_long = phys_put_long;
 	if (currprefs.cpu_memory_cycle_exact || currprefs.cpu_compatible) {
 		x_phys_get_iword = get_word_icache040;
 		x_phys_get_ilong = get_long_icache040;
@@ -1451,23 +1492,7 @@ void REGPARAM2 mmu_set_funcs(void)
 			x_phys_put_byte = mem_access_delay_byte_write_c040;
 			x_phys_put_word = mem_access_delay_word_write_c040;
 			x_phys_put_long = mem_access_delay_long_write_c040;
-		} else {
-			x_phys_get_byte = phys_get_byte;
-			x_phys_get_word = phys_get_word;
-			x_phys_get_long = phys_get_long;
-			x_phys_put_byte = phys_put_byte;
-			x_phys_put_word = phys_put_word;
-			x_phys_put_long = phys_put_long;
 		}
-	} else {
-		x_phys_get_iword = phys_get_word;
-		x_phys_get_ilong = phys_get_long;
-		x_phys_get_byte = phys_get_byte;
-		x_phys_get_word = phys_get_word;
-		x_phys_get_long = phys_get_long;
-		x_phys_put_byte = phys_put_byte;
-		x_phys_put_word = phys_put_word;
-		x_phys_put_long = phys_put_long;
 	}
 }
 
@@ -1480,7 +1505,7 @@ void REGPARAM2 mmu_reset(void)
 uae_u16 REGPARAM2 mmu_set_tc(uae_u16 tc)
 {
 	if (currprefs.mmu_ec) {
- 		tc &= ~(0x8000 | 0x4000);
+		tc &= ~(0x8000 | 0x4000);
 		// at least 68EC040 always returns zero when TC is read.
 		if (currprefs.cpu_model == 68040)
 			tc = 0;
@@ -1511,7 +1536,7 @@ uae_u16 REGPARAM2 mmu_set_tc(uae_u16 tc)
 
 	mmu_flush_atc_all(true);
 
-	write_log(_T("%d MMU: TC=%04x enabled=%d page8k=%d PC=%08x\n"), tc, currprefs.mmu_model, regs.mmu_enabled, mmu_pagesize_8k, m68k_getpc());
+	write_log(_T("%d MMU: TC=%04x enabled=%d page8k=%d PC=%08x\n"), currprefs.mmu_model, tc, regs.mmu_enabled, mmu_pagesize_8k, m68k_getpc());
 	return tc;
 }
 
@@ -1522,6 +1547,8 @@ void REGPARAM2 mmu_set_super(bool super)
 
 void REGPARAM2 mmu_flush_cache(void)
 {
+	if (!currprefs.mmu_model)
+		return;
 #if MMU_ICACHE
 	int len = sizeof(mmu_icache_data);
 	memset(&mmu_icache_data, 0xff, sizeof(mmu_icache_data));
@@ -1546,6 +1573,9 @@ void m68k_do_rte_mmu040 (uaecptr a7)
 		write_log (_T("MMU restarted MOVEM EA=%08X\n"), mmu040_movem_ea);
 #endif
 	}
+	if (mmu_restart) {
+		set_special(SPCFLAG_MMURESTART);
+	}
 }
 
 void m68k_do_rte_mmu060 (uaecptr a7)
@@ -1553,6 +1583,7 @@ void m68k_do_rte_mmu060 (uaecptr a7)
 #if 0
 	mmu060_state = 2;
 #endif
+	set_special(SPCFLAG_MMURESTART);
 }
 
 void flush_mmu040 (uaecptr addr, int n)
@@ -1601,13 +1632,13 @@ void uae_mmu_put_lrmw (uaecptr addr, uae_u32 v, int size, int type)
 	if (size == sz_byte) {
 		mmu_put_user_byte(addr, v, regs.s, sz_byte, true);
 	} else if (size == sz_word) {
-		if (unlikely(is_unaligned(addr, 2))) {
+		if (unlikely(is_unaligned_page(addr, 2))) {
 			mmu_put_lrmw_word_unaligned(addr, v);
 		} else {
 			mmu_put_user_word(addr, v, regs.s != 0, sz_word, true);
 		}
 	} else {
-		if (unlikely(is_unaligned(addr, 4)))
+		if (unlikely(is_unaligned_page(addr, 4)))
 			mmu_put_lrmw_long_unaligned(addr, v);
 		else
 			mmu_put_user_long(addr, v, regs.s, sz_long, true);
@@ -1621,13 +1652,13 @@ uae_u32 uae_mmu_get_lrmw (uaecptr addr, int size, int type)
 	if (size == sz_byte) {
 		v = mmu_get_user_byte(addr, regs.s != 0, true, sz_byte, true);
 	} else if (size == sz_word) {
-		if (unlikely(is_unaligned(addr, 2))) {
+		if (unlikely(is_unaligned_page(addr, 2))) {
 			v = mmu_get_lrmw_word_unaligned(addr);
 		} else {
 			v = mmu_get_user_word(addr, regs.s != 0, true, sz_word, true);
 		}
 	} else {
-		if (unlikely(is_unaligned(addr, 4)))
+		if (unlikely(is_unaligned_page(addr, 4)))
 			v = mmu_get_lrmw_long_unaligned(addr);
 		else
 			v = mmu_get_user_long(addr, regs.s != 0, true, sz_long, true);
