@@ -7,6 +7,11 @@
 * Andreas Grabher and Toni Wilen
 *
 */
+
+#ifdef WIN32 /* WINUAE_FOR_HATARI */
+#define __USE_MINGW_ANSI_STDIO 1        /* Hack for %lld format specifiers */
+#endif
+
 #define __USE_ISOC9X  /* We might be able to pick up a NaN */
 
 #define SOFTFLOAT_FAST_INT64
@@ -15,13 +20,15 @@
 #include <float.h>
 #include <fenv.h>
 
-#include "main.h"
-#include "hatari-glue.h"
-
 #include "sysconfig.h"
 #include "sysdeps.h"
 
-#include "options.h"
+#ifdef WINUAE_FOR_HATARI
+#include "main.h"
+#include "hatari-glue.h"
+#endif
+
+#include "options_cpu.h"
 #include "memory.h"
 #include "newcpu.h"
 #include "fpp.h"
@@ -94,11 +101,16 @@ static void fp_get_status(uae_u32 *status)
 	if (fs.float_exception_flags & float_flag_decimal)
 		*status |= FPSR_INEX1;
 }
+
 STATIC_INLINE void fp_clear_status(void)
 {
 	fs.float_exception_flags = 0;
 }
 
+static uae_u32 fp_get_support_flags(void)
+{
+	return FPU_FEATURE_EXCEPTIONS | FPU_FEATURE_DENORMALS;
+}
 
 static const TCHAR *fp_printx80(floatx80 *fx, int mode)
 {
@@ -125,7 +137,7 @@ static const TCHAR *fp_printx80(floatx80 *fx, int mode)
 		int8_t save_exception_flags = fs.float_exception_flags;
 		fs.float_exception_flags = 0;
 		floatx80 x = floatx80_to_floatdecimal(*fx, &len, &fs);
-		_stprintf(fsout, _T("%c%01lld.%016llde%c%04d%s%s"), n ? '-' : '+',
+		_stprintf(fsout, _T("%c%01lld.%016llde%c%05u%s%s"), n ? '-' : '+',
 				x.low / LIT64(10000000000000000), x.low % LIT64(10000000000000000),
 				(x.high & 0x4000) ? '-' : '+', x.high & 0x3FFF, d ? _T("D") : u ? _T("U") : _T(""),
 				(fs.float_exception_flags & float_flag_inexact) ? _T("~") : _T(""));
@@ -144,6 +156,10 @@ static const TCHAR *fp_print(fpdata *fpd, int mode)
 }
 
 /* Functions for detecting float type */
+static bool fp_is_init(fpdata *fpd)
+{
+	return false;
+}
 static bool fp_is_snan(fpdata *fpd)
 {
 	return floatx80_is_signaling_nan(fpd->fpx) != 0;
@@ -151,16 +167,20 @@ static bool fp_is_snan(fpdata *fpd)
 static bool fp_unset_snan(fpdata *fpd)
 {
 	fpd->fpx.low |= LIT64(0x4000000000000000);
-	return 0;
+	return false;
 }
-static bool fp_is_nan (fpdata *fpd)
+static bool fp_is_nan(fpdata *fpd)
 {
 	return floatx80_is_any_nan(fpd->fpx) != 0;
 }
-static bool fp_is_infinity (fpdata *fpd)
+static bool fp_is_infinity(fpdata *fpd)
 {
 	return floatx80_is_infinity(fpd->fpx) != 0;
 }
+static void fp_fix_infinity(fpdata *fpd)
+{
+	fpd->fpx.low = 0;
+}
 static bool fp_is_zero(fpdata *fpd)
 {
 	return floatx80_is_zero(fpd->fpx) != 0;
@@ -341,7 +361,11 @@ static void fp_getman(fpdata *a, fpdata *b)
 }
 static void fp_mod(fpdata *a, fpdata *b, uae_u64 *q, uae_u8 *s)
 {
+#ifndef WINUAE_FOR_HATARI
+	a->fpx = floatx80_mod(a->fpx, b->fpx, q, s, &fs);
+#else
 	a->fpx = floatx80_mod(a->fpx, b->fpx, (uint64_t *)q, s, &fs);
+#endif
 }
 static void fp_sgldiv(fpdata *a, fpdata *b)
 {
@@ -353,7 +377,11 @@ static void fp_sglmul(fpdata *a, fpdata *b)
 }
 static void fp_rem(fpdata *a, fpdata *b, uae_u64 *q, uae_u8 *s)
 {
+#ifndef WINUAE_FOR_HATARI
+	a->fpx = floatx80_rem(a->fpx, b->fpx, q, s, &fs);
+#else
 	a->fpx = floatx80_rem(a->fpx, b->fpx, (uint64_t *)q, s, &fs);
+#endif
 }
 static void fp_scale(fpdata *a, fpdata *b)
 {
@@ -368,15 +396,15 @@ static void fp_tst(fpdata *a, fpdata *b)
 	a->fpx = floatx80_tst(b->fpx, &fs);
 }
 
+static const uint8_t prectable[] = { 0, 32, 64, 80 };
+
 #define SETPREC \
 	uint8_t oldprec = fs.floatx80_rounding_precision; \
-	if (prec > 0) \
-		set_floatx80_rounding_precision(prec, &fs);
+	if (prec > PREC_NORMAL) \
+		set_floatx80_rounding_precision(prectable[prec], &fs);
 
 #define RESETPREC \
-	if (prec > 0) \
-		set_floatx80_rounding_precision(oldprec, &fs);
-
+	set_floatx80_rounding_precision(oldprec, &fs);
 
 /* Functions with fixed precision */
 static void fp_move(fpdata *a, fpdata *b, int prec)
@@ -501,6 +529,10 @@ static void fp_cos(fpdata *a, fpdata *b)
 {
     a->fpx = floatx80_cos(b->fpx, &fs);
 }
+static void fp_sincos(fpdata *a, fpdata *b, fpdata *c)
+{
+	a->fpx = floatx80_sincos(b->fpx, &c->fpx, &fs);
+}
 
 /* Functions for converting between float formats */
 static const fptype twoto32 = 4294967296.0;
@@ -512,12 +544,13 @@ static void to_native(fptype *fp, fpdata *fpd)
 	
 	expon = fpd->fpx.high & 0x7fff;
 	
+	fp_is_init(fpd);
 	if (fp_is_zero(fpd)) {
 		*fp = fp_is_neg(fpd) ? -0.0 : +0.0;
 		return;
 	}
 	if (fp_is_nan(fpd)) {
-#if USE_LONG_DOUBLE
+#ifdef USE_LONG_DOUBLE
 		*fp = sqrtl(-1);
 #else
 		*fp = sqrt(-1);
@@ -526,7 +559,7 @@ static void to_native(fptype *fp, fpdata *fpd)
 	}
 	if (fp_is_infinity(fpd)) {
 		double zero = 0.0;
-#if USE_LONG_DOUBLE
+#ifdef USE_LONG_DOUBLE
 		*fp = fp_is_neg(fpd) ? logl(0.0) : (1.0 / zero);
 #else
 		*fp = fp_is_neg(fpd) ? log(0.0) : (1.0 / zero);
@@ -537,7 +570,7 @@ static void to_native(fptype *fp, fpdata *fpd)
 	frac = (fptype)fpd->fpx.low / (fptype)(twoto32 * 2147483648.0);
 	if (fp_is_neg(fpd))
 		frac = -frac;
-#if USE_LONG_DOUBLE
+#ifdef USE_LONG_DOUBLE
 	*fp = ldexpl (frac, expon - 16383);
 #else
 	*fp = ldexp (frac, expon - 16383);
@@ -571,7 +604,7 @@ static void from_native(fptype fp, fpdata *fpd)
 	if (fp < 0.0)
 		fp = -fp;
 	
-#if USE_LONG_DOUBLE
+#ifdef USE_LONG_DOUBLE
 	 frac = frexpl (fp, &expon);
 #else
 	 frac = frexp (fp, &expon);
@@ -688,7 +721,7 @@ static void fp_from_pack(fpdata *fp, uae_u32 *wrd, int kfactor)
 			digit = significand % 10;
 			significand /= 10;
 			if (len == 0) {
-				pack_int = digit;
+				pack_int = (uae_u32)digit;
 			} else {
 				pack_frac |= digit << (64 - len * 4);
 			}
@@ -702,7 +735,7 @@ static void fp_from_pack(fpdata *fp, uae_u32 *wrd, int kfactor)
 			digit = exponent % 10;
 			exponent /= 10;
 			if (len == 0) {
-				pack_exp4 = digit;
+				pack_exp4 = (uae_u32)digit;
 			} else {
 				pack_exp |= digit << (12 - len * 4);
 			}
@@ -722,28 +755,33 @@ static void fp_from_pack(fpdata *fp, uae_u32 *wrd, int kfactor)
 	}
 }
 
-void fp_init_softfloat(void)
+void fp_init_softfloat(int fpu_model)
 {
-	float_status fsx = { 0 };
-	set_floatx80_rounding_precision(80, &fsx);
-	set_float_rounding_mode(float_round_to_zero, &fsx);
+	if (fpu_model == 68040) {
+		set_special_flags(cmp_signed_nan, &fs);
+	} else if (fpu_model == 68060) {
+		set_special_flags(infinity_clear_intbit, &fs);
+	} else {
+		set_special_flags(addsub_swap_inf, &fs);
+	}
 
 	fpp_print = fp_print;
-	fpp_is_snan = fp_is_snan;
 	fpp_unset_snan = fp_unset_snan;
+
+	fpp_is_init = fp_is_init;
+	fpp_is_snan = fp_is_snan;
 	fpp_is_nan = fp_is_nan;
 	fpp_is_infinity = fp_is_infinity;
 	fpp_is_zero = fp_is_zero;
 	fpp_is_neg = fp_is_neg;
 	fpp_is_denormal = fp_is_denormal;
 	fpp_is_unnormal = fp_is_unnormal;
+	fpp_fix_infinity = fp_fix_infinity;
 
 	fpp_get_status = fp_get_status;
 	fpp_clear_status = fp_clear_status;
 	fpp_set_mode = fp_set_mode;
-
-	fpp_from_native = from_native;
-	fpp_to_native = to_native;
+	fpp_get_support_flags = fp_get_support_flags;
 
 	fpp_to_int = to_int;
 	fpp_from_int = from_int;
@@ -798,6 +836,7 @@ void fp_init_softfloat(void)
 	fpp_neg = fp_neg;
 	fpp_acos = fp_acos;
 	fpp_cos = fp_cos;
+	fpp_sincos = fp_sincos;
 	fpp_getexp = fp_getexp;
 	fpp_getman = fp_getman;
 	fpp_div = fp_div;
