@@ -1,7 +1,7 @@
 /*
   Hatari - breakcond.c
 
-  Copyright (c) 2009-2016 by Eero Tamminen
+  Copyright (c) 2009-2024 by Eero Tamminen
 
   This file is distributed under the GNU General Public License, version 2
   or at your option any later version. Read the file gpl.txt for details.
@@ -11,7 +11,7 @@
   whether the breakpoint should be triggered.  See BreakCond_Help()
   for the syntax.
 */
-const char BreakCond_fileid[] = "Hatari breakcond.c : " __DATE__ " " __TIME__;
+const char BreakCond_fileid[] = "Hatari breakcond.c";
 
 #include <ctype.h>
 #include <stdlib.h>
@@ -41,7 +41,7 @@ const char BreakCond_fileid[] = "Hatari breakcond.c : " __DATE__ " " __TIME__;
 #define DEBUG 0
 
 /* needs to go through long long to handle x=32 */
-#define BITMASK(x)      ((Uint32)(((unsigned long long)1<<(x))-1))
+#define BITMASK(x)      ((uint32_t)(((unsigned long long)1<<(x))-1))
 
 #define BC_DEFAULT_DSP_SPACE 'P'
 
@@ -50,14 +50,14 @@ typedef struct {
 	char dsp_space;	/* DSP has P, X, Y address spaces, zero if not DSP */
 	value_t valuetype;	/* Hatari value variable type */
 	union {
-		Uint32 number;
-		Uint16 (*func16)(void);
-		Uint32 (*func32)(void);
-		Uint16 *reg16;
-		Uint32 *reg32;
+		uint32_t number;
+		uint16_t (*func16)(void);
+		uint32_t (*func32)(void);
+		uint16_t *reg16;
+		uint32_t *reg32;
 	} value;
-	Uint32 bits;	/* CPU has 8/16/32 bit address widths */
-	Uint32 mask;	/* <width mask> && <value mask> */
+	uint32_t bits;	/* CPU has 8/16/32 bit address widths */
+	uint32_t mask;	/* <width mask> && <value mask> */
 } bc_value_t;
 
 typedef struct {
@@ -68,10 +68,11 @@ typedef struct {
 } bc_condition_t;
 
 typedef struct {
+	info_func_t info;  /* pointer to specified ":info" function */
 	char *filename;	/* file where to read commands to do on hit */
 	int skip;	/* how many times to hit before breaking */
 	bool once;	/* remove after hit&break */
-	bool quiet;	/* no output from setting & hitting */
+	bool quiet;	/* set / hit breakpoint quietly */
 	bool trace;	/* trace mode, don't break */
 	bool noinit;	/* prevent debugger inits on break */
 	bool lock;	/* tracing + show locked info */
@@ -174,7 +175,7 @@ static void _spaces(void)
 /**
  * Return value from given DSP memory space/address
  */
-static Uint32 BreakCond_ReadDspMemory(Uint32 addr, const bc_value_t *bc_value)
+static uint32_t BreakCond_ReadDspMemory(uint32_t addr, const bc_value_t *bc_value)
 {
 	const char *dummy;
 	return DSP_ReadMemory(addr, bc_value->dsp_space, &dummy) & BITMASK(24);
@@ -183,7 +184,7 @@ static Uint32 BreakCond_ReadDspMemory(Uint32 addr, const bc_value_t *bc_value)
 /**
  * Return value of given size read from given ST memory address
  */
-static Uint32 BreakCond_ReadSTMemory(Uint32 addr, const bc_value_t *bc_value)
+static uint32_t BreakCond_ReadSTMemory(uint32_t addr, const bc_value_t *bc_value)
 {
 	switch (bc_value->bits) {
 	case 32:
@@ -200,11 +201,11 @@ static Uint32 BreakCond_ReadSTMemory(Uint32 addr, const bc_value_t *bc_value)
 
 
 /**
- * Return Uint32 value according to given bc_value_t specification
+ * Return uint32_t value according to given bc_value_t specification
  */
-static Uint32 BreakCond_GetValue(const bc_value_t *bc_value)
+static uint32_t BreakCond_GetValue(const bc_value_t *bc_value)
 {
-	Uint32 value;
+	uint32_t value;
 
 	switch (bc_value->valuetype) {
 	case VALUE_TYPE_NUMBER:
@@ -238,9 +239,9 @@ static Uint32 BreakCond_GetValue(const bc_value_t *bc_value)
 /**
  * Show & update rvalue for a tracked breakpoint condition to lvalue
  */
-static void BreakCond_UpdateTracked(bc_condition_t *condition, Uint32 value)
+static void BreakCond_UpdateTracked(bc_condition_t *condition, uint32_t value)
 {
-	Uint32 addr;
+	uint32_t addr;
 
 	/* next monitor changes to this new value */
 	condition->rvalue.value.number = value;
@@ -251,7 +252,7 @@ static void BreakCond_UpdateTracked(bc_condition_t *condition, Uint32 value)
 		addr = condition->lvalue.value.number;
 		fprintf(stderr, "  $%x = $%x\n", addr, value);
 	} else {
-		/* register tms. */
+		/* register etc. */
 		fprintf(stderr, "  $%x\n", value);
 	}
 }
@@ -262,10 +263,10 @@ static void BreakCond_UpdateTracked(bc_condition_t *condition, Uint32 value)
  */
 static bool BreakCond_MatchConditions(bc_condition_t *condition, int count)
 {
-	Uint32 lvalue, rvalue;
+	uint32_t lvalue, rvalue;
 	bool hit = false;
 	int i;
-	
+
 	for (i = 0; i < count; condition++, i++) {
 
 		lvalue = BreakCond_GetValue(&(condition->lvalue));
@@ -330,24 +331,27 @@ static bool BreakCond_MatchBreakPoints(bc_breakpoints_t *bps)
 			}
 			if (!bp->options.quiet) {
 				fprintf(stderr, "%d. %s breakpoint condition(s) matched %d times.\n",
-					i+1, bps->name, bp->hits);			
+					i+1, bps->name, bp->hits);
 				BreakCond_Print(bp);
 			}
 			History_Mark(bps->reason);
 
-			if (bp->options.lock || bp->options.filename) {
+			if (bp->options.info || bp->options.lock || bp->options.filename) {
 				bool reinit = !bp->options.noinit;
 
 				if (reinit) {
 					DebugCpu_InitSession();
 					DebugDsp_InitSession();
 				}
-
+				if (bp->options.info) {
+					bp->options.info(stderr, 0);
+				}
 				if (bp->options.lock) {
 					DebugInfo_ShowSessionInfo();
 				}
 				if (bp->options.filename) {
-					DebugUI_ParseFile(bp->options.filename, reinit);
+					bool verbose = !bp->options.quiet;
+					DebugUI_ParseFile(bp->options.filename, reinit, verbose);
 					changes = true;
 				}
 			}
@@ -445,7 +449,7 @@ static bool BreakCond_ParseVariable(const char *name, bc_value_t *bc_value)
 static bool BreakCond_ParseSymbol(const char *name, bc_value_t *bc_value)
 {
 	symtype_t symtype;
-	Uint32 addr;
+	uint32_t addr;
 
 	ENTERFUNC(("BreakCond_ParseSymbol('%s')\n", name));
 	if (bc_value->is_indirect) {
@@ -455,7 +459,7 @@ static bool BreakCond_ParseSymbol(const char *name, bc_value_t *bc_value)
 		/* direct value can be compared for anything */
 		symtype = SYMTYPE_ALL;
 	}
-	
+
 	if (bc_value->dsp_space) {
 		if (!Symbols_GetDspAddress(symtype, name, &addr)) {
 			EXITFUNC(("-> false (DSP)\n"));
@@ -468,7 +472,7 @@ static bool BreakCond_ParseSymbol(const char *name, bc_value_t *bc_value)
 		EXITFUNC(("-> true (DSP)\n"));
 		return true;
 	}
-	
+
 	if (!Symbols_GetCpuAddress(symtype, name, &addr)) {
 		EXITFUNC(("-> false (CPU)\n"));
 		return false;
@@ -487,16 +491,16 @@ static bool BreakCond_ParseSymbol(const char *name, bc_value_t *bc_value)
 
 
 /**
- * Helper function to get CPU PC register value with static inline as Uint32
+ * Helper function to get CPU PC register value with static inline as uint32_t
  */
-static Uint32 GetCpuPC(void)
+static uint32_t GetCpuPC(void)
 {
 	return M68000_GetPC();
 }
 /**
- * Helper function to get CPU SR register value with static inline as Uint32
+ * Helper function to get CPU SR register value with static inline as uint32_t
  */
-static Uint32 GetCpuSR(void)
+static uint32_t GetCpuSR(void)
 {
 	return M68000_GetSR();
 }
@@ -563,7 +567,7 @@ static bool BreakCond_ParseRegister(const char *regname, bc_value_t *bc_value)
  */
 static bool BreakCond_CheckAddress(bc_value_t *bc_value)
 {
-	Uint32 addr = bc_value->value.number;
+	uint32_t addr = bc_value->value.number;
 	int size = bc_value->bits >> 8;
 
 	ENTERFUNC(("BreakCond_CheckAddress(%x)\n", addr));
@@ -675,7 +679,7 @@ static bool BreakCond_ParseMaskModifier(parser_state_t *pstate, bc_value_t *bc_v
 		fprintf(stderr, "WARNING: plain numbers shouldn't need masks.\n");
 	}
 	pstate->arg++;
-	if (!Eval_Number(pstate->argv[pstate->arg], &(bc_value->mask))) {
+	if (!Eval_Number(pstate->argv[pstate->arg], &(bc_value->mask), NUM_TYPE_NORMAL)) {
 		pstate->error = "invalid dec/hex/bin value";
 		EXITFUNC(("arg:%d -> false\n", pstate->arg));
 		return false;
@@ -718,7 +722,7 @@ static bool BreakCond_ParseValue(parser_state_t *pstate, bc_value_t *bc_value)
 			skip = 2;
 		}
 	}
-	
+
 	str = pstate->argv[pstate->arg];
 	if (isalpha((unsigned char)*str) || *str == '_') {
 		/* parse direct or indirect variable/register/symbol name */
@@ -744,7 +748,7 @@ static bool BreakCond_ParseValue(parser_state_t *pstate, bc_value_t *bc_value)
 		}
 	} else {
 		/* a number */
-		if (!Eval_Number(str, &(bc_value->value.number))) {
+		if (!Eval_Number(str, &(bc_value->value.number), NUM_TYPE_NORMAL)) {
 			pstate->error = "invalid dec/hex/bin value";
 			EXITFUNC(("arg:%d -> false\n", pstate->arg));
 			return false;
@@ -783,7 +787,7 @@ static bool BreakCond_ParseValue(parser_state_t *pstate, bc_value_t *bc_value)
 static char BreakCond_ParseComparison(parser_state_t *pstate)
 {
 	const char *comparison;
-	
+
 	ENTERFUNC(("BreakCond_ParseComparison(), arg:%d\n", pstate->arg));
 	if (pstate->arg >= pstate->argc) {
 		pstate->error = "breakpoint comparison missing";
@@ -822,7 +826,7 @@ static char BreakCond_ParseComparison(parser_state_t *pstate)
 /**
  * If no value, use the other value, if that also missing, use default
  */
-static void BreakCond_InheritDefault(Uint32 *value1, Uint32 value2, Uint32 defvalue)
+static void BreakCond_InheritDefault(uint32_t *value1, uint32_t value2, uint32_t defvalue)
 {
 	if (!*value1) {
 		if (value2) {
@@ -842,7 +846,7 @@ static bool BreakCond_CrossCheckValues(parser_state_t *pstate,
 				       bc_value_t *bc_value1,
 				       bc_value_t *bc_value2)
 {
-	Uint32 mask1, mask2, defbits;
+	uint32_t mask1, mask2, defbits;
 	ENTERFUNC(("BreakCond_CrossCheckValues()\n"));
 
 	/* make sure there're valid bit widths and that masks have some value */
@@ -858,7 +862,7 @@ static bool BreakCond_CrossCheckValues(parser_state_t *pstate,
 
 	/* check first value mask & bit width */
 	mask1 = BITMASK(bc_value1->bits) & bc_value1->mask;
-	
+
 	if (mask1 != bc_value1->mask) {
 		fprintf(stderr, "WARNING: mask 0x%x doesn't fit into %d address/register bits.\n",
 			bc_value1->mask, bc_value1->bits);
@@ -869,7 +873,7 @@ static bool BreakCond_CrossCheckValues(parser_state_t *pstate,
 		fprintf(stderr, "WARNING: odd CPU address 0x%x given without using byte (.b) width.\n",
 			bc_value1->value.number);
 	}
-	
+
 	/* cross-check both values masks */
 	mask2 = BITMASK(bc_value2->bits) & bc_value2->mask;
 
@@ -1086,7 +1090,7 @@ static char *BreakCond_TokenizeExpression(const char *expression,
 
 
 /**
- * Select corrent breakpoints struct and provide name for it.
+ * Select current breakpoints struct and provide name for it.
  * Make sure there's always space for at least one additional breakpoint.
  * Return pointer to the breakpoints struct
  */
@@ -1138,12 +1142,12 @@ static void BreakCond_CheckTracking(bc_breakpoint_t *bp)
 {
 	bc_condition_t *condition;
 	bool track = false;
-	Uint32 value;
+	uint32_t value;
 	int i;
 
 	condition = bp->conditions;
 	for (i = 0; i < bp->ccount; condition++, i++) {
-		
+
 		if (memcmp(&(condition->lvalue), &(condition->rvalue), sizeof(bc_value_t)) == 0) {
 			/* set current value to right side */
 			value = BreakCond_GetValue(&(condition->rvalue));
@@ -1212,16 +1216,20 @@ static bool BreakCond_Parse(const char *expression, bc_options_t *options, bool
 				fprintf(stderr, "-> Break only on every %d hit.\n", options->skip);
 			}
 			if (options->once) {
-				fprintf(stderr, "-> Once, delete after breaking.\n");
+				fprintf(stderr, "-> Break only once, and delete breakpoint afterwards.\n");
 			}
 			if (options->trace) {
-				fprintf(stderr, "-> Trace instead of breaking, but show still hits.\n");
+				fprintf(stderr, "-> Trace (just show breakpoint info, instead of dropping to debugger).\n");
+				/* all of these options enable also trace option */
+				if (options->info) {
+					fprintf(stderr, "-> Call selected info command.\n");
+				}
 				if (options->lock) {
-					fprintf(stderr, "-> Show also info selected with lock command.\n");
-				}				
+					fprintf(stderr, "-> Call locked info command.\n");
+				}
 				if (options->noinit) {
-					fprintf(stderr, "-> Skip debugger inits on hit.\n");
-				}				
+					fprintf(stderr, "-> Skip debugger initialization on hit.\n");
+				}
 			}
 			if (options->filename) {
 				fprintf(stderr, "-> Execute debugger commands from '%s' file on hit.\n", options->filename);
@@ -1233,6 +1241,7 @@ static bool BreakCond_Parse(const char *expression, bc_options_t *options, bool
 		bp->options.skip = options->skip;
 		bp->options.once = options->once;
 		bp->options.trace = options->trace;
+		bp->options.info = options->info;
 		bp->options.lock = options->lock;
 		bp->options.noinit = options->noinit;
 		if (options->filename) {
@@ -1271,33 +1280,35 @@ static bool BreakCond_Parse(const char *expression, bc_options_t *options, bool
  */
 static void BreakCond_Print(bc_breakpoint_t *bp)
 {
-		fprintf(stderr, "\t%s", bp->expression);
-		if (bp->options.skip) {
-			fprintf(stderr, " :%d", bp->options.skip);
+	fprintf(stderr, "\t%s", bp->expression);
+	if (bp->options.skip) {
+		fprintf(stderr, " :%d", bp->options.skip);
+	}
+	if (bp->options.once) {
+		fprintf(stderr, " :once");
+	}
+	if (bp->options.quiet) {
+		fprintf(stderr, " :quiet");
+	}
+	if (bp->options.trace) {
+		fprintf(stderr, " :trace");
+		if (bp->options.info) {
+			fprintf(stderr, " :info");
 		}
-		if (bp->options.once) {
-			fprintf(stderr, " :once");
+		if (bp->options.lock) {
+			fprintf(stderr, " :lock");
 		}
-		if (bp->options.quiet) {
-			fprintf(stderr, " :quiet");
+		if (bp->options.noinit) {
+			fprintf(stderr, " :noinit");
 		}
-		if (bp->options.trace) {
-			if (bp->options.lock) {
-				fprintf(stderr, " :lock");
-			} else {
-				fprintf(stderr, " :trace");
-			}
-			if (bp->options.noinit) {
-				fprintf(stderr, " :noinit");
-			}
-		}
-		if (bp->options.filename) {
-			fprintf(stderr, " :file %s", bp->options.filename);
-		}
-		if (bp->options.deleted) {
-			fprintf(stderr, " (deleted)");
-		}
-		fprintf(stderr, "\n");
+	}
+	if (bp->options.filename) {
+		fprintf(stderr, " :file %s", bp->options.filename);
+	}
+	if (bp->options.deleted) {
+		fprintf(stderr, " (deleted)");
+	}
+	fprintf(stderr, "\n");
 }
 
 /**
@@ -1462,11 +1473,12 @@ const char BreakCond_Description[] =
 	"\tMultiple breakpoint action options can be specified after\n"
 	"\tthe breakpoint condition(s):\n"
 	"\t- 'trace', print the breakpoint match without stopping\n"
-	"\t- 'lock', print the debugger entry info without stopping\n"
+	"\t- 'info <name>', call indicated info functionality (enables 'trace')\n"
+	"\t- 'lock', print the locked debugger entry info (enables 'trace')\n"
 	"\t- 'noinit', no debugger inits on hit, useful for stack tracing\n"
 	"\t- 'file <file>', execute debugger commands from given <file>\n"
 	"\t- 'once', delete the breakpoint after it's hit\n"
-	"\t- 'quiet', no output from setting & hitting breakpoint\n"
+	"\t- 'quiet', set / hit breakpoint quietly\n"
 	"\t- '<count>', break only on every <count> hit";
 
 /**
@@ -1475,9 +1487,9 @@ const char BreakCond_Description[] =
  */
 static bool BreakCond_Options(char *str, bc_options_t *options, char marker)
 {
-	char *option, *next, *filename;
+	char *option, *next, *filename, *info;
 	int skip;
-	
+
 	memset(options, 0, sizeof(*options));
 
 	option = strchr(str, marker);
@@ -1487,10 +1499,12 @@ static bool BreakCond_Options(char *str, bc_options_t *options, char marker)
 	}
 	for (next = option; option; option = next) {
 
+		char splitter[3] = {' ', marker, '\0'};
 		/* skip marker + end & trim this option */
 		option = next + 1;
-		next = strchr(option, marker);
+		next = strstr(option, splitter);
 		if (next) {
+			next++; /* skip space to next marker */
 			*next = 0;
 		}
 		option = Str_Trim(option);
@@ -1507,6 +1521,14 @@ static bool BreakCond_Options(char *str, bc_options_t *options, char marker)
 		} else if (strcmp(option, "noinit") == 0) {
 			options->trace = true;
 			options->noinit = true;
+		} else if (strncmp(option, "info ", 5) == 0) {
+			options->trace = true;
+			info = Str_Trim(option+4);
+			options->info = DebugInfo_GetInfoFunc(info);
+			if (!options->info) {
+				fprintf(stderr, "ERROR: no info for '%s'!\n", info);
+				return false;
+			}
 		} else if (strncmp(option, "file ", 5) == 0) {
 			filename = Str_Trim(option+4);
 			if (!File_Exists(filename)) {
@@ -1552,9 +1574,9 @@ bool BreakCond_Command(const char *args, bool bForDsp)
 	}
 	argscopy = strdup(args);
 	assert(argscopy);
-	
+
 	expression = Str_Trim(argscopy);
-	
+
 	/* subcommands? */
 	if (strncmp(expression, "help", 4) == 0) {
 		fputs(BreakCond_Help, stderr);
@@ -1603,7 +1625,7 @@ const char BreakAddr_Description[] =
 	"\t- 'trace', print the breakpoint match without stopping\n"
 	"\t- 'lock', print the debugger entry info without stopping\n"
 	"\t- 'once', delete the breakpoint after it's hit\n"
-	"\t- 'quiet', no output from setting & hitting breakpoint\n"
+	"\t- 'quiet', set / hit breakpoint quietly\n"
 	"\t- '<count>', break only on every <count> hit\n"
 	"\n"
 	"\tUse conditional breakpoint commands to manage the created\n"
@@ -1618,7 +1640,7 @@ bool BreakAddr_Command(char *args, bool bForDsp)
 {
 	const char *errstr, *expression = (const char *)args;
 	char *cut, command[32];
-	Uint32 addr;
+	uint32_t addr;
 	int offset;
 
 	if (!args) {
