@@ -12,11 +12,14 @@
 
 extern void DebugCpu_Check(void);
 extern void DebugCpu_SetDebugging(void);
-extern Uint32 DebugCpu_InstrCount(void);
-extern Uint32 DebugCpu_OpcodeType(void);
+
+extern uint32_t DebugCpu_CallDepth(void);
+extern uint32_t DebugCpu_InstrCount(void);
+extern uint32_t DebugCpu_OpcodeType(void);
+
 extern int DebugCpu_DisAsm(int nArgc, char *psArgs[]);
 extern int DebugCpu_MemDump(int nArgc, char *psArgs[]);
 extern int DebugCpu_Register(int nArgc, char *psArgs[]);
-extern int DebugCpu_GetRegisterAddress(const char *reg, Uint32 **addr);
+extern int DebugCpu_GetRegisterAddress(const char *reg, uint32_t **addr);
 
 #endif /* HATARI_DEBUGCPU_H */
