@@ -13,7 +13,7 @@ typedef enum {
 	HISTORY_TRACK_NONE = 0,
 	HISTORY_TRACK_CPU = 1,
 	HISTORY_TRACK_DSP = 2,
-	HISTORY_TRACK_ALL = 3
+	HISTORY_TRACK_ALL = (HISTORY_TRACK_CPU|HISTORY_TRACK_DSP)
 } history_type_t;
 
 extern history_type_t HistoryTracking;
@@ -30,9 +30,10 @@ static inline bool History_TrackDsp(void)
 /* for debugcpu/dsp.c */
 extern void History_AddCpu(void);
 extern void History_AddDsp(void);
+extern uint32_t History_DisasmAddr(uint32_t pc, uint32_t offset, bool for_dsp);
 
 /* for debugInfo.c */
-extern void History_Show(Uint32 count);
+extern void History_Show(FILE *fp, uint32_t count);
 
 /* for debugui */
 extern void History_Mark(debug_reason_t reason);
