@@ -34,8 +34,8 @@
     $FF8911 (byte) : Frame End Mi
     $FF8913 (byte) : Frame End Lo
     $FF8920 (word) : Sound Mode Control (frequency, mono/stereo)
-    $FF8922 (byte) : Microwire Data Register
-    $FF8924 (byte) : Microwire Mask Register
+    $FF8922 (word) : Microwire Data Register
+    $FF8924 (word) : Microwire Mask Register
 
   
   The Microwire and LMC 1992 commands :
@@ -93,12 +93,14 @@
 */
 
 
-const char DmaSnd_fileid[] = "Hatari dmaSnd.c : " __DATE__ " " __TIME__;
+const char DmaSnd_fileid[] = "Hatari dmaSnd.c";
 
+#include <SDL_stdinc.h>		/* Required for M_PI */
 #include "main.h"
 #include "audio.h"
 #include "configuration.h"
 #include "dmaSnd.h"
+#include "cycles.h"
 #include "cycInt.h"
 #include "ioMem.h"
 #include "log.h"
@@ -107,9 +109,9 @@ const char DmaSnd_fileid[] = "Hatari dmaSnd.c : " __DATE__ " " __TIME__;
 #include "sound.h"
 #include "stMemory.h"
 #include "crossbar.h"
-#include "screen.h"
 #include "video.h"
 #include "m68000.h"
+#include "clocks_timings.h"
 
 #define TONE_STEPS 13
 
@@ -125,46 +127,48 @@ static float DmaSnd_IIRfilterL(float xn);
 static float DmaSnd_IIRfilterR(float xn);
 static struct first_order_s *DmaSnd_Treble_Shelf(float g, float fc, float Fs);
 static struct first_order_s *DmaSnd_Bass_Shelf(float g, float fc, float Fs);
-static Sint16 DmaSnd_LowPassFilterLeft(Sint16 in);
-static Sint16 DmaSnd_LowPassFilterRight(Sint16 in);
+static int16_t DmaSnd_LowPassFilterLeft(int16_t in);
+static int16_t DmaSnd_LowPassFilterRight(int16_t in);
 static bool DmaSnd_LowPass;
 
 
-Uint16 nDmaSoundControl;                /* Sound control register */
+uint16_t nDmaSoundControl;              /* Sound control register */
 
 struct first_order_s  { float a1, b0, b1; };
 struct second_order_s { float a1, a2, b0, b1, b2; };
 
 struct dma_s {
-	Uint16 soundMode;		/* Sound mode register */
-	Uint32 frameStartAddr;		/* Sound frame start */
-	Uint32 frameEndAddr;		/* Sound frame end */
-	Uint32 frameCounterAddr;	/* Sound frame current address counter */
+	uint16_t soundMode;		/* Sound mode register */
+	uint32_t frameStartAddr;	/* Sound frame start */
+	uint32_t frameEndAddr;		/* Sound frame end */
+	uint32_t frameCounterAddr;	/* Sound frame current address counter */
 
 	/* Internal 8 byte FIFO */
-	Sint8 FIFO[ DMASND_FIFO_SIZE ];
-	Uint16 FIFO_Pos;		/* from 0 to DMASND_FIFO_SIZE-1 */
-	Uint16 FIFO_NbBytes;		/* from 0 to DMASND_FIFO_SIZE */
+	int8_t FIFO[ DMASND_FIFO_SIZE ];
+	uint16_t FIFO_Pos;		/* from 0 to DMASND_FIFO_SIZE-1 */
+	uint16_t FIFO_NbBytes;		/* from 0 to DMASND_FIFO_SIZE */
 
-	Sint16 FrameLeft;		/* latest values read from the FIFO */
-	Sint16 FrameRight;
+	int16_t FrameLeft;		/* latest values read from the FIFO */
+	int16_t FrameRight;
+
+	uint8_t  XSINT_Signal;		/* Value of the XSINT signal (connected to MFP) */
 };
 
-static Sint64	frameCounter_float = 0;
+static int64_t	frameCounter_float = 0;
 static bool	DmaInitSample = false;
 
 
 struct microwire_s {
-	Uint16 data;			/* Microwire Data register */
-	Uint16 mask;			/* Microwire Mask register */
-	Uint16 mwTransferSteps;		/* Microwire shifting counter */
-	Uint16 pendingCyclesOver;	/* Number of delayed cycles for the interrupt */
-	Uint16 mixing;			/* Mixing command */
-	Uint16 bass;			/* Bass command */
-	Uint16 treble;			/* Treble command */
-	Uint16 masterVolume;		/* Master volume command */
-	Uint16 leftVolume;		/* Left channel volume command */
-	Uint16 rightVolume;		/* Right channel volume command */
+	uint16_t data;			/* Microwire Data register */
+	uint16_t mask;			/* Microwire Mask register */
+	uint16_t mwTransferSteps;	/* Microwire shifting counter */
+	uint16_t pendingCyclesOver;	/* Number of delayed cycles for the interrupt */
+	uint16_t mixing;		/* Mixing command */
+	uint16_t bass;			/* Bass command */
+	uint16_t treble;		/* Treble command */
+	uint16_t masterVolume;		/* Master volume command */
+	uint16_t leftVolume;		/* Left channel volume command */
+	uint16_t rightVolume;		/* Right channel volume command */
 };
 
 struct lmc1992_s {
@@ -183,7 +187,7 @@ static struct lmc1992_s lmc1992;
 /* Table gain values = (int)(powf(10.0, dB/20.0)*65536.0 + 0.5)  2dB steps   */
 
 /* Values for LMC1992 Master volume control (*65536) */
-static const Uint16 LMC1992_Master_Volume_Table[64] =
+static const uint16_t LMC1992_Master_Volume_Table[64] =
 {
 	    7,     8,    10,    13,    16,    21,    26,    33,    41,    52,  /* -80dB */
 	   66,    83,   104,   131,   165,   207,   261,   328,   414,   521,  /* -60dB */
@@ -195,7 +199,7 @@ static const Uint16 LMC1992_Master_Volume_Table[64] =
 };
 
 /* Values for LMC1992 Left and right volume control (*65536) */
-static const Uint16 LMC1992_LeftRight_Volume_Table[32] =
+static const uint16_t LMC1992_LeftRight_Volume_Table[32] =
 {
 	  655,   825,  1039,  1308,  1646,  2072,  2609,  3285,  4135,  5206,  /* -40dB */
 	 6554,  8250, 10387, 13076, 16462, 20724, 26090, 32846, 41350, 52057,  /* -20dB */
@@ -204,7 +208,7 @@ static const Uint16 LMC1992_LeftRight_Volume_Table[32] =
 };
 
 /* Values for LMC1992 BASS and TREBLE */
-static const Sint16 LMC1992_Bass_Treble_Table[16] =
+static const int16_t LMC1992_Bass_Treble_Table[16] =
 {
 	0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 12, 12, 12
 };
@@ -220,8 +224,10 @@ static const int DmaSndSampleRates[4] =
 /* Local functions prototypes					*/
 /*--------------------------------------------------------------*/
 
+static void	DmaSnd_Update_XSINT_Line ( uint8_t Bit );
+
 static void	DmaSnd_FIFO_Refill(void);
-static Sint8	DmaSnd_FIFO_PullByte(void);
+static int8_t	DmaSnd_FIFO_PullByte(void);
 static void	DmaSnd_FIFO_SetStereo(void);
 
 static int	DmaSnd_DetectSampleRate(void);
@@ -250,6 +256,8 @@ void DmaSnd_Reset(bool bCold)
 	dma.FrameLeft = 0;
 	dma.FrameRight = 0;
 
+	DmaSnd_Update_XSINT_Line ( MFP_GPIP_STATE_LOW );	/* O/LOW=dma sound idle */
+
 	if ( bCold )
 	{
 		/* Microwire has no reset signal, it will keep its values on warm reset */
@@ -282,6 +290,39 @@ void DmaSnd_MemorySnapShot_Capture(bool bSave)
 }
 
 
+/*-----------------------------------------------------------------------*/
+/**
+ * Update the value of the XSINT line ; this line is connected to TAI and to GPIP7
+ * Depending on the transition, this can trigger MFP interrupt for Timer A or for GPIP7
+ *  - Bit is set to 0/LOW when dma sound is idle
+ *  - Bit is set to 1/HIGH when dma sound is playing
+ *
+ * Timer A input is associated to GPIP4. Under default TOS behaviour AER bit for GPIP4 is set to 0
+ * (because it's also shared with ACIA's interrupt lines which are active low).
+ * This means that each time XSINT goes to idle/0 (when reaching end of frame in single or repeat mode)
+ * an interrupt will trigger on Timer A and Timer A event count mode can be used to count
+ * the number of "end of frame" events
+ */
+static void DmaSnd_Update_XSINT_Line ( uint8_t Bit )
+{
+	dma.XSINT_Signal = Bit;
+	MFP_GPIP_Set_Line_Input ( pMFP_Main , MFP_GPIP_LINE7 , Bit );
+	MFP_TimerA_Set_Line_Input ( pMFP_Main , Bit );			/* Update events count / interrupt for timer A if needed */
+}
+
+
+/*-----------------------------------------------------------------------*/
+/**
+ * Return the value of the XSINT line
+ *  0=dma sound is idle
+ *  1=dma sound is playing
+ */
+uint8_t	DmaSnd_Get_XSINT_Line ( void )
+{
+	return dma.XSINT_Signal;
+}
+
+
 /*-----------------------------------------------------------------------*/
 /**
  * This function is called on every HBL to ensure the DMA Audio's FIFO
@@ -310,10 +351,10 @@ static void DmaSnd_FIFO_Refill(void)
 	{
 		/* Add one word to the FIFO */
 		LOG_TRACE(TRACE_DMASND, "DMA snd fifo refill adr=%x pos %d nb %d %x %x\n", dma.frameCounterAddr , dma.FIFO_Pos , dma.FIFO_NbBytes ,
-			STRam[ dma.frameCounterAddr ] , STRam[ dma.frameCounterAddr+1 ] );
+			STMemory_DMA_ReadByte ( dma.frameCounterAddr ) , STMemory_DMA_ReadByte ( dma.frameCounterAddr+1 ) );
 
-		dma.FIFO[ ( dma.FIFO_Pos+dma.FIFO_NbBytes+0 ) & DMASND_FIFO_SIZE_MASK ] = (Sint8)STRam[ dma.frameCounterAddr ];	/* add upper byte of the word */
-		dma.FIFO[ ( dma.FIFO_Pos+dma.FIFO_NbBytes+1 ) & DMASND_FIFO_SIZE_MASK ] = (Sint8)STRam[ dma.frameCounterAddr+1 ];	/* add lower byte of the word */
+		dma.FIFO[ ( dma.FIFO_Pos+dma.FIFO_NbBytes+0 ) & DMASND_FIFO_SIZE_MASK ] = (int8_t)STMemory_DMA_ReadByte ( dma.frameCounterAddr );	/* add upper byte of the word */
+		dma.FIFO[ ( dma.FIFO_Pos+dma.FIFO_NbBytes+1 ) & DMASND_FIFO_SIZE_MASK ] = (int8_t)STMemory_DMA_ReadByte ( dma.frameCounterAddr+1 );	/* add lower byte of the word */
 
 		dma.FIFO_NbBytes += 2;				/* One word more in the FIFO */
 
@@ -344,9 +385,9 @@ static void DmaSnd_FIFO_Refill(void)
  * to be called if FIFO is empty but DMA sound is still ON.
  * This way, sound remains correct even if the user uses very low output freq.
  */
-static Sint8 DmaSnd_FIFO_PullByte(void)
+static int8_t DmaSnd_FIFO_PullByte(void)
 {
-	Sint8	sample;
+	int8_t	sample;
 
 	if ( dma.FIFO_NbBytes == 0 )
 	{
@@ -359,7 +400,7 @@ static Sint8 DmaSnd_FIFO_PullByte(void)
 	}
 
 
-	LOG_TRACE(TRACE_DMASND, "DMA snd fifo pull pos %d nb %d %02x\n", dma.FIFO_Pos , dma.FIFO_NbBytes , (Uint8)dma.FIFO[ dma.FIFO_Pos ] );
+	LOG_TRACE(TRACE_DMASND, "DMA snd fifo pull pos %d nb %d %02x\n", dma.FIFO_Pos , dma.FIFO_NbBytes , (uint8_t)dma.FIFO[ dma.FIFO_Pos ] );
 
 	sample = dma.FIFO[ dma.FIFO_Pos ];			/* Get oldest byte from the FIFO */
 	dma.FIFO_Pos = (dma.FIFO_Pos+1) & DMASND_FIFO_SIZE_MASK;/* Pos to be pulled on next call */
@@ -378,7 +419,7 @@ static Sint8 DmaSnd_FIFO_PullByte(void)
  */
 static void DmaSnd_FIFO_SetStereo(void)
 {
-	Uint16	NewPos;
+	uint16_t	NewPos;
 
 	if ( dma.FIFO_Pos & 1 )
 	{
@@ -431,7 +472,11 @@ static void DmaSnd_StartNewFrame(void)
 	{
 		nDmaSoundControl &= ~DMASNDCTRL_PLAY;
 		LOG_TRACE(TRACE_DMASND, "DMA snd stopped because new frame start=end=%x and repeat=off\n", dma.frameStartAddr);
+		return;
 	}
+
+	/* DMA sound play : update XSINT */
+	DmaSnd_Update_XSINT_Line ( MFP_GPIP_STATE_HIGH );	/* 1/HIGH=dma sound play */
 }
 
 
@@ -440,19 +485,26 @@ static void DmaSnd_StartNewFrame(void)
  * End-of-frame has been reached. Raise interrupts if needed.
  * Returns true if DMA sound processing should be stopped now and false
  * if it continues (DMA PLAYLOOP mode).
+ *
+ * NOTE : on early STE models the XSINT signal was directly connected
+ * to MFP GPIP7 and to Timer A input (for event count mode).
+ * On later revisions, as well as on TT, the signal to Timer A input
+ * is now delayed by 8 shifts using a 74LS164 running at 2 MHz, which
+ * is equivalent to 32 CPU cycles when the CPU runs at 8 MHz.
+ * At the emulation level, we don't take into account this delay of
+ * 32 CPU cycles, as it would add complexity and no program are known
+ * so far to require this delay.
  */
 static inline int DmaSnd_EndOfFrameReached(void)
 {
 	LOG_TRACE(TRACE_DMASND, "DMA snd end of frame\n");
 
-	/* Raise end-of-frame interrupts (MFP-i7 and Time-A) */
-	MFP_InputOnChannel ( MFP_INT_GPIP7 , 0 );
-	if (MFP_TACR == 0x08)       /* Is timer A in Event Count mode? */
-		MFP_TimerA_EventCount_Interrupt();
+	/* DMA sound idle : update XSINT */
+	DmaSnd_Update_XSINT_Line ( MFP_GPIP_STATE_LOW );		/* O/LOW=dma sound idle */
 
 	if (nDmaSoundControl & DMASNDCTRL_PLAYLOOP)
 	{
-		DmaSnd_StartNewFrame();
+		DmaSnd_StartNewFrame();					/* update XSINT */
 	}
 	else
 	{
@@ -488,9 +540,9 @@ void DmaSnd_GenerateSamples(int nMixBufIdx, int nSamplesToGenerate)
 {
 	int i;
 	int nBufIdx;
-	Sint8 MonoByte , LeftByte , RightByte;
+	int8_t MonoByte , LeftByte , RightByte;
 	unsigned n;
-	Sint64 FreqRatio;
+	int64_t FreqRatio;
 
 
 	/* DMA Audio OFF and FIFO empty : process YM2149's output */
@@ -498,20 +550,20 @@ void DmaSnd_GenerateSamples(int nMixBufIdx, int nSamplesToGenerate)
 	{
 		for (i = 0; i < nSamplesToGenerate; i++)
 		{
-			nBufIdx = (nMixBufIdx + i) % MIXBUFFER_SIZE;
+			nBufIdx = (nMixBufIdx + i) & AUDIOMIXBUFFER_SIZE_MASK;
 
 			switch (microwire.mixing) {
 				case 1:
 					/* DMA and YM2149 mixing */
-					MixBuffer[nBufIdx][0] = MixBuffer[nBufIdx][0] + dma.FrameLeft * -((256*3/4)/4)/4;
-					MixBuffer[nBufIdx][1] = MixBuffer[nBufIdx][1] + dma.FrameRight * -((256*3/4)/4)/4;
+					AudioMixBuffer[nBufIdx][0] = AudioMixBuffer[nBufIdx][0] + dma.FrameLeft * -((256*3/4)/4)/4;
+					AudioMixBuffer[nBufIdx][1] = AudioMixBuffer[nBufIdx][1] + dma.FrameRight * -((256*3/4)/4)/4;
 					break;
 				default:
 					/* mixing=0 DMA only */
 					/* mixing=2 DMA and input 2 (YM2149 LPF) -> DMA */
 					/* mixing=3 DMA and input 3 -> DMA */
-					MixBuffer[nBufIdx][0] = dma.FrameLeft * -((256*3/4)/4)/4;
-					MixBuffer[nBufIdx][1] = dma.FrameRight * -((256*3/4)/4)/4;
+					AudioMixBuffer[nBufIdx][0] = dma.FrameLeft * -((256*3/4)/4)/4;
+					AudioMixBuffer[nBufIdx][1] = dma.FrameRight * -((256*3/4)/4)/4;
 					break;
 			}
 		}
@@ -533,7 +585,7 @@ void DmaSnd_GenerateSamples(int nMixBufIdx, int nSamplesToGenerate)
 
 	/* Compute ratio between DMA's sound frequency and host computer's sound frequency, */
 	/* use << 32 to simulate floating point precision */
-	FreqRatio = ( ((Sint64)DmaSnd_DetectSampleRate()) << 32 ) / nAudioFrequency;
+	FreqRatio = ( ((int64_t)DmaSnd_DetectSampleRate()) << 32 ) / nAudioFrequency;
 
 	if (dma.soundMode & DMASNDMODE_MONO)
 	{
@@ -543,27 +595,27 @@ void DmaSnd_GenerateSamples(int nMixBufIdx, int nSamplesToGenerate)
 			if ( DmaInitSample )
 			{
 				MonoByte = DmaSnd_FIFO_PullByte ();
-				dma.FrameLeft  = DmaSnd_LowPassFilterLeft( (Sint16)MonoByte );
-				dma.FrameRight = DmaSnd_LowPassFilterRight( (Sint16)MonoByte );
+				dma.FrameLeft  = DmaSnd_LowPassFilterLeft( (int16_t)MonoByte );
+				dma.FrameRight = DmaSnd_LowPassFilterRight( (int16_t)MonoByte );
 				DmaInitSample = false;
 			}
 
-			nBufIdx = (nMixBufIdx + i) % MIXBUFFER_SIZE;
+			nBufIdx = (nMixBufIdx + i) & AUDIOMIXBUFFER_SIZE_MASK;
 
 			switch (microwire.mixing) {
 				case 1:
 					/* DMA and YM2149 mixing */
-					MixBuffer[nBufIdx][0] = MixBuffer[nBufIdx][0] + dma.FrameLeft * -((256*3/4)/4)/4;
+					AudioMixBuffer[nBufIdx][0] = AudioMixBuffer[nBufIdx][0] + dma.FrameLeft * -((256*3/4)/4)/4;
 					break;
 				default:
 					/* mixing=0 DMA only */
 					/* mixing=2 DMA and input 2 (YM2149 LPF) -> DMA */
 					/* mixing=3 DMA and input 3 -> DMA */
-					MixBuffer[nBufIdx][0] = dma.FrameLeft * -((256*3/4)/4)/4;
+					AudioMixBuffer[nBufIdx][0] = dma.FrameLeft * -((256*3/4)/4)/4;
 					break;
 			}
 
-			MixBuffer[nBufIdx][1] = MixBuffer[nBufIdx][0];		/* right = left */
+			AudioMixBuffer[nBufIdx][1] = AudioMixBuffer[nBufIdx][0];	/* right = left */
 
 			/* Increase freq counter */
 			frameCounter_float += FreqRatio;
@@ -571,8 +623,8 @@ void DmaSnd_GenerateSamples(int nMixBufIdx, int nSamplesToGenerate)
 			while ( n > 0 )						/* pull as many bytes from the FIFO as needed */
 			{
 				MonoByte = DmaSnd_FIFO_PullByte ();
-				dma.FrameLeft  = DmaSnd_LowPassFilterLeft( (Sint16)MonoByte );
-				dma.FrameRight = DmaSnd_LowPassFilterRight( (Sint16)MonoByte );
+				dma.FrameLeft  = DmaSnd_LowPassFilterLeft( (int16_t)MonoByte );
+				dma.FrameRight = DmaSnd_LowPassFilterRight( (int16_t)MonoByte );
 				n--;
 			}
 			frameCounter_float &= 0xffffffff;			/* only keep the fractional part */
@@ -587,25 +639,25 @@ void DmaSnd_GenerateSamples(int nMixBufIdx, int nSamplesToGenerate)
 			{
 				LeftByte = DmaSnd_FIFO_PullByte ();
 				RightByte = DmaSnd_FIFO_PullByte ();
-				dma.FrameLeft  = DmaSnd_LowPassFilterLeft( (Sint16)LeftByte );
-				dma.FrameRight = DmaSnd_LowPassFilterRight( (Sint16)RightByte );
+				dma.FrameLeft  = DmaSnd_LowPassFilterLeft( (int16_t)LeftByte );
+				dma.FrameRight = DmaSnd_LowPassFilterRight( (int16_t)RightByte );
 				DmaInitSample = false;
 			}
 
-			nBufIdx = (nMixBufIdx + i) % MIXBUFFER_SIZE;
+			nBufIdx = (nMixBufIdx + i) & AUDIOMIXBUFFER_SIZE_MASK;
 
 			switch (microwire.mixing) {
 				case 1:
 					/* DMA and YM2149 mixing */
-					MixBuffer[nBufIdx][0] = MixBuffer[nBufIdx][0] + dma.FrameLeft * -((256*3/4)/4)/4;
-					MixBuffer[nBufIdx][1] = MixBuffer[nBufIdx][1] + dma.FrameRight * -((256*3/4)/4)/4;
+					AudioMixBuffer[nBufIdx][0] = AudioMixBuffer[nBufIdx][0] + dma.FrameLeft * -((256*3/4)/4)/4;
+					AudioMixBuffer[nBufIdx][1] = AudioMixBuffer[nBufIdx][1] + dma.FrameRight * -((256*3/4)/4)/4;
 					break;
 				default:
 					/* mixing=0 DMA only */
 					/* mixing=2 DMA and input 2 (YM2149 LPF) -> DMA */
 					/* mixing=3 DMA and input 3 -> DMA */
-					MixBuffer[nBufIdx][0] = dma.FrameLeft * -((256*3/4)/4)/4;
-					MixBuffer[nBufIdx][1] = dma.FrameRight * -((256*3/4)/4)/4;
+					AudioMixBuffer[nBufIdx][0] = dma.FrameLeft * -((256*3/4)/4)/4;
+					AudioMixBuffer[nBufIdx][1] = dma.FrameRight * -((256*3/4)/4)/4;
 					break;
 			}
 
@@ -616,8 +668,8 @@ void DmaSnd_GenerateSamples(int nMixBufIdx, int nSamplesToGenerate)
 			{
 				LeftByte = DmaSnd_FIFO_PullByte ();
 				RightByte = DmaSnd_FIFO_PullByte ();
-				dma.FrameLeft  = DmaSnd_LowPassFilterLeft( (Sint16)LeftByte );
-				dma.FrameRight = DmaSnd_LowPassFilterRight( (Sint16)RightByte );
+				dma.FrameLeft  = DmaSnd_LowPassFilterLeft( (int16_t)LeftByte );
+				dma.FrameRight = DmaSnd_LowPassFilterRight( (int16_t)RightByte );
 				n--;
 			}
 			frameCounter_float &= 0xffffffff;			/* only keep the fractional part */
@@ -639,25 +691,25 @@ static void DmaSnd_Apply_LMC(int nMixBufIdx, int nSamplesToGenerate)
 {
 	int nBufIdx;
 	int i;
-	Sint32 sample;
+	int32_t sample;
 
 	/* Apply LMC1992 sound modifications (Left, Right and Master Volume) */
 	for (i = 0; i < nSamplesToGenerate; i++) {
-		nBufIdx = (nMixBufIdx + i) % MIXBUFFER_SIZE;
+		nBufIdx = (nMixBufIdx + i) & AUDIOMIXBUFFER_SIZE_MASK;
 
-		sample = DmaSnd_IIRfilterL( Subsonic_IIR_HPF_Left( MixBuffer[nBufIdx][0]));
+		sample = DmaSnd_IIRfilterL( Subsonic_IIR_HPF_Left( AudioMixBuffer[nBufIdx][0]));
 		if (sample<-32767)						/* check for overflow to clip waveform */
 			sample = -32767;
 		else if (sample>32767)
 			sample = 32767;
-		MixBuffer[nBufIdx][0] = sample;
+		AudioMixBuffer[nBufIdx][0] = sample;
 
-		sample = DmaSnd_IIRfilterR( Subsonic_IIR_HPF_Right(MixBuffer[nBufIdx][1]));
+		sample = DmaSnd_IIRfilterR( Subsonic_IIR_HPF_Right(AudioMixBuffer[nBufIdx][1]));
 		if (sample<-32767)						/* check for overflow to clip waveform */
 			sample = -32767;
 		else if (sample>32767)
 			sample = 32767;
-		MixBuffer[nBufIdx][1] = sample;
+		AudioMixBuffer[nBufIdx][1] = sample;
  	}
 }
 
@@ -671,18 +723,19 @@ static void DmaSnd_Apply_LMC(int nMixBufIdx, int nSamplesToGenerate)
  * (eg the demo 'Mental Hangover' or the game 'Power Up Plus')
  * We first check if the FIFO needs to be refilled, then we call Sound_Update.
  * This function should be called from the HBL's handler (in video.c)
+ * We should call it also in the case of the TT which uses the same DMA sound
  */
 void DmaSnd_STE_HBL_Update(void)
 {
-	if (!Config_IsMachineSTE())
+	if ( !Config_IsMachineSTE() && !Config_IsMachineTT() )
 		return;
 
-	/* The DMA starts refilling the FIFO when display is OFF (eg cycle 376 in low res 50 Hz) */
+	/* The DMA starts refilling the FIFO when display is OFF (eg cycle 376 in STE low res 50 Hz) */
 	DmaSnd_FIFO_Refill ();
 
 	/* If DMA sound is ON or FIFO is not empty, update sound */
 	if  ( (nDmaSoundControl & DMASNDCTRL_PLAY) || ( dma.FIFO_NbBytes > 0 ) )
-		Sound_Update(false);
+		Sound_Update ( CyclesGlobalClockCounter );
 
 	/* As long as display is OFF, the DMA will refill the FIFO after playing some samples during the HBL */
 	DmaSnd_FIFO_Refill ();
@@ -693,12 +746,12 @@ void DmaSnd_STE_HBL_Update(void)
 /**
  * Return current frame counter address (value is always even)
  */
-static Uint32 DmaSnd_GetFrameCount(void)
+static uint32_t DmaSnd_GetFrameCount(void)
 {
-	Uint32 nActCount;
+	uint32_t nActCount;
 
 	/* Update sound to get the current DMA frame address */
-	Sound_Update(false);
+	Sound_Update ( CyclesGlobalClockCounter );
 
 	if (nDmaSoundControl & DMASNDCTRL_PLAY)
 		nActCount = dma.frameCounterAddr;
@@ -719,8 +772,8 @@ void DmaSnd_SoundControl_ReadWord(void)
 
 	if(LOG_TRACE_LEVEL(TRACE_DMASND))
 	{
-                int FrameCycles, HblCounterVideo, LineCycles;
-                Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
 		LOG_TRACE_PRINT("DMA snd control read: 0x%04x video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
 			nDmaSoundControl,
 			FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
@@ -734,19 +787,19 @@ void DmaSnd_SoundControl_ReadWord(void)
  */
 void DmaSnd_SoundControl_WriteWord(void)
 {
-	Uint16 DMASndCtrl_old;
+	uint16_t DMASndCtrl_old;
 
 	if(LOG_TRACE_LEVEL(TRACE_DMASND))
 	{
-                int FrameCycles, HblCounterVideo, LineCycles;
-                Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
 		LOG_TRACE_PRINT("DMA snd control write: 0x%04x video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
 			IoMem_ReadWord(0xff8900),
 			FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
 	}
 
-        /* Before starting/stopping DMA sound, create samples up until this point with current values */
-	Sound_Update(false);
+	/* Before starting/stopping DMA sound, create samples up until this point with current values */
+	Sound_Update ( Cycles_GetClockCounterOnWriteAccess() );
 
 	DMASndCtrl_old = nDmaSoundControl;
 	nDmaSoundControl = IoMem_ReadWord(0xff8900) & 3;
@@ -756,11 +809,12 @@ void DmaSnd_SoundControl_WriteWord(void)
 		LOG_TRACE(TRACE_DMASND, "DMA snd control write: starting dma sound output\n");
 		DmaInitSample = true;
 		frameCounter_float = 0;
-		DmaSnd_StartNewFrame();			/* this can clear DMASNDCTRL_PLAY */
+		DmaSnd_StartNewFrame();				/* update XSINT + this can clear DMASNDCTRL_PLAY */
 	}
 	else if ((DMASndCtrl_old & DMASNDCTRL_PLAY) && !(nDmaSoundControl & DMASNDCTRL_PLAY))
 	{
 		LOG_TRACE(TRACE_DMASND, "DMA snd control write: stopping dma sound output\n");
+		DmaSnd_Update_XSINT_Line ( MFP_GPIP_STATE_LOW );		/* O/LOW=dma sound idle */
 	}
 }
 
@@ -803,8 +857,8 @@ void DmaSnd_FrameStartHigh_WriteByte(void)
 {
 	if(LOG_TRACE_LEVEL(TRACE_DMASND))
 	{
-                int FrameCycles, HblCounterVideo, LineCycles;
-                Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
 		LOG_TRACE_PRINT("DMA snd frame start high: 0x%02x at pos %d/%d video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
 			IoMem_ReadByte(0xff8903) , dma.frameCounterAddr - dma.frameStartAddr , dma.frameEndAddr - dma.frameStartAddr  ,
 			FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
@@ -818,8 +872,8 @@ void DmaSnd_FrameStartMed_WriteByte(void)
 {
 	if(LOG_TRACE_LEVEL(TRACE_DMASND))
 	{
-                int FrameCycles, HblCounterVideo, LineCycles;
-                Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
 		LOG_TRACE_PRINT("DMA snd frame start med: 0x%02x at pos %d/%d video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
 			IoMem_ReadByte(0xff8905) , dma.frameCounterAddr - dma.frameStartAddr , dma.frameEndAddr - dma.frameStartAddr  ,
 			FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
@@ -830,8 +884,8 @@ void DmaSnd_FrameStartLow_WriteByte(void)
 {
 	if(LOG_TRACE_LEVEL(TRACE_DMASND))
 	{
-                int FrameCycles, HblCounterVideo, LineCycles;
-                Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
  		LOG_TRACE_PRINT("DMA snd frame start low: 0x%02x at pos %d/%d video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
 			IoMem_ReadByte(0xff8907) , dma.frameCounterAddr - dma.frameStartAddr , dma.frameEndAddr - dma.frameStartAddr  ,
 			FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
@@ -845,8 +899,8 @@ void DmaSnd_FrameCountHigh_WriteByte(void)
 {
 	if(LOG_TRACE_LEVEL(TRACE_DMASND))
 	{
-                int FrameCycles, HblCounterVideo, LineCycles;
-                Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
  		LOG_TRACE_PRINT("DMA snd frame count high: 0x%02x at pos %d/%d video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
 			IoMem_ReadByte(0xff8909) , dma.frameCounterAddr - dma.frameStartAddr , dma.frameEndAddr - dma.frameStartAddr  ,
 			FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
@@ -860,8 +914,8 @@ void DmaSnd_FrameCountMed_WriteByte(void)
 {
 	if(LOG_TRACE_LEVEL(TRACE_DMASND))
 	{
-                int FrameCycles, HblCounterVideo, LineCycles;
-                Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
  		LOG_TRACE_PRINT("DMA snd frame count med: 0x%02x at pos %d/%d video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
 			IoMem_ReadByte(0xff890b) , dma.frameCounterAddr - dma.frameStartAddr , dma.frameEndAddr - dma.frameStartAddr  ,
 			FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
@@ -872,8 +926,8 @@ void DmaSnd_FrameCountLow_WriteByte(void)
 {
 	if(LOG_TRACE_LEVEL(TRACE_DMASND))
 	{
-                int FrameCycles, HblCounterVideo, LineCycles;
-                Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
  		LOG_TRACE_PRINT("DMA snd frame count low: 0x%02x at pos %d/%d video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
 			IoMem_ReadByte(0xff890d) , dma.frameCounterAddr - dma.frameStartAddr , dma.frameEndAddr - dma.frameStartAddr  ,
 			FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
@@ -884,8 +938,8 @@ void DmaSnd_FrameEndHigh_WriteByte(void)
 {
 	if(LOG_TRACE_LEVEL(TRACE_DMASND))
 	{
-                int FrameCycles, HblCounterVideo, LineCycles;
-                Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
  		LOG_TRACE_PRINT("DMA snd frame end high: 0x%02x at pos %d/%d video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
 			IoMem_ReadByte(0xff890f) , dma.frameCounterAddr - dma.frameStartAddr , dma.frameEndAddr - dma.frameStartAddr  ,
 			FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
@@ -899,8 +953,8 @@ void DmaSnd_FrameEndMed_WriteByte(void)
 {
 	if(LOG_TRACE_LEVEL(TRACE_DMASND))
 	{
-                int FrameCycles, HblCounterVideo, LineCycles;
-                Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
 		LOG_TRACE_PRINT("DMA snd frame end med: 0x%02x at pos %d/%d video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
 			IoMem_ReadByte(0xff8911) , dma.frameCounterAddr - dma.frameStartAddr , dma.frameEndAddr - dma.frameStartAddr  ,
 			FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
@@ -911,8 +965,8 @@ void DmaSnd_FrameEndLow_WriteByte(void)
 {
 	if(LOG_TRACE_LEVEL(TRACE_DMASND))
 	{
-                int FrameCycles, HblCounterVideo, LineCycles;
-                Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
 		LOG_TRACE_PRINT("DMA snd frame end low: 0x%02x at pos %d/%d video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
 			IoMem_ReadByte(0xff8913) , dma.frameCounterAddr - dma.frameStartAddr , dma.frameEndAddr - dma.frameStartAddr  ,
 			FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
@@ -933,8 +987,8 @@ void DmaSnd_SoundModeCtrl_ReadByte(void)
 
 	if(LOG_TRACE_LEVEL(TRACE_DMASND))
 	{
-                int FrameCycles, HblCounterVideo, LineCycles;
-                Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
 		LOG_TRACE_PRINT("DMA snd mode read: 0x%02x video_cyc=%d %d@%d pc=%x instr_cycle %d\n", dma.soundMode,
 			FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
 	}
@@ -947,14 +1001,14 @@ void DmaSnd_SoundModeCtrl_ReadByte(void)
  */
 void DmaSnd_SoundModeCtrl_WriteByte(void)
 {
-	Uint16	SoundModeNew;
+	uint16_t	SoundModeNew;
 
 	SoundModeNew = IoMem_ReadByte(0xff8921);
 
 	if(LOG_TRACE_LEVEL(TRACE_DMASND))
 	{
-                int FrameCycles, HblCounterVideo, LineCycles;
-                Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
 		LOG_TRACE_PRINT("DMA snd mode write: 0x%02x mode=%s freq=%d video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
 			SoundModeNew, SoundModeNew & DMASNDMODE_MONO ? "mono" : "stereo" , DmaSndSampleRates[ SoundModeNew & 3 ],
 			FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
@@ -984,7 +1038,7 @@ void DmaSnd_SoundModeCtrl_WriteByte(void)
 void DmaSnd_InterruptHandler_Microwire(void)
 {
 	int	i;
-	Uint16	cmd;
+	uint16_t	cmd;
 	int	cmd_len;
 
 	/* If emulated computer is the Falcon, let's the crossbar Microwire code do the job. */
@@ -995,11 +1049,8 @@ void DmaSnd_InterruptHandler_Microwire(void)
 	}
 	
 	/* How many cycle was this sound interrupt delayed (>= 0) */
-#ifdef OLD_CPU_SHIFT
 	microwire.pendingCyclesOver += -INT_CONVERT_FROM_INTERNAL ( PendingInterruptCount , INT_CPU_CYCLE );
-#else
-	microwire.pendingCyclesOver += -INT_CONVERT_FROM_INTERNAL ( PendingInterruptCount , INT_CPU8_CYCLE );
-#endif
+
 	/* Remove this interrupt from list and re-order */
 	CycInt_AcknowledgeInterrupt();
 
@@ -1022,11 +1073,7 @@ void DmaSnd_InterruptHandler_Microwire(void)
 	{
 		/* No ==> start a new internal interrupt to continue to transfer the data */
 		microwire.pendingCyclesOver = 8 - microwire.pendingCyclesOver;
-#ifdef OLD_CPU_SHIFT
-		CycInt_AddRelativeInterrupt(microwire.pendingCyclesOver, INT_CPU_CYCLE, INTERRUPT_DMASOUND_MICROWIRE);
-#else
 		CycInt_AddRelativeInterrupt(microwire.pendingCyclesOver, INT_CPU8_CYCLE, INTERRUPT_DMASOUND_MICROWIRE);
-#endif
 	}
 	else 
 	{
@@ -1103,20 +1150,20 @@ void DmaSnd_InterruptHandler_Microwire(void)
 				/* Master volume command */
 				LOG_TRACE ( TRACE_DMASND, "Microwire new master volume=0x%x\n", cmd & 0x3f );
 				microwire.masterVolume = LMC1992_Master_Volume_Table[ cmd & 0x3f ];
-				lmc1992.left_gain = (microwire.leftVolume * (Uint32)microwire.masterVolume) * (2.0/(65536.0*65536.0));
-				lmc1992.right_gain = (microwire.rightVolume * (Uint32)microwire.masterVolume) * (2.0/(65536.0*65536.0));
+				lmc1992.left_gain = (microwire.leftVolume * (uint32_t)microwire.masterVolume) * (2.0/(65536.0*65536.0));
+				lmc1992.right_gain = (microwire.rightVolume * (uint32_t)microwire.masterVolume) * (2.0/(65536.0*65536.0));
 				break;
 			case 4:
 				/* Right channel volume */
 				LOG_TRACE ( TRACE_DMASND, "Microwire new right volume=0x%x\n", cmd & 0x1f );
 				microwire.rightVolume = LMC1992_LeftRight_Volume_Table[ cmd & 0x1f ];
-				lmc1992.right_gain = (microwire.rightVolume * (Uint32)microwire.masterVolume) * (2.0/(65536.0*65536.0));
+				lmc1992.right_gain = (microwire.rightVolume * (uint32_t)microwire.masterVolume) * (2.0/(65536.0*65536.0));
 				break;
 			case 5:
 				/* Left channel volume */
 				LOG_TRACE ( TRACE_DMASND, "Microwire new left volume=0x%x\n", cmd & 0x1f );
 				microwire.leftVolume = LMC1992_LeftRight_Volume_Table[ cmd & 0x1f ];
-				lmc1992.left_gain = (microwire.leftVolume * (Uint32)microwire.masterVolume) * (2.0/(65536.0*65536.0));
+				lmc1992.left_gain = (microwire.leftVolume * (uint32_t)microwire.masterVolume) * (2.0/(65536.0*65536.0));
 				break;
 			default:
 				/* Do nothing */
@@ -1134,8 +1181,8 @@ void DmaSnd_MicrowireData_ReadWord(void)
 	/* Shifting is done in DmaSnd_InterruptHandler_Microwire! */
 	if(LOG_TRACE_LEVEL(TRACE_DMASND))
 	{
-                int FrameCycles, HblCounterVideo, LineCycles;
-                Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
 		LOG_TRACE_PRINT("Microwire data read: 0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
 			IoMem_ReadWord(0xff8922),
 			FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
@@ -1155,17 +1202,13 @@ void DmaSnd_MicrowireData_WriteWord(void)
 		/* Start shifting events to simulate a microwire transfer */
 		microwire.mwTransferSteps = 16;
 		microwire.pendingCyclesOver = 8;
-#ifdef OLD_CPU_SHIFT
-		CycInt_AddRelativeInterrupt(microwire.pendingCyclesOver, INT_CPU_CYCLE, INTERRUPT_DMASOUND_MICROWIRE);
-#else
 		CycInt_AddRelativeInterrupt(microwire.pendingCyclesOver, INT_CPU8_CYCLE, INTERRUPT_DMASOUND_MICROWIRE);
-#endif
 	}
 
 	if(LOG_TRACE_LEVEL(TRACE_DMASND))
 	{
-                int FrameCycles, HblCounterVideo, LineCycles;
-                Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
 		LOG_TRACE_PRINT("Microwire data write: 0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
 			IoMem_ReadWord(0xff8922),
 			FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
@@ -1181,8 +1224,8 @@ void DmaSnd_MicrowireMask_ReadWord(void)
 	/* Same as with data register, but mask is rotated, not shifted. */
 	if(LOG_TRACE_LEVEL(TRACE_DMASND))
 	{
-                int FrameCycles, HblCounterVideo, LineCycles;
-                Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
 		LOG_TRACE_PRINT("Microwire mask read: 0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
 			IoMem_ReadWord(0xff8924),
 			FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
@@ -1203,8 +1246,8 @@ void DmaSnd_MicrowireMask_WriteWord(void)
 
 	if(LOG_TRACE_LEVEL(TRACE_DMASND))
 	{
-                int FrameCycles, HblCounterVideo, LineCycles;
-                Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
 		LOG_TRACE_PRINT("Microwire mask write: 0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
 			IoMem_ReadWord(0xff8924),
 			FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
@@ -1270,10 +1313,10 @@ static float DmaSnd_IIRfilterR(float xn)
 /**
  * LowPass Filter Left
  */
-static Sint16 DmaSnd_LowPassFilterLeft(Sint16 in)
+static int16_t DmaSnd_LowPassFilterLeft(int16_t in)
 {
-	static	Sint16	lowPassFilter[2] = { 0, 0 };
-	static	Sint16	out = 0;
+	static	int16_t	lowPassFilter[2] = { 0, 0 };
+	static	int16_t	out = 0;
 
 	if (DmaSnd_LowPass)
 		out = lowPassFilter[0] + (lowPassFilter[1]<<1) + in;
@@ -1289,10 +1332,10 @@ static Sint16 DmaSnd_LowPassFilterLeft(Sint16 in)
 /**
  * LowPass Filter Right
  */
-static Sint16 DmaSnd_LowPassFilterRight(Sint16 in)
+static int16_t DmaSnd_LowPassFilterRight(int16_t in)
 {
-	static	Sint16	lowPassFilter[2] = { 0, 0 };
-	static	Sint16	out = 0;
+	static	int16_t	lowPassFilter[2] = { 0, 0 };
+	static	int16_t	out = 0;
 
 	if (DmaSnd_LowPass)
 		out = lowPassFilter[0] + (lowPassFilter[1]<<1) + in;
@@ -1328,7 +1371,7 @@ static struct first_order_s *DmaSnd_Bass_Shelf(float g, float fc, float Fs)
 	static struct first_order_s bass;
 	float  a1;
 
-	/* g, fc, Fs must be positve real numbers > 0.0 */
+	/* g, fc, Fs must be positive real numbers > 0.0 */
 	if (g < 1.0)
 		bass.a1 = a1 = (tanf(M_PI*fc/Fs) - g  ) / (tanf(M_PI*fc/Fs) + g  );
 	else
@@ -1349,7 +1392,7 @@ static struct first_order_s *DmaSnd_Treble_Shelf(float g, float fc, float Fs)
 	static struct first_order_s treb;
 	float  a1;
 
-	/* g, fc, Fs must be positve real numbers > 0.0 */
+	/* g, fc, Fs must be positive real numbers > 0.0 */
 	if (g < 1.0)
 		treb.a1 = a1 = (g*tanf(M_PI*fc/Fs) - 1.0) / (g*tanf(M_PI*fc/Fs) + 1.0);
 	else
@@ -1373,8 +1416,8 @@ void DmaSnd_Init_Bass_and_Treble_Tables(void)
 	float  dB_adjusted, dB, g, fc_bt, fc_tt, Fs;
 	int    n;
 
-	fc_bt = 118.2763;
-	fc_tt = 8438.756;
+	fc_bt = 118.2763f;
+	fc_tt = 8438.756f;
 	Fs = (float)nAudioFrequency;
 
 	if ((Fs < 8000.0) || (Fs > 96000.0))
@@ -1415,6 +1458,37 @@ void DmaSnd_Init_Bass_and_Treble_Tables(void)
 			      LMC1992_Bass_Treble_Table[microwire.treble & 0xf]);
 
 	/* Initialize IIR Filter Gain and use as a Volume Control */
-	lmc1992.left_gain = (microwire.leftVolume * (Uint32)microwire.masterVolume) * (2.0/(65536.0*65536.0));
-	lmc1992.right_gain = (microwire.rightVolume * (Uint32)microwire.masterVolume) * (2.0/(65536.0*65536.0));
+	lmc1992.left_gain = (microwire.leftVolume * (uint32_t)microwire.masterVolume) * (2.0/(65536.0*65536.0));
+	lmc1992.right_gain = (microwire.rightVolume * (uint32_t)microwire.masterVolume) * (2.0/(65536.0*65536.0));
+}
+
+
+void DmaSnd_Info(FILE *fp, uint32_t dummy)
+{
+	if (Config_IsMachineST())
+	{
+		fprintf(fp, "ST doesn't include DMA!\n");
+		return;
+	}
+	fprintf(fp, "$FF8900.b : Sound DMA control  : %02x\n", IoMem_ReadByte(0xff8900));
+	fprintf(fp, "$FF8901.b : Sound DMA control  : %02x\n", IoMem_ReadByte(0xff8901));
+	fprintf(fp, "$FF8903.b : Frame Start High   : %02x\n", IoMem_ReadByte(0xff8903));
+	fprintf(fp, "$FF8905.b : Frame Start middle : %02x\n", IoMem_ReadByte(0xff8905));
+	fprintf(fp, "$FF8907.b : Frame Start low    : %02x\n", IoMem_ReadByte(0xff8907));
+	fprintf(fp, "$FF8909.b : Frame Count High   : %02x\n", IoMem_ReadByte(0xff8909));
+	fprintf(fp, "$FF890B.b : Frame Count middle : %02x\n", IoMem_ReadByte(0xff890b));
+	fprintf(fp, "$FF890D.b : Frame Count low    : %02x\n", IoMem_ReadByte(0xff890d));
+	fprintf(fp, "$FF890F.b : Frame End High     : %02x\n", IoMem_ReadByte(0xff890f));
+	fprintf(fp, "$FF8911.b : Frame End middle   : %02x\n", IoMem_ReadByte(0xff8911));
+	fprintf(fp, "$FF8913.b : Frame End low      : %02x\n", IoMem_ReadByte(0xff8913));
+	fprintf(fp, "\n");
+	fprintf(fp, "$FF8920.b : Sound Mode Control : %02x\n", IoMem_ReadByte(0xff8920));
+	fprintf(fp, "$FF8921.b : Sound Mode Control : %02x\n", IoMem_ReadByte(0xff8921));
+	if (Config_IsMachineFalcon())
+	{
+		return;
+	}
+	fprintf(fp, "\n");
+	fprintf(fp, "$FF8922.w : Microwire Data     : %04x\n", IoMem_ReadWord(0xff8922));
+	fprintf(fp, "$FF8924.w : Microwire Mask     : %04x\n", IoMem_ReadWord(0xff8924));
 }
