@@ -8,14 +8,7 @@
 #ifndef HATARI_MICROPHONE_H
 #define HATARI_MICROPHONE_H
 
-#if HAVE_PORTAUDIO
 extern bool Microphone_Start (int sampleRate);
-extern bool Microphone_Stop (void);
-#else
-/* replace function calls with NOPs (could also be empty static inlines) */
-#define Microphone_Start(rate) false
-#define Microphone_Stop() false
-#endif
-
+extern void Microphone_Stop (void);
 
 #endif /* HATARI_MICROPHONE_H */
