@@ -63,7 +63,7 @@
 	$FFFF98FC (long) : VDL_PAL - Videl palette Register 255
 */
 
-const char VIDEL_fileid[] = "Hatari videl.c : " __DATE__ " " __TIME__;
+const char VIDEL_fileid[] = "Hatari videl.c";
 
 #include "main.h"
 #include "configuration.h"
@@ -75,6 +75,7 @@ const char VIDEL_fileid[] = "Hatari videl.c : " __DATE__ " " __TIME__;
 #include "avi_record.h"
 #include "statusbar.h"
 #include "stMemory.h"
+#include "tos.h"
 #include "videl.h"
 #include "video.h"				/* for bUseHighRes variable */
 #include "vdi.h"				/* for bUseVDIRes variable */
@@ -106,6 +107,13 @@ struct videl_s {
 
 static struct videl_s videl;
 
+static void Videl_SetDefaultSavedRes(void)
+{
+	/* Default resolution to boot with */
+	videl.save_scrWidth = 640;
+	videl.save_scrHeight = 480;
+	videl.save_scrBpp = 4;
+}
 
 /**
  * Called upon startup (and via VIDEL_reset())
@@ -113,11 +121,7 @@ static struct videl_s videl;
 void Videl_Init(void)
 {
 	videl.hostColorsSync = false;
-
-	/* Default resolution to boot with */
-	videl.save_scrWidth = 640;
-	videl.save_scrHeight = 480;
-	videl.save_scrBpp = 4;
+	Videl_SetDefaultSavedRes();
 }
 
 /**
@@ -126,8 +130,7 @@ void Videl_Init(void)
 void VIDEL_reset(void)
 {
 	Videl_Init();
-	Screen_SetGenConvSize(videl.save_scrWidth, videl.save_scrHeight,
-	                      ConfigureParams.Screen.nForceBpp, false);
+	Screen_SetGenConvSize(videl.save_scrWidth, videl.save_scrHeight, false);
 
 	videl.bUseSTShifter = false;				/* Use Falcon color palette by default */
 	videl.reg_ffff8006_save = IoMem_ReadByte(0xff8006);
@@ -138,9 +141,6 @@ void VIDEL_reset(void)
 	/* Reset IO register (some are not initialized by TOS) */
 	IoMem_WriteWord(0xff820e, 0);    /* Line offset */
 	IoMem_WriteWord(0xff8264, 0);    /* Horizontal scroll */
-
-	/* Init sync mode register */
-	VIDEL_SyncMode_WriteByte();
 }
 
 /**
@@ -150,8 +150,106 @@ void VIDEL_MemorySnapShot_Capture(bool bSave)
 {
 	/* Save/Restore details */
 	MemorySnapShot_Store(&videl, sizeof(videl));
+
+	/* Make sure that the save_scr* variables match the ones during reset,
+	 * so that resolution changes get evaluated properly (e.g. to set the
+	 * right zooming variables */
+	if (!bSave)
+		Videl_SetDefaultSavedRes();
 }
 
+
+static bool Videl_GetPixelCyclesAndDivider(int *cyc_per_pixel, int *divider)
+{
+	Uint16 vdm = IoMem_ReadWord(0xff82c2) & 0xc;
+
+	/* Compute cycles per pixel */
+	if (vdm == 0)
+		*cyc_per_pixel = 4;
+	else if (vdm == 4)
+		*cyc_per_pixel = 2;
+	else
+		*cyc_per_pixel = 1;
+
+	/* Compute the divider */
+	if (videl.monitor_type == FALCON_MONITOR_VGA) {
+		if (vdm == 0)
+			*divider = 4;
+		else
+			*divider = 2;
+	}
+	else if (videl.bUseSTShifter == true) {
+		*divider = 16;
+	}
+	else {
+		*divider = *cyc_per_pixel;
+	}
+
+	return vdm != 0xc;	/* Return whether it is a valid setting */
+}
+
+
+/**
+ * Return the vertical refresh rate for the current video mode
+ * We use the following formula :
+ *   VFreq = ( HFreq / (VFT+1) ) * 2
+ * HFreq is 15625 Hz in RGB/TV mode or 31250 Hz in VGA mode (in VGA mode HFreq can take other values in the same range)
+ *
+ * Some VFT values set by TOS :
+ *  - 320x200 16 colors, RGB : VFT = 625	-> 50 Hz
+ *  - 320x200 16 colors, VGA : VFT = 1049	-> 60 Hz
+ */
+int VIDEL_Get_VFreq(void)
+{
+	static int prev_hfreq, prev_vfreq;
+	int baseclock, hfreq, vfreq;
+	int cyc_per_pixel, divider;
+	int hht, vft;
+
+	Videl_GetPixelCyclesAndDivider(&cyc_per_pixel, &divider);
+
+	if ( IoMem_ReadWord(0xff82c0) & 4 )		/* VC0 : bit2=0 32 MHz   bit2=1 25 MHz */
+	{
+		baseclock = 25175000;
+	}
+	else
+	{
+		/* 32.084988 MHz for PAL and 32.215905 for NTSC systems? */
+		baseclock = 32084988;
+	}
+
+	vft = IoMem_ReadWord(0xff82a2);
+	if (vft < 2)			/* Avoid bad value */
+		return 60;
+
+	hht = IoMem_ReadWord(0xff8282);
+	hfreq = baseclock / (2 * divider * (hht + 2));
+	vfreq = round(2.0 * hfreq / (vft - !(IoMem_ReadWord(0xff82C2) & 2)));
+
+	if (prev_hfreq != hfreq || prev_vfreq != vfreq)
+	{
+		Log_Printf(LOG_DEBUG, "Videl refresh rate changed: "
+		                      "baseclock=%i hfreq=%i vfreq=%i\n",
+		           baseclock, hfreq, vfreq);
+		prev_hfreq = hfreq;
+		prev_vfreq = vfreq;
+	}
+
+	return vfreq;
+}
+
+
+/**
+ * Return the content of videl.bUseSTShifter.
+ * This tells if the current video mode is compatible with ST/STE
+ * video mode or not
+ */
+bool VIDEL_Use_STShifter(void)
+{
+	return videl.bUseSTShifter;
+}
+
+
 /**
  * Monitor write access to Falcon color palette registers
  */
@@ -176,22 +274,19 @@ void VIDEL_Monitor_WriteByte(void)
 }
 
 /**
- * VIDEL_SyncMode_WriteByte : Videl synchronization mode.
- *             $FFFF820A [R/W] _______0  .................................. SYNC-MODE
-                                     ||
-                                     |+--Synchronisation [ 0:internal / 1:external ]
-                                     +---Vertical frequency [ Read-only bit ]
-                                         [ Monochrome monitor:0 / Colour monitor:1 ]
+ * VIDEL_SyncMode_WriteByte:
+ * Videl synchronization mode. Bit 1 is used by TOS 4.04 to set either 50 Hz
+ * (bit set) or 60 Hz (bit cleared).
+ * Note: There are documentation files out there that claim that bit 1 is
+ * used to distinguish between monochrome or color monitor, but these are
+ * definitely wrong.
  */
 void VIDEL_SyncMode_WriteByte(void)
 {
 	Uint8 syncMode = IoMem_ReadByte(0xff820a);
 	LOG_TRACE(TRACE_VIDEL, "Videl : $ff820a Sync Mode write: 0x%02x\n", syncMode);
 
-	if (videl.monitor_type == FALCON_MONITOR_MONO)
-		syncMode &= 0xfd;
-	else
-		syncMode |= 0x2;
+	syncMode &= 0x03;	/* Upper bits are hard-wired to 0 */
 
 	IoMem_WriteByte(0xff820a, syncMode);
 }
@@ -342,6 +437,32 @@ void VIDEL_ST_ShiftModeWriteByte(void)
 
 	/* Set video mode ($FFFF82C2) */
 	IoMem_WriteWord(0xff82c2, video_mode);
+
+	/* Hack for Sparrow-TOS (which does not know about Videl registers): */
+	if (TosVersion == 0x207)
+	{
+		if (st_shiftMode == 2)  /* Mono? */
+		{
+			IoMem_WriteWord(0xff82a4, 0);
+			IoMem_WriteWord(0xff82a6, 0);
+			IoMem_WriteWord(0xff82a8, 0x43);
+			IoMem_WriteWord(0xff82aa, 0x363);
+		}
+		else if (ConfigureParams.Screen.nMonitorType == MONITOR_TYPE_VGA)
+		{
+			IoMem_WriteWord(0xff82a4, 0x3af);
+			IoMem_WriteWord(0xff82a6, 0x8f);
+			IoMem_WriteWord(0xff82a8, 0x8f);
+			IoMem_WriteWord(0xff82aa, 0x3af);
+		}
+		else
+		{
+			IoMem_WriteWord(0xff82a4, 0x20e);
+			IoMem_WriteWord(0xff82a6, 0x7e);
+			IoMem_WriteWord(0xff82a8, 0x7e);
+			IoMem_WriteWord(0xff82aa, 0x20e);
+		}
+	}
 }
 
 /**
@@ -565,27 +686,12 @@ void VIDEL_VMD_WriteWord(void)
 }
 
 
-/**
- *  VIDEL_getVideoramAddress: returns the video RAM address.
- *  On Falcon, video address must be a multiple of four in bitplane modes.
- */
-static Uint32 VIDEL_getVideoramAddress(void)
-{
-	Uint32 videoBase;
-
-	videoBase  = (Uint32) IoMem_ReadByte(0xff8201) << 16;
-	videoBase |= (Uint32) IoMem_ReadByte(0xff8203) << 8;
-	videoBase |= IoMem_ReadByte(0xff820d) & ~3;
-
-	return videoBase;
-}
-
 /**
  * Reset appropriate registers on VBL etc
  */
 void VIDEL_RestartVideoCounter(void)
 {
-	videl.videoRaster = VIDEL_getVideoramAddress();
+	videl.videoRaster = Video_GetScreenBaseAddr();
 	/* counter for VFC register $ff82a0 */
 	videl.vertFreqCounter = 0;
 }
@@ -648,11 +754,11 @@ static Uint16 VIDEL_getScreenBpp(void)
  */
 static int VIDEL_getScreenWidth(void)
 {
-	Uint16 hbb, hbe, hdb, hde, vdm, hht;
-	Uint16 cycPerPixel, divider;
+	Uint16 hbb, hbe, hdb, hde, hht;
 	Sint16 hdb_offset, hde_offset;
 	Sint16 leftBorder, rightBorder;
 	Uint16 bpp = VIDEL_getScreenBpp();
+	int cycPerPixel, divider;
 
 	/* X Size of the Display area */
 	videl.XSize = (IoMem_ReadWord(0xff8210) & 0x03ff) * 16 / bpp;
@@ -684,30 +790,9 @@ static int VIDEL_getScreenWidth(void)
 	hbe = IoMem_ReadWord(0xff8286) & 0x01ff;
 	hdb = IoMem_ReadWord(0xff8288) & 0x01ff;
 	hde = IoMem_ReadWord(0xff828a) & 0x01ff;
-	vdm = IoMem_ReadWord(0xff82c2) & 0xc;
 	hht = IoMem_ReadWord(0xff8282) & 0x1ff;
 
-	/* Compute cycles per pixel */
-	if (vdm == 0)
-		cycPerPixel = 4;
-	else if (vdm == 4)
-		cycPerPixel = 2;
-	else
-		cycPerPixel = 1;
-
-	/* Compute the divider */
-	if (videl.monitor_type == FALCON_MONITOR_VGA) {
-		if (cycPerPixel == 4)
-			divider = 4;
-		else
-			divider = 2;
-	}
-	else if (videl.bUseSTShifter == true) {
-		divider = 16;
-	}
-	else {
-		divider = cycPerPixel;
-	}
+	Videl_GetPixelCyclesAndDivider(&cycPerPixel, &divider);
 
 	/* Compute hdb_offset and hde_offset */
 	if (videl.bUseSTShifter == false) {
@@ -875,28 +960,10 @@ void VIDEL_UpdateColors(void)
 
 void Videl_ScreenModeChanged(bool bForceChange)
 {
-	int bpp;
-
-	if (ConfigureParams.Screen.nForceBpp)
-	{
-		bpp = ConfigureParams.Screen.nForceBpp;
-	}
-	else if (Avi_AreWeRecording())
-	{
-		/* Avoid changing the bpp if we are recording */
-		bpp = sdlscrn->format->BitsPerPixel;
-	}
-	else
-	{
-		/* Using SDL's 16 bpp conversion function is a bit faster */
-		bpp = (videl.save_scrBpp == 16) ? 16 : 0;
-	}
-
 	LOG_TRACE(TRACE_VIDEL, "Videl : video mode change to %dx%d@%d\n",
 	          videl.save_scrWidth, videl.save_scrHeight, videl.save_scrBpp);
 
-	Screen_SetGenConvSize(videl.save_scrWidth, videl.save_scrHeight,
-	                      bpp, bForceChange);
+	Screen_SetGenConvSize(videl.save_scrWidth, videl.save_scrHeight, bForceChange);
 }
 
 
@@ -914,7 +981,7 @@ bool VIDEL_renderScreen(void)
 
 	bool change = false;
 
-	Uint32 videoBase = VIDEL_getVideoramAddress();
+	Uint32 videoBase = Video_GetScreenBaseAddr();
 
 	if (vw > 0 && vw != videl.save_scrWidth) {
 		LOG_TRACE(TRACE_VIDEL, "Videl : width change from %d to %d\n", videl.save_scrWidth, vw);
@@ -963,7 +1030,7 @@ bool VIDEL_renderScreen(void)
 
 	VIDEL_UpdateColors();
 
-	Screen_GenConvert(&STRam[videoBase], videl.XSize, videl.YSize,
+	Screen_GenConvert(videoBase, &STRam[videoBase], videl.XSize, videl.YSize,
 	                  videl.save_scrBpp, nextline, hscrolloffset,
 	                  videl.leftBorderSize, videl.rightBorderSize,
 	                  videl.upperBorderSize, videl.lowerBorderSize);
@@ -1105,7 +1172,7 @@ void Videl_Color15_WriteWord(void)
 /**
  * display Videl registers values (for debugger info command)
  */
-void Videl_Info(FILE *fp, Uint32 dummy)
+void Videl_Info(FILE *fp, uint32_t dummy)
 {
 	if (ConfigureParams.System.nMachineType != MACHINE_FALCON) {
 		fprintf(fp, "Not Falcon - no Videl!\n");
@@ -1156,4 +1223,8 @@ void Videl_Info(FILE *fp, Uint32 dummy)
 		(IoMem_ReadByte(0xff8205)<<16) +
 		(IoMem_ReadByte(0xff8207)<<8)  +
 		 IoMem_ReadByte(0xff8209));
+
+	fprintf(fp, "Palette type: %s\n", videl.bUseSTShifter ?
+		"ST/STE compat ($FF8240)" :
+		"Falcon ($FF9800)");
 }
