@@ -26,11 +26,14 @@
 #define	FLOPPY_IMAGE_TYPE_IPF			4		/* handled by capsimage library */
 #define	FLOPPY_IMAGE_TYPE_STX			5
 
+#define	FLOPPY_BOOT_SECTOR_EXE_SUM		0x1234
+
+
 /* Structure for each drive connected as emulation */
 typedef struct
 {
 	int ImageType;
-	Uint8 *pBuffer;
+	uint8_t *pBuffer;
 	char sFileName[FILENAME_MAX];
 	int nImageBytes;
 	bool bDiskInserted;
@@ -59,8 +62,8 @@ extern const char* Floppy_SetDiskFileName(int Drive, const char *pszFileName, co
 extern int Floppy_DriveTransitionUpdateState ( int Drive );
 extern bool Floppy_InsertDiskIntoDrive(int Drive);
 extern bool Floppy_EjectDiskFromDrive(int Drive);
-extern void Floppy_FindDiskDetails(const Uint8 *pBuffer, int nImageBytes, Uint16 *pnSectorsPerTrack, Uint16 *pnSides);
-extern bool Floppy_ReadSectors(int Drive, Uint8 **pBuffer, Uint16 Sector, Uint16 Track, Uint16 Side, short Count, int *pnSectorsPerTrack, int *pSectorSize);
-extern bool Floppy_WriteSectors(int Drive, Uint8 *pBuffer, Uint16 Sector, Uint16 Track, Uint16 Side, short Count, int *pnSectorsPerTrack, int *pSectorSize);
+extern void Floppy_FindDiskDetails(const uint8_t *pBuffer, int nImageBytes, uint16_t *pnSectorsPerTrack, uint16_t *pnSides);
+extern bool Floppy_ReadSectors(int Drive, uint8_t **pBuffer, uint16_t Sector, uint16_t Track, uint16_t Side, short Count, int *pnSectorsPerTrack, int *pSectorSize);
+extern bool Floppy_WriteSectors(int Drive, uint8_t *pBuffer, uint16_t Sector, uint16_t Track, uint16_t Side, short Count, int *pnSectorsPerTrack, int *pSectorSize);
 
 #endif
