@@ -10,31 +10,29 @@
 
 #include "config.h"
 
-#if ENABLE_SMALL_MEM
-# include "sysdeps.h"
-# include "maccess.h"
-# include "main.h"
+#include "sysdeps.h"
+#include "maccess.h"
+#include "main.h"
 extern uae_u8 *IOmemory;
-# define IoMem (IOmemory-0xff0000)
-#else
-# include "stMemory.h"
-# define IoMem STRam
-#endif  /* ENABLE_SMALL_MEM */
-
+#define IoMem (IOmemory-0xff0000)
 
 extern int nIoMemAccessSize;
-extern Uint32 IoAccessFullAddress;
-extern Uint32 IoAccessBaseAddress;
-extern Uint32 IoAccessCurrentAddress;
+extern uint32_t IoAccessFullAddress;
+extern uint32_t IoAccessBaseAddress;
+extern uint32_t IoAccessCurrentAddress;
 
 extern int	IoAccessInstrCount;
 
+enum FALCON_BUS_MODE {
+	STE_BUS_COMPATIBLE,
+	FALCON_ONLY_BUS
+};
 
 /**
  * Read 32-bit word from IO memory space without interception.
  * NOTE - value will be converted to PC endian.
  */
-static inline Uint32 IoMem_ReadLong(Uint32 Address)
+static inline uint32_t IoMem_ReadLong(uint32_t Address)
 {
 	Address &= 0x0ffffff;
 	return do_get_mem_long(&IoMem[Address]);
@@ -45,7 +43,7 @@ static inline Uint32 IoMem_ReadLong(Uint32 Address)
  * Read 16-bit word from IO memory space without interception.
  * NOTE - value will be converted to PC endian.
  */
-static inline Uint16 IoMem_ReadWord(Uint32 Address)
+static inline uint16_t IoMem_ReadWord(uint32_t Address)
 {
 	Address &= 0x0ffffff;
 	return do_get_mem_word(&IoMem[Address]);
@@ -55,7 +53,7 @@ static inline Uint16 IoMem_ReadWord(Uint32 Address)
 /**
  * Read 8-bit byte from IO memory space without interception.
  */
-static inline Uint8 IoMem_ReadByte(Uint32 Address)
+static inline uint8_t IoMem_ReadByte(uint32_t Address)
 {
 	Address &= 0x0ffffff;
  	return IoMem[Address];
@@ -66,7 +64,7 @@ static inline Uint8 IoMem_ReadByte(Uint32 Address)
  * Write 32-bit word into IO memory space without interception.
  * NOTE - value will be convert to 68000 endian
  */
-static inline void IoMem_WriteLong(Uint32 Address, Uint32 Var)
+static inline void IoMem_WriteLong(uint32_t Address, uint32_t Var)
 {
 	Address &= 0x0ffffff;
 	do_put_mem_long(&IoMem[Address], Var);
@@ -77,7 +75,7 @@ static inline void IoMem_WriteLong(Uint32 Address, Uint32 Var)
  * Write 16-bit word into IO memory space without interception.
  * NOTE - value will be convert to 68000 endian.
  */
-static inline void IoMem_WriteWord(Uint32 Address, Uint16 Var)
+static inline void IoMem_WriteWord(uint32_t Address, uint16_t Var)
 {
 	Address &= 0xffffff;
 	do_put_mem_word(&IoMem[Address], Var);
@@ -87,17 +85,23 @@ static inline void IoMem_WriteWord(Uint32 Address, Uint16 Var)
 /**
  * Write 8-bit byte into IO memory space without interception.
  */
-static inline void IoMem_WriteByte(Uint32 Address, Uint8 Var)
+static inline void IoMem_WriteByte(uint32_t Address, uint8_t Var)
 {
 	Address &= 0x0ffffff;
 	IoMem[Address] = Var;
 }
 
+extern void IoMem_Intercept ( uint32_t addr , void (*read_f)(void) , void (*write_f)(void) );
 
 extern void IoMem_Init(void);
-extern void IoMem_UnInit(void);
-extern void IoMem_Init_FalconInSTeBuscompatibilityMode(Uint8 value);
+extern void IoMem_UnInit (int MachineType);
+extern void IoMem_Reset(void);
 
+extern uint8_t IoMemTabMegaSTE_DIPSwitches_Read(void);
+
+extern uint8_t IoMemTabFalcon_DIPSwitches_Read(void);
+extern void IoMem_SetFalconBusMode(enum FALCON_BUS_MODE mode);
+extern bool IoMem_IsFalconBusMode(void);
 
 extern uae_u32 REGPARAM3 IoMem_bget(uaecptr addr);
 extern uae_u32 REGPARAM3 IoMem_wget(uaecptr addr);
@@ -107,6 +111,7 @@ extern void REGPARAM3 IoMem_bput(uaecptr addr, uae_u32 val);
 extern void REGPARAM3 IoMem_wput(uaecptr addr, uae_u32 val);
 extern void REGPARAM3 IoMem_lput(uaecptr addr, uae_u32 val);
 
+extern bool IoMem_CheckBusError ( uint32_t addr );
 extern void IoMem_BusErrorEvenReadAccess(void);
 extern void IoMem_BusErrorOddReadAccess(void);
 extern void IoMem_BusErrorEvenWriteAccess(void);
