@@ -14,14 +14,16 @@
 
   Clock cycles per line (50Hz)      : 512
   NOPs per scan line (50Hz)         : 128
-  Scan lines per VBL (50Hz)         : 313 (64 at top,200 screen,49 bottom)
+  Scan lines per VBL (50Hz)         : 313 (63 at top,200 screen,47 bottom)
 
   Clock cycles per line (60Hz)      : 508
   NOPs per scan line (60Hz)         : 127
-  Scan lines per VBL (60Hz)         : 263
+  Scan lines per VBL (60Hz)         : 263 (34 at top,200 screen,26 bottom)
 
   Clock cycles per VBL (50Hz)       : 160256
   NOPs per VBL (50Hz)               : 40064
+  Clock cycles per VBL (60Hz)       : 133604
+  NOPs per VBL (50Hz)               : 33401
 
   Pixels per clock cycle (low res)  : 1
   Pixels per clock cycle (med res)  : 2
@@ -58,9 +60,12 @@
 #define VIDEO_HEIGHT_HBL_COLOR  200	/* This is usually the height of the screen */
 #define VIDEO_HEIGHT_HBL_MONO   400
 
-#define VIDEO_HEIGHT_BOTTOM_50HZ 45	/* number of extra pixel lines in a 50 Hz screen when removing bottom border */
-					/* (2 extra lines can be displayed by doing a 2nd 60/50 Hz switch at the end of the removed bottom border) */
+#define VIDEO_HEIGHT_BOTTOM_50HZ 47	/* number of extra pixel lines in a 50 Hz screen when removing bottom border */
+					/* (the 2 last lines are masked by the vblank signal and can be displayed */
+					/* by doing a 2nd 60/50 Hz switch at the end of line 308) */
 #define VIDEO_HEIGHT_BOTTOM_60HZ 26	/* number of extra pixel lines in a 60 Hz screen when removing bottom border */
+					/* (the 2 last lines are masked by the vblank signal and can be displayed */
+					/* by doing a 2nd 50/60 Hz switch at the end of line 258) */
 
 #define VIDEO_END_HBL_50HZ	( VIDEO_START_HBL_50HZ + VIDEO_HEIGHT_HBL_COLOR )	/* 263 */
 #define VIDEO_END_HBL_60HZ	( VIDEO_START_HBL_60HZ + VIDEO_HEIGHT_HBL_COLOR )	/* 234 */
@@ -124,6 +129,15 @@
 #define RESTART_VIDEO_COUNTER_CYCLE_STF		( 56 )
 #define RESTART_VIDEO_COUNTER_CYCLE_STE		( 56 + 4 )	/* 4 cycles later than STF */
 
+
+/* Values for some HW video signals, should be 0 or 1 */
+#define	VSYNC_SIGNAL_OFF		0
+#define	VSYNC_SIGNAL_ON			1
+#define	VBLANK_SIGNAL_OFF		0
+#define	VBLANK_SIGNAL_ON		1
+
+
+
 /* anything above 4 uses automatic frameskip */
 #define AUTO_FRAMESKIP_LIMIT	5
 
@@ -136,11 +150,7 @@ extern int nHBL;
 extern int nStartHBL;
 extern int nEndHBL;
 extern int VerticalOverscan;
-extern Uint16 HBLPalettes[HBL_PALETTE_LINES];
-extern Uint16 *pHBLPalettes;
-extern Uint32 HBLPaletteMasks[HBL_PALETTE_MASKS];
-extern Uint32 *pHBLPaletteMasks;
-extern Uint32 VideoBase;
+extern uint32_t VideoBase;
 extern int nScreenRefreshRate;
 
 extern int nScanlinesPerFrame;
@@ -172,7 +182,12 @@ extern void	Video_InitTimings(void);
 extern void	Video_SetTimings( MACHINETYPE MachineType , VIDEOTIMINGMODE Mode );
 extern const char* Video_GetTimings_Name ( void );
 
+extern uint8_t	Video_Get_MONO_Line ( void );
+
 extern void	Video_ConvertPosition( int FrameCycles , int *pHBL , int *pLineCycles );
+extern int	Video_GetCyclesSinceVbl ( void );
+extern int	Video_GetCyclesSinceVbl_OnWriteAccess ( void );
+extern int	Video_GetCyclesSinceVbl_OnReadAccess ( void );
 extern void	Video_GetPosition( int *pFrameCycles , int *pHBL , int *pLineCycles );
 extern void	Video_GetPosition_OnWriteAccess( int *pFrameCycles , int *pHBL , int *pLineCycles );
 extern void	Video_GetPosition_OnReadAccess( int *pFrameCycles , int *pHBL , int *pLineCycles );
@@ -184,6 +199,8 @@ extern int	Video_TimerB_GetPos( int LineNumber );
 extern void	Video_InterruptHandler_HBL(void);
 extern void	Video_InterruptHandler_EndLine(void);
 
+extern void	Video_Set_Memcpy ( bool Force_MMU_Translation );
+
 extern void	Video_SetScreenRasters(void);
 extern void	Video_GetTTRes(int *width, int *height, int *bpp);
 extern bool	Video_RenderTTScreen(void);
@@ -193,13 +210,15 @@ extern void	Video_AddInterruptTimerB ( int LineVideo , int CycleVideo , int Pos
 extern void	Video_StartInterrupts ( int PendingCyclesOver );
 extern void	Video_InterruptHandler_VBL(void);
 
+extern uint32_t Video_GetScreenBaseAddr(void);
 extern void Video_ScreenBase_WriteByte(void);
 extern void Video_ScreenCounter_ReadByte(void);
 extern void Video_ScreenCounter_WriteByte(void);
 extern void Video_Sync_ReadByte(void);
 extern void Video_BaseLow_ReadByte(void);
 extern void Video_LineWidth_ReadByte(void);
-extern void Video_Res_ReadByte(void);
+extern void Video_ResGlueShifter_ReadByte(void);
+extern void Video_ResShifter_ReadByte(void);
 extern void Video_HorScroll_Read(void);
 extern void Video_LineWidth_WriteByte(void);
 extern void Video_Color0_WriteWord(void);
@@ -234,7 +253,8 @@ extern void Video_Color12_ReadWord(void);
 extern void Video_Color13_ReadWord(void);
 extern void Video_Color14_ReadWord(void);
 extern void Video_Color15_ReadWord(void);
-extern void Video_Res_WriteByte(void);
+extern void Video_ResGlueShifter_WriteByte(void);
+extern void Video_ResShifter_WriteByte(void);
 extern void Video_HorScroll_Read_8264(void);
 extern void Video_HorScroll_Read_8265(void);
 extern void Video_HorScroll_Write_8264(void);
@@ -244,6 +264,6 @@ extern void Video_TTShiftMode_WriteWord(void);
 extern void Video_TTColorRegs_Write(void);
 extern void Video_TTColorRegs_STRegWrite(void);
 
-extern void Video_Info(FILE *fp, Uint32 dummy);
+extern void Video_Info(FILE *fp, uint32_t dummy);
 
 #endif  /* HATARI_VIDEO_H */
