@@ -9,7 +9,7 @@
 #define HATARI_XBIOS_H
 
 extern bool XBios(void);
-extern void XBios_Info(FILE *fp, Uint32 dummy);
-extern void XBios_ToggleCommands(void);
+extern void XBios_Info(FILE *fp, uint32_t dummy);
+extern void XBios_EnableCommands(bool enable);
 
 #endif
