@@ -6,11 +6,11 @@
 
   TOS *.INF file overloading for autostarting & TOS resolution overriding.
 */
-const char INFFILE_fileid[] = "Hatari inffile.c : " __DATE__ " " __TIME__;
+const char INFFILE_fileid[] = "Hatari inffile.c";
 
-#include <SDL_endian.h>
 #include <assert.h>
 #include <errno.h>
+#include <ctype.h>
 
 #include "main.h"
 #include "configuration.h"
@@ -20,6 +20,7 @@ const char INFFILE_fileid[] = "Hatari inffile.c : " __DATE__ " " __TIME__;
 #include "file.h"
 #include "log.h"
 #include "str.h"
+#include "screen.h"
 #include "tos.h"
 #include "vdi.h"
 
@@ -41,6 +42,9 @@ typedef enum {
 	RES_COUNT
 } res_value_t;
 
+/* map values 0-6 to EmuTOS: N/A, ST low, med, high, TT med, high, low */
+static const res_value_t emutos_map[] = { 0, 0, 1, 2, 4, 6, 7 };
+
 
 static struct {
 	FILE *file;          /* file pointer to contents of INF file */
@@ -58,10 +62,15 @@ static struct {
  * '@' character in the INF files #Z line
  * (first value is 00: TOS, 01: GEM).
  *
- * Resolution is specified in the 2nd hex value in #E line
- * in normal TOS and EmuTOS <= 0.9.6, and in 4th hex value
- * in EmuTOS >= 0.9.7.  Hatari supports only latter EmuTOS
- * versions.
+ * #E line content differs between TOS versions:
+ * + Atari TOS:
+ *   - Resolution is specified in the 2nd hex value
+ *   - Blitter enabling is 0x10 bit for that
+ * + EmuTOS v0.9.7 or newer:
+ *   - Resolution is specified in the 4th hex value
+ *   - Blitter enabling is 0x80 bit in the 2nd hex value
+ * + Older EmuTOS versions (not supported!):
+ *   - Resolution is in the 2nd hex value
  *
  * TOS versions expect both of these to be within certain
  * number of bytes from the beginning of the file, and there
@@ -69,12 +78,15 @@ static struct {
  *
  * More documentation on the DESKTOP.INF file content:
  * http://st-news.com/issues/st-news-volume-2-issue-6/education/the-desktopinf-file/
+ *
+ * EmuTOS INF file content is documented only in the sources:
+ * https://github.com/emutos/emutos/blob/master/desk/deskapp.c
  */
 
 /* EmuDesk INF file format and values differ from normal TOS */
 static const char emudesk_inf[] =
 "#R 01\r\n"
-"#E 1A 61 FF 00 00\r\n"
+"#E 1A E1 FF 00 00\r\n"
 "#W 00 00 02 08 26 0C 00 @\r\n"
 "#W 00 00 02 0A 26 0C 00 @\r\n"
 "#W 00 00 02 0D 26 0C 00 @\r\n"
@@ -151,6 +163,10 @@ static const char newdesk_inf[] =
 "#M 01 00 00 FF B FLOPPY DISK@ @ \r\n"
 "#T 00 03 02 FF   TRASH@ @ \r\n";
 
+/* TODO: when support for Falcon resolutions is added,
+ * builtin TOS v4 NEWDESK.INF file contents are needed too
+ */
+
 
 /*-----------------------------------------------------------------------*/
 /**
@@ -172,7 +188,7 @@ bool INF_SetAutoStart(const char *name, int opt_id)
 		/* full path */
 		const char *ptr;
 		int offset;
-		prgname = malloc(len+2);
+		prgname = Str_Alloc(len + 1);  /* +1 for additional backslash */
 		ptr = strrchr(name, '\\');
 		if (ptr)
 			offset = ptr - name + 1;
@@ -188,29 +204,31 @@ bool INF_SetAutoStart(const char *name, int opt_id)
 			/* NOT OK: A:DIR\NAME.PRG */
 			if (ptr)
 			{
+				Log_Printf(LOG_WARN, "rejecting auto-start path that doesn't have '\\' after drive ID:\n\t%s\n", name);
 				free(prgname);
 				return false;
 			}
 			/* A:NAME.PRG -> A:\NAME.PRG */
 			prgname[offset] = '\\';
 			/* copy/upcase file part */
-			Str_Filename2TOSname(name+offset, prgname+offset+1);
+			Str_Filename_Host2Atari(name+offset, prgname+offset+1);
 		} else {
 			/* copy/upcase file part */
-			Str_Filename2TOSname(name+offset, prgname+offset);
+			Str_Filename_Host2Atari(name+offset, prgname+offset);
 		}
 	}
 	else if (strchr(name, '\\'))
 	{
 		/* partial path not accepted */
+		Log_Printf(LOG_WARN, "rejecting auto-start path starting with '\\', but without drive ID:\n\t%s\n", name);
 		return false;
 	}
 	else
 	{
 		/* just program -> add path */
-		prgname = malloc(3 + len + 1);
+		prgname = Str_Alloc(3 + len);
 		strcpy(prgname, "C:\\");
-		Str_Filename2TOSname(name, prgname+3);
+		Str_Filename_Host2Atari(name, prgname+3);
 	}
 	if (TosOverride.prgname)
 		free(TosOverride.prgname);
@@ -286,8 +304,8 @@ int INF_ValidateAutoStart(const char **val, const char **err)
 	/* exact drive checking for hard drives would require:
 	 *
 	 * For images:
-	 * - finding out what partitions each of the IDE master &
-	 *   Slave, 8 ACSI, and 8 SCSI images do have, *and*
+	 * - finding out what partitions each of the 2 IDE, 8 ACSI, and
+	 *   8 SCSI images do have, *and*
 	 * - finding out which of those partitions the native Atari
 	 *   harddisk driver happens to support...
 	 * -> not feasible
@@ -305,11 +323,11 @@ int INF_ValidateAutoStart(const char **val, const char **err)
 		return 0;
 	}
 	/* IDE */
-	else if (ConfigureParams.HardDisk.bUseIdeMasterHardDiskImage && ConfigureParams.HardDisk.szIdeMasterHardDiskImage[0])
+	else if (ConfigureParams.Ide[0].bUseDevice && ConfigureParams.Ide[0].sDeviceFile[0])
 	{
 		return 0;
 	}
-	else if (ConfigureParams.HardDisk.bUseIdeMasterHardDiskImage && ConfigureParams.HardDisk.szIdeMasterHardDiskImage[0])
+	else if (ConfigureParams.Ide[1].bUseDevice && ConfigureParams.Ide[1].sDeviceFile[0])
 	{
 		return 0;
 	}
@@ -333,16 +351,39 @@ int INF_ValidateAutoStart(const char **val, const char **err)
 
 
 /**
- * Map / set VDI to INF file resolution
+ * Map VDI / HW resolution to INF file resolution value
  */
 static res_value_t vdi2inf(res_value_t mode)
 {
-	res_value_t res = TosOverride.reso;
-	res_value_t newres = mode + 1;
+	res_value_t newres, res = TosOverride.reso;
+
+	switch (mode)
+	{
+	case ST_LOW_RES:
+		newres = RES_ST_LOW;
+		break;
+	case ST_MEDIUM_RES:
+		newres = RES_ST_MED;
+		break;
+	case ST_HIGH_RES:
+		newres = RES_ST_HIGH;
+		break;
+	case TT_LOW_RES:
+		newres = RES_TT_LOW;
+		break;
+	case TT_MEDIUM_RES:
+		newres = RES_TT_MED;
+		break;
+	case TT_HIGH_RES:
+		newres = RES_TT_HIGH;
+		break;
+	default:
+		newres = res;
+	}
 	if (res != newres)
 	{
 		if (res)
-			Log_Printf(LOG_WARN, "Overriding TOS resolution %d with VDI resolution %d.\n",
+			Log_Printf(LOG_WARN, "Overriding TOS INF resolution %d with VDI resolution %d\n",
 				   res, newres);
 		res = newres;
 	}
@@ -376,6 +417,7 @@ extern void INF_SetVdiMode(int vdi_res)
  */
 static int INF_ValidateResolution(int *set_res, const char **val, const char **err)
 {
+#define MONO_WARN_STR "Correcting virtual INF file resolution to mono on mono monitor\n"
 	res_value_t res = TosOverride.reso;
 	*set_res = 0;
 
@@ -400,27 +442,33 @@ static int INF_ValidateResolution(int *set_res, const char **val, const char **e
 		case MACHINE_MEGA_STE:
 		case MACHINE_ST:
 		case MACHINE_MEGA_ST:
-			if (monitor == MONITOR_TYPE_MONO && res != RES_ST_HIGH)
+			if (monitor == MONITOR_TYPE_MONO)
 			{
-				res = RES_ST_HIGH;
-				Log_Printf(LOG_WARN, "With mono monitor, TOS can use only mono resolution, correcting.\n");
+				if (res != RES_ST_HIGH)
+				{
+					res = RES_ST_HIGH;
+					Log_Printf(LOG_WARN, MONO_WARN_STR);
+				}
 			}
 			else if (res >= RES_ST_HIGH)
 			{
-				*err = "invalid ST/STE color TOS resolution";
+				*err = "invalid TOS resolution for ST/STE color monitor";
 				return TosOverride.reso_id;
 			}
 			break;
 
 		case MACHINE_TT:
-			if (monitor == MONITOR_TYPE_MONO && res != RES_TT_HIGH)
+			if (monitor == MONITOR_TYPE_MONO)
 			{
-				res = RES_TT_HIGH;
-				Log_Printf(LOG_WARN, "With mono monitor, TOS can use only mono resolution, correcting.\n");
+				if (res != RES_TT_HIGH)
+				{
+					res = RES_TT_HIGH;
+					Log_Printf(LOG_WARN, MONO_WARN_STR);
+				}
 			}
 			else if (res == RES_TT_HIGH)
 			{
-				*err = "invalid TT color TOS resolution";
+				*err = "invalid TOS resolution for TT color monitor";
 				return TosOverride.reso_id;
 			}
 			break;
@@ -429,11 +477,11 @@ static int INF_ValidateResolution(int *set_res, const char **val, const char **e
 			if (monitor == MONITOR_TYPE_MONO && res != RES_ST_HIGH)
 			{
 				res = RES_ST_HIGH;
-				Log_Printf(LOG_WARN, "With mono monitor, TOS can use only mono resolution, correcting.\n");
+				Log_Printf(LOG_WARN, MONO_WARN_STR);
 			}
 			else if (res == RES_TT_HIGH)
 			{
-				*err = "TT-mono is invalid Falcon TOS resolution";
+				*err = "TT-mono is invalid TOS resolution for Falcon";
 				return TosOverride.reso_id;
 			}
 			else
@@ -443,9 +491,11 @@ static int INF_ValidateResolution(int *set_res, const char **val, const char **e
 			/* TODO:
 			 * Falcon resolution setting doesn't have effect,
 			 * seems that #E Falcon settings in columns 6 & 7
-			 * are also needed:
+			 * (5th & 6th hex values) are also needed:
 			 * - line doubling / interlace
 			 * - ST compat, RGB/VGA, columns & #colors
+			 * These should be same as for VsetMode:
+			 *   http://toshyp.atari.org/en/Screen_functions.html#Vsetmode
 			 */
 			break;
 		}
@@ -453,10 +503,8 @@ static int INF_ValidateResolution(int *set_res, const char **val, const char **e
 
 	if (bIsEmuTOS)
 	{
-		/* map values 0-6: N/A, ST low, med, high, TT med, high, low */
-		unsigned char map[] = { 0, 0, 1, 2, 4, 6, 7 };
-		res = map[res];
-		Log_Printf(LOG_INFO, "Remapped TOS resolution for EmuTOS.\n");
+		res = emutos_map[res];
+		Log_Printf(LOG_DEBUG, "Remapped INF file TOS resolution for EmuTOS\n");
 	}
 	else if (TosVersion >= 0x0160)
 	{
@@ -473,7 +521,7 @@ static int INF_ValidateResolution(int *set_res, const char **val, const char **e
 		}
 	}
 
-	Log_Printf(LOG_INFO, "Resulting INF file TOS resolution: 0x%02x -> 0x%02x.\n", TosOverride.reso, res);
+	Log_Printf(LOG_DEBUG, "Resulting INF file TOS resolution: 0x%02x -> 0x%02x.\n", TosOverride.reso, res);
 	*set_res = res;
 	return 0;
 }
@@ -484,7 +532,7 @@ static int INF_ValidateResolution(int *set_res, const char **val, const char **e
  * Get builtin INF file contents which open window for the boot drive,
  * if any.
  *
- * NOTE: this won't work for EmuTOS because it opens INF file second
+ * TODO: this won't work for EmuTOS because it opens INF file second
  * time to read window info, at which point the temporary virtual INF
  * file has already disappeared.  Real TOS versions read INF file
  * only once and work fine.
@@ -501,8 +549,7 @@ static char *get_builtin_inf(const char *contents)
 
 	inflen = strlen(contents);
 	winlen = strlen(drivewin);
-	inf = malloc(inflen + winlen + 1);
-	assert(inf);
+	inf = Str_Alloc(inflen + winlen);
 
 	/* drive letter offset on drive window line */
 	driveoffset = strchr(drivewin, 'X') - drivewin;
@@ -548,7 +595,7 @@ static char *get_inf_file(const char **set_infname, int *set_size, int *res_col)
 {
 	char *hostname;
 	const char *contents, *infname;
-	Uint8 *host_content;
+	uint8_t *host_content;
 	long host_size;
 	int size;
 
@@ -569,7 +616,7 @@ static char *get_inf_file(const char **set_infname, int *set_size, int *res_col)
 		*res_col = 12;
 	}
 	/* need to match file TOS searches first */
-	else if (TosVersion >= 0x0200)
+	else if (TosVersion >= 0x0200 && TosVersion != 0x300)
 	{
 		infname = "NEWDESK.INF";
 		size = sizeof(newdesk_inf);
@@ -589,14 +636,11 @@ static char *get_inf_file(const char **set_infname, int *set_size, int *res_col)
 	 */
 	if (!(ConfigureParams.HardDisk.bBootFromHardDisk && GemDOS_IsDriveEmulated(2)))
 	{
-#if INF_DEBUG
-	fprintf(stderr, "No GEMDOS HD boot drive, using builtin INF autostart file.\n");
-#endif
+		Log_Printf(LOG_DEBUG, "No GEMDOS HD boot drive, using builtin INF autostart file.\n");
 		return get_builtin_inf(contents);
 	}
 
-	hostname = malloc(FILENAME_MAX);
-	assert(hostname);
+	hostname = Str_Alloc(FILENAME_MAX);
 
 	/* convert to host file name, and read that */
 	GemDOS_CreateHardDriveFileName(2, infname, hostname, FILENAME_MAX);
@@ -604,16 +648,16 @@ static char *get_inf_file(const char **set_infname, int *set_size, int *res_col)
 	GemDOS_Info(stderr, 0);
 	fprintf(stderr, "\nChecking for existing INF file '%s' -> '%s'...\n", infname, hostname);
 #endif
-	host_content = File_Read(hostname, &host_size, NULL);
+	host_content = File_ReadAsIs(hostname, &host_size);
 
 	if (host_content)
 	{
-		Log_Printf(LOG_INFO, "Going to modify '%s'.\n", hostname);
+		Log_Printf(LOG_DEBUG, "Going to modify '%s'.\n", hostname);
 		free(hostname);
 		*set_size = host_size;
 		return (char *)host_content;
 	}
-	Log_Printf(LOG_INFO, "Using builtin '%s'.\n", infname);
+	Log_Printf(LOG_DEBUG, "Using builtin '%s'.\n", infname);
 	free(hostname);
 	return get_builtin_inf(contents);
 }
@@ -678,23 +722,17 @@ static FILE* write_inf_file(const char *contents, int size, int res, int res_col
 	int offset, off_prg, off_rez, endcheck;
 	FILE *fp;
 
-#if defined(WIN32)	/* unfortunately tmpfile() needs administrative privileges on windows, so this needs special care */
-	char *ptr = WinTmpFile();
-	if (ptr != NULL)
-		fp = fopen(ptr,"w+b");
-	else
-		fp = NULL;
-#else
-# if INF_DEBUG
+#if INF_DEBUG
 	{
+		/* insecure file path + leaving it behind for debugging */
 		const char *debugfile = "/tmp/hatari-desktop-inf.txt";
 		fprintf(stderr, "Virtual INF file: '%s'\n", debugfile);
 		fp = fopen(debugfile, "w+b");
 	}
-# else
-	fp = tmpfile();
-# endif
+#else
+	fp = File_OpenTempFile(NULL);
 #endif
+
 	prgname = TosOverride.prgname;
 	infname = TosOverride.infname;
 
@@ -772,13 +810,13 @@ static FILE* write_inf_file(const char *contents, int size, int res, int res_col
 	if (!(fwrite(contents+offset, size-offset-1, 1, fp) && fseek(fp, 0, SEEK_SET) == 0))
 	{
 		fclose(fp);
-		Log_Printf(LOG_ERROR, "Virtual '%s' file writing failed!\n", infname);
+		Log_Printf(LOG_ERROR, "Virtual '%s' INF file writing failed!\n", infname);
 		return NULL;
 	}
 	if (prgname)
-		Log_Printf(LOG_WARN, "Virtual '%s' autostart file created for '%s'.\n", infname, prgname);
+		Log_Printf(LOG_DEBUG, "Virtual '%s' autostart INF file created for '%s'\n", infname, prgname);
 	else
-		Log_Printf(LOG_WARN, "Virtual '%s' TOS resolution override file created.\n", infname);
+		Log_Printf(LOG_DEBUG, "Virtual '%s' TOS resolution override INF file created\n", infname);
 	return fp;
 }
 
@@ -813,7 +851,7 @@ void INF_CreateOverride(void)
 		return;
 
 	/* GEMDOS HD / INF overriding not supported? */
-	if (TosVersion < 0x0104)
+	if (bUseTos && TosVersion < 0x0104)
 	{
 		Log_Printf(LOG_WARN, "Only TOS versions >= 1.04 support autostarting & resolution overriding!\n");
 		return;
@@ -852,9 +890,9 @@ FILE *INF_OpenOverride(const char *filename)
 		if (ConfigureParams.Debugger.nExceptionDebugMask & EXCEPT_AUTOSTART)
 		{
 			ExceptionDebugMask = ConfigureParams.Debugger.nExceptionDebugMask & ~EXCEPT_AUTOSTART;
-			fprintf(stderr, "Exception debugging enabled (0x%x).\n", ExceptionDebugMask);
+			Log_Printf(LOG_INFO, "Exception debugging enabled (0x%x).\n", ExceptionDebugMask);
 		}
-		Log_Printf(LOG_WARN, "Virtual INF file '%s' matched.\n", filename);
+		Log_Printf(LOG_DEBUG, "Virtual INF file '%s' matched.\n", filename);
 		return TosOverride.file;
 	}
 	return NULL;
@@ -876,7 +914,7 @@ bool INF_CloseOverride(FILE *fp)
 		 */
 		fclose(TosOverride.file);
 		TosOverride.file = NULL;
-		Log_Printf(LOG_WARN, "Virtual INF file removed.\n");
+		Log_Printf(LOG_DEBUG, "Virtual INF file removed.\n");
 		return true;
 	}
 	return false;
