@@ -15,7 +15,7 @@
 	move.b $ff8264,d0  -> 82
 	move.b $ff8265,d0  -> 65
 */
-const char IoMemTabTT_fileid[] = "Hatari ioMemTabTT.c : " __DATE__ " " __TIME__;
+const char IoMemTabTT_fileid[] = "Hatari ioMemTabTT.c";
 
 #include "main.h"
 #include "configuration.h"
@@ -27,13 +27,14 @@ const char IoMemTabTT_fileid[] = "Hatari ioMemTabTT.c : " __DATE__ " " __TIME__;
 #include "joy.h"
 #include "mfp.h"
 #include "midi.h"
+#include "ncr5380.h"
 #include "nvram.h"
 #include "psg.h"
 #include "rs232.h"
 #include "rtc.h"
-#include "screen.h"
+#include "scc.h"
+#include "scu_vme.h"
 #include "video.h"
-#include "blitter.h"
 #include "stMemory.h"
 
 
@@ -44,7 +45,7 @@ const char IoMemTabTT_fileid[] = "Hatari ioMemTabTT.c : " __DATE__ " " __TIME__;
  *   1      off (on = CaTTamaran installed, not an official setting)
  *   2 - 6  off
  *   7      on = 1.4mb HD floppy drive fitted
- *   8      off (on = Disables the DMA sound hardware)
+ *   8      off (on = disable the DMA sound hardware)
  *
  * Switch 1 is represented by the lowest bit in the $FF9200 register,
  * and switch 8 is represented by the highest bit. Logic is inverted,
@@ -79,8 +80,8 @@ const INTERCEPT_ACCESS_FUNC IoMemTable_TT[] =
 	{ 0xff820d, SIZE_BYTE, Video_BaseLow_ReadByte, Video_ScreenBase_WriteByte },
 	{ 0xff820e, SIZE_BYTE, IoMem_VoidRead, IoMem_VoidWrite },                               /* No bus error here */
 	{ 0xff820f, SIZE_BYTE, IoMem_VoidRead, IoMem_VoidWrite },                               /* No bus error here */
-	{ 0xff8240, 16*SIZE_WORD, IoMem_ReadWithoutInterception, Video_TTColorRegs_STRegWrite },        /* 16 TT ST-palette entries */
-	{ 0xff8260, SIZE_BYTE, Video_Res_ReadByte, Video_Res_WriteByte },
+	{ 0xff8240, 16*SIZE_WORD, IoMem_ReadWithoutInterception, Video_TTColorRegs_STRegWrite },/* 16 TT ST-palette entries */
+	{ 0xff8260, SIZE_BYTE, Video_ResGlueShifter_ReadByte, Video_ResGlueShifter_WriteByte },	/* Resolution */
 	{ 0xff8261, SIZE_BYTE, IoMem_VoidRead_00, IoMem_VoidWrite },                            /* No bus errors here : return 0 not ff */
 	{ 0xff8262, SIZE_WORD, IoMem_ReadWithoutInterception, Video_TTShiftMode_WriteWord },    /* TT screen mode */
 	{ 0xff8264, SIZE_BYTE, IoMem_ReadWithoutInterception, IoMem_WriteWithoutInterception },
@@ -97,10 +98,30 @@ const INTERCEPT_ACCESS_FUNC IoMemTable_TT[] =
 	{ 0xff860b, SIZE_BYTE, FDC_DmaAddress_ReadByte, FDC_DmaAddress_WriteByte },		/* DMA base and counter med byte  */
 	{ 0xff860c, SIZE_BYTE, IoMem_VoidRead, IoMem_VoidWrite },                               /* No bus error here */
 	{ 0xff860d, SIZE_BYTE, FDC_DmaAddress_ReadByte, FDC_DmaAddress_WriteByte },		/* DMA base and counter low byte  */
-	{ 0xff860e, SIZE_BYTE, IoMem_VoidRead, IoMem_VoidWrite },                               /* No bus error here */
-	{ 0xff860f, SIZE_BYTE, IoMem_VoidRead, IoMem_VoidWrite },                               /* No bus error here */
+	{ 0xff860e, SIZE_WORD, FDC_DensityMode_ReadWord , FDC_DensityMode_WriteWord },		/* Choose DD/HD mode */
 
-	{ 0xff8780, 16, IoMem_VoidRead_00, IoMem_WriteWithoutInterception },                    /* TT SCSI controller */
+	{ 0xff8700, SIZE_BYTE, IoMem_VoidRead_00, IoMem_VoidWrite },                            /* No bus error here */
+	{ 0xff8701, SIZE_BYTE, IoMem_ReadWithoutInterception, IoMem_WriteWithoutInterception }, /* SCSI DMA Address Pointer (Highest byte) */
+	{ 0xff8702, SIZE_BYTE, IoMem_VoidRead_00, IoMem_VoidWrite },                            /* No bus error here */
+	{ 0xff8703, SIZE_BYTE, IoMem_ReadWithoutInterception, IoMem_WriteWithoutInterception }, /* SCSI DMA Address Pointer (High byte)    */
+	{ 0xff8704, SIZE_BYTE, IoMem_VoidRead_00, IoMem_VoidWrite },                            /* No bus error here */
+	{ 0xff8705, SIZE_BYTE, IoMem_ReadWithoutInterception, IoMem_WriteWithoutInterception }, /* SCSI DMA Address Pointer (Low byte)     */
+	{ 0xff8706, SIZE_BYTE, IoMem_VoidRead_00, IoMem_VoidWrite },                            /* No bus error here */
+	{ 0xff8707, SIZE_BYTE, IoMem_ReadWithoutInterception, IoMem_WriteWithoutInterception }, /* SCSI DMA Address Pointer (Lowest byte)  */
+	{ 0xff8708, SIZE_BYTE, IoMem_VoidRead_00, IoMem_VoidWrite },                            /* No bus error here */
+	{ 0xff8709, SIZE_BYTE, IoMem_ReadWithoutInterception, IoMem_WriteWithoutInterception }, /* SCSI DMA Byte Count (Highest byte)      */
+	{ 0xff870a, SIZE_BYTE, IoMem_VoidRead_00, IoMem_VoidWrite },                            /* No bus error here */
+	{ 0xff870b, SIZE_BYTE, IoMem_ReadWithoutInterception, IoMem_WriteWithoutInterception }, /* SCSI DMA Byte Count (High byte)         */
+	{ 0xff870c, SIZE_BYTE, IoMem_VoidRead_00, IoMem_VoidWrite },                            /* No bus error here */
+	{ 0xff870d, SIZE_BYTE, IoMem_ReadWithoutInterception, IoMem_WriteWithoutInterception }, /* SCSI DMA Byte Count (Low byte)          */
+	{ 0xff870e, SIZE_BYTE, IoMem_VoidRead_00, IoMem_VoidWrite },                            /* No bus error here */
+	{ 0xff870f, SIZE_BYTE, IoMem_ReadWithoutInterception, IoMem_WriteWithoutInterception }, /* SCSI DMA Byte Count (Lowest byte)       */
+	{ 0xff8710, SIZE_WORD, IoMem_ReadWithoutInterception, IoMem_WriteWithoutInterception }, /* SCSI Residue Data Register (High Word)  */
+	{ 0xff8712, SIZE_WORD, IoMem_ReadWithoutInterception, IoMem_WriteWithoutInterception }, /* SCSI Residue Data Register (Low Word)   */
+	{ 0xff8714, SIZE_WORD, IoMem_ReadWithoutInterception, Ncr5380_TT_DMA_Ctrl_WriteWord },  /* SCSI Control register                   */
+	{ 0xff8716, 10, IoMem_VoidRead_00, IoMem_VoidWrite },                                   /* No bus error here */
+
+	{ 0xff8780, 16, Ncr5380_IoMemTT_ReadByte, Ncr5380_IoMemTT_WriteByte },                  /* TT SCSI controller */
 
 	{ 0xff8800, SIZE_BYTE, PSG_ff8800_ReadByte, PSG_ff8800_WriteByte },
 	{ 0xff8802, SIZE_BYTE, PSG_ff880x_ReadByte, PSG_ff8802_WriteByte },
@@ -135,16 +156,38 @@ const INTERCEPT_ACCESS_FUNC IoMemTable_TT[] =
 
 	/* Note: The TT does not have a blitter (0xff8a00 - 0xff8a3e) */
 
-	//{ 0xff8c80, 8, IoMem_VoidRead, IoMem_WriteWithoutInterception },         /* SCC */
+	{ 0xff8c00, SIZE_BYTE, IoMem_VoidRead_00, IoMem_VoidWrite },                            /* No bus error here */
+	{ 0xff8c01, SIZE_BYTE, IoMem_ReadWithoutInterception, IoMem_WriteWithoutInterception }, /* SCC DMA Address Pointer (Highest byte) */
+	{ 0xff8c02, SIZE_BYTE, IoMem_VoidRead_00, IoMem_VoidWrite },                            /* No bus error here */
+	{ 0xff8c03, SIZE_BYTE, IoMem_ReadWithoutInterception, IoMem_WriteWithoutInterception }, /* SCC DMA Address Pointer (High byte)    */
+	{ 0xff8c04, SIZE_BYTE, IoMem_VoidRead_00, IoMem_VoidWrite },                            /* No bus error here */
+	{ 0xff8c05, SIZE_BYTE, IoMem_ReadWithoutInterception, IoMem_WriteWithoutInterception }, /* SCC DMA Address Pointer (Low byte)     */
+	{ 0xff8c06, SIZE_BYTE, IoMem_VoidRead_00, IoMem_VoidWrite },                            /* No bus error here */
+	{ 0xff8c07, SIZE_BYTE, IoMem_ReadWithoutInterception, IoMem_WriteWithoutInterception }, /* SCC DMA Address Pointer (Lowest byte)  */
+	{ 0xff8c08, SIZE_BYTE, IoMem_VoidRead_00, IoMem_VoidWrite },                            /* No bus error here */
+	{ 0xff8c09, SIZE_BYTE, IoMem_ReadWithoutInterception, IoMem_WriteWithoutInterception }, /* SCC DMA Byte Count (Highest byte)      */
+	{ 0xff8c0a, SIZE_BYTE, IoMem_VoidRead_00, IoMem_VoidWrite },                            /* No bus error here */
+	{ 0xff8c0b, SIZE_BYTE, IoMem_ReadWithoutInterception, IoMem_WriteWithoutInterception }, /* SCC DMA Byte Count (High byte)         */
+	{ 0xff8c0c, SIZE_BYTE, IoMem_VoidRead_00, IoMem_VoidWrite },                            /* No bus error here */
+	{ 0xff8c0d, SIZE_BYTE, IoMem_ReadWithoutInterception, IoMem_WriteWithoutInterception }, /* SCC DMA Byte Count (Low byte)          */
+	{ 0xff8c0e, SIZE_BYTE, IoMem_VoidRead_00, IoMem_VoidWrite },                            /* No bus error here */
+	{ 0xff8c0f, SIZE_BYTE, IoMem_ReadWithoutInterception, IoMem_WriteWithoutInterception }, /* SCC DMA Byte Count (Lowest byte)       */
+	{ 0xff8c10, SIZE_WORD, IoMem_ReadWithoutInterception, IoMem_WriteWithoutInterception }, /* SCC Residue Data Register (High Word)  */
+	{ 0xff8c12, SIZE_WORD, IoMem_ReadWithoutInterception, IoMem_WriteWithoutInterception }, /* SCC Residue Data Register (Low Word)   */
+	{ 0xff8c14, SIZE_WORD, IoMem_VoidRead_00, IoMem_WriteWithoutInterception },             /* SCC Control register                   */
+	{ 0xff8c16, 10, IoMem_VoidRead_00, IoMem_VoidWrite },                                   /* No bus error here */
 
-	{ 0xff8e01, 1, IoMem_ReadWithoutInterception, IoMem_WriteWithoutInterception },         /* SCU system interrupt mask */
-	{ 0xff8e03, 1, IoMem_ReadWithoutInterception, IoMem_WriteWithoutInterception },         /* SCU system interrupt state */
-	{ 0xff8e05, 1, IoMem_ReadWithoutInterception, IoMem_WriteWithoutInterception },         /* SCU system interrupter */
-	{ 0xff8e07, 1, IoMem_ReadWithoutInterception, IoMem_WriteWithoutInterception },         /* SCU VME interrupter */
-	{ 0xff8e09, 1, IoMem_ReadWithoutInterception, IoMem_WriteWithoutInterception },         /* SCU general purpose 1 */
-	{ 0xff8e0b, 1, IoMem_ReadWithoutInterception, IoMem_WriteWithoutInterception },         /* SCU general purpose 2 */
-	{ 0xff8e0d, 1, IoMem_ReadWithoutInterception, IoMem_WriteWithoutInterception },         /* SCU VME interrupt mask */
-	{ 0xff8e0f, 1, IoMem_ReadWithoutInterception, IoMem_WriteWithoutInterception },         /* SCU VME interrupt state */
+	{ 0xff8c80, 8, SCC_IoMem_ReadByte, SCC_IoMem_WriteByte },                               /* SCC */
+	{ 0xff8c88, 8, IoMem_VoidRead_00, IoMem_VoidWrite },                                    /* No bus error here */
+
+	{ 0xff8e01, SIZE_BYTE, SCU_SysIntMask_ReadByte, SCU_SysIntMask_WriteByte },
+	{ 0xff8e03, SIZE_BYTE, SCU_SysIntState_ReadByte, SCU_SysIntState_WriteByte },
+	{ 0xff8e05, SIZE_BYTE, SCU_SysInterrupter_ReadByte, SCU_SysInterrupter_WriteByte },
+	{ 0xff8e07, SIZE_BYTE, SCU_VmeInterrupter_ReadByte, SCU_VmeInterrupter_WriteByte },
+	{ 0xff8e09, SIZE_BYTE, SCU_GPR1_ReadByte, SCU_GPR1_WriteByte },
+	{ 0xff8e0b, SIZE_BYTE, SCU_GPR2_ReadByte, SCU_GPR2_WriteByte },
+	{ 0xff8e0d, SIZE_BYTE, SCU_VmeIntMask_Readyte, SCU_VmeIntMask_WriteByte },
+	{ 0xff8e0f, SIZE_BYTE, SCU_VmeIntState_ReadByte, SCU_VmeIntState_WriteByte },
 
 	{ 0xff9000, SIZE_WORD, IoMem_VoidRead, IoMem_VoidWrite },                /* No bus error here */
 	{ 0xff9200, SIZE_WORD, IoMemTabTT_ReadDIPSwitches, IoMem_VoidWrite },    /* DIP switches */
@@ -184,7 +227,39 @@ const INTERCEPT_ACCESS_FUNC IoMemTable_TT[] =
 	{ 0xfffa3d, SIZE_BYTE, IoMem_VoidRead, IoMem_VoidWrite },           /* No bus error here */
 	{ 0xfffa3f, SIZE_BYTE, IoMem_VoidRead, IoMem_VoidWrite },           /* No bus error here */
 
-	{ 0xfffa80, 48, IoMem_VoidRead, IoMem_WriteWithoutInterception },  /* 2nd TT MFP */
+	{ 0xfffa81, SIZE_BYTE, MFP_GPIP_ReadByte, MFP_GPIP_WriteByte },			/* TT MFP GPIP */
+	{ 0xfffa83, SIZE_BYTE, MFP_ActiveEdge_ReadByte, MFP_ActiveEdge_WriteByte },	/* TT MFP AER */
+	{ 0xfffa85, SIZE_BYTE, MFP_DataDirection_ReadByte, MFP_DataDirection_WriteByte	/* TT MFP DDR */},
+	{ 0xfffa87, SIZE_BYTE, MFP_EnableA_ReadByte, MFP_EnableA_WriteByte },		/* TT MFP IERA */
+	{ 0xfffa89, SIZE_BYTE, MFP_EnableB_ReadByte, MFP_EnableB_WriteByte },		/* TT MFP IERB */
+	{ 0xfffa8b, SIZE_BYTE, MFP_PendingA_ReadByte, MFP_PendingA_WriteByte },		/* TT MFP IPRA */
+	{ 0xfffa8d, SIZE_BYTE, MFP_PendingB_ReadByte, MFP_PendingB_WriteByte },		/* TT MFP IPRB */
+	{ 0xfffa8f, SIZE_BYTE, MFP_InServiceA_ReadByte, MFP_InServiceA_WriteByte },	/* TT MFP ISRA */
+	{ 0xfffa91, SIZE_BYTE, MFP_InServiceB_ReadByte, MFP_InServiceB_WriteByte },	/* TT MFP ISRB */
+	{ 0xfffa93, SIZE_BYTE, MFP_MaskA_ReadByte, MFP_MaskA_WriteByte },               /* TT MFP IMRA */
+	{ 0xfffa95, SIZE_BYTE, MFP_MaskB_ReadByte, MFP_MaskB_WriteByte },		/* TT MFP IMRB */
+	{ 0xfffa97, SIZE_BYTE, MFP_VectorReg_ReadByte, MFP_VectorReg_WriteByte },	/* TT MFP VR */
+	{ 0xfffa99, SIZE_BYTE, MFP_TimerACtrl_ReadByte, MFP_TimerACtrl_WriteByte },	/* TT MFP TACR */
+	{ 0xfffa9b, SIZE_BYTE, MFP_TimerBCtrl_ReadByte, MFP_TimerBCtrl_WriteByte },	/* TT MFP TBCR */
+	{ 0xfffa9d, SIZE_BYTE, MFP_TimerCDCtrl_ReadByte, MFP_TimerCDCtrl_WriteByte },	/* TT MFP TCDCR */
+	{ 0xfffa9f, SIZE_BYTE, MFP_TimerAData_ReadByte, MFP_TimerAData_WriteByte },	/* TT MFP TADR */
+	{ 0xfffaa1, SIZE_BYTE, MFP_TimerBData_ReadByte, MFP_TimerBData_WriteByte },	/* TT MFP TBDR */
+	{ 0xfffaa3, SIZE_BYTE, MFP_TimerCData_ReadByte, MFP_TimerCData_WriteByte },	/* TT MFP TCDR */
+	{ 0xfffaa5, SIZE_BYTE, MFP_TimerDData_ReadByte, MFP_TimerDData_WriteByte },	/* TT MFP TDDR */
+	{ 0xfffaa7, SIZE_BYTE, IoMem_VoidRead, IoMem_VoidWrite },			/* TT MFP SCR */
+	{ 0xfffaa9, SIZE_BYTE, IoMem_VoidRead, IoMem_VoidWrite },			/* TT MFP UCR */
+	{ 0xfffaab, SIZE_BYTE, IoMem_VoidRead, IoMem_VoidWrite },			/* TT MFP RSR */
+	{ 0xfffaad, SIZE_BYTE, IoMem_VoidRead, IoMem_VoidWrite },			/* TT MFP TSR */
+	{ 0xfffaaf, SIZE_BYTE, IoMem_VoidRead, IoMem_VoidWrite },			/* TT MFP UDR */
+
+	{ 0xfffab1, SIZE_BYTE, IoMem_VoidRead, IoMem_VoidWrite },           /* No bus error here */
+	{ 0xfffab3, SIZE_BYTE, IoMem_VoidRead, IoMem_VoidWrite },           /* No bus error here */
+	{ 0xfffab5, SIZE_BYTE, IoMem_VoidRead, IoMem_VoidWrite },           /* No bus error here */
+	{ 0xfffab7, SIZE_BYTE, IoMem_VoidRead, IoMem_VoidWrite },           /* No bus error here */
+	{ 0xfffab9, SIZE_BYTE, IoMem_VoidRead, IoMem_VoidWrite },           /* No bus error here */
+	{ 0xfffabb, SIZE_BYTE, IoMem_VoidRead, IoMem_VoidWrite },           /* No bus error here */
+	{ 0xfffabd, SIZE_BYTE, IoMem_VoidRead, IoMem_VoidWrite },           /* No bus error here */
+	{ 0xfffabf, SIZE_BYTE, IoMem_VoidRead, IoMem_VoidWrite },           /* No bus error here */
 
 	{ 0xfffc00, SIZE_BYTE, ACIA_IKBD_Read_SR, ACIA_IKBD_Write_CR },
 	{ 0xfffc02, SIZE_BYTE, ACIA_IKBD_Read_RDR, ACIA_IKBD_Write_TDR },
