@@ -1,11 +1,11 @@
-#!/usr/bin/env python
+#!/usr/bin/python3
 #
 # Hatari console:
 # Allows using Hatari shortcuts & debugger, changing paths, toggling
 # devices and changing Hatari command line options (even for things you
 # cannot change from the UI) from the console while Hatari is running.
 #
-# Copyright (C) 2008-2014 by Eero Tamminen
+# Copyright (C) 2008-2024 by Eero Tamminen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,15 +24,6 @@ import signal
 import socket
 import readline
 
-# Python v2:
-# - lacks Python v3 encoding arg for bytes()
-# - input() evaluates given string and fails on empty one
-if str is bytes:
-    def bytes(s, encoding):
-        return s
-    def input(prompt):
-        return raw_input(prompt)
-
 class Scancode:
     "Atari scancodes for keys without alphanumeric characters"
     # US keyboard scancode mapping for characters which need shift
@@ -125,13 +116,13 @@ class Hatari:
         "check Hatari compatibility and return error string if it's not"
         print("Using following Hatari binary:")
         os.system("which %s" % self.hataribin)
-        error = True
+        error = "Hatari lacks '--control-socket' option (Windows?)"
         pipe = os.popen(self.hataribin + " -h")
         for line in pipe.readlines():
             if line.find("--addr24") >= 0:
                 self.winuae = True
             if line.find("--control-socket") >= 0:
-                error = False
+                error = None
                 break
         try:
             pipe.close()
@@ -140,7 +131,7 @@ class Hatari:
         if error:
             print("ERROR: %s" % error)
             sys.exit(-1)
-        
+
     def is_running(self):
         if not self.pid:
             return False
@@ -154,7 +145,7 @@ class Hatari:
                 self.control = None
             return False
         return True
-    
+
     def run_hatari(self, args):
         if self.control:
             print("ERROR: Hatari is already running, stop it first")
@@ -173,7 +164,7 @@ class Hatari:
         else:
             # child runs Hatari
             allargs = [self.hataribin, "--control-socket", self.controlpath] + args
-            print("RUN:", allargs)
+            print('RUN: "%s"' % ' '.join(allargs))
             os.execvp(self.hataribin, allargs)
 
     def send_message(self, msg, fast = False):
@@ -191,19 +182,19 @@ class Hatari:
         else:
             print("ERROR: no Hatari (control socket)")
             return False
-        
+
     def change_option(self, option):
         return self.send_message("hatari-option %s" % option)
 
     def trigger_shortcut(self, shortcut):
         return self.send_message("hatari-shortcut %s" % shortcut)
-    
+
     def _shift_up(self):
         if self.shiftdown:
             self.shiftdown = False
             return self.send_message("hatari-event keyup %s" % Scancode.LeftShift, True)
         return True
-    
+
     def _unshifted_keypress(self, key):
         self._shift_up()
         if key == ' ':
@@ -239,10 +230,10 @@ class Hatari:
 
     def debug_command(self, cmd):
         return self.send_message("hatari-debug %s" % cmd)
-    
+
     def change_path(self, path):
         return self.send_message("hatari-path %s" % path)
-    
+
     def toggle_device(self, device):
         return self.send_message("hatari-toggle %s" % device)
 
@@ -271,14 +262,14 @@ class Hatari:
 class CommandInput:
     prompt = "hatari-command: "
     historysize = 99
-    
+
     def __init__(self, commands):
         readline.set_history_length(self.historysize)
         readline.parse_and_bind("tab: complete")
         readline.set_completer_delims(" \t\r\n")
         readline.set_completer(self.complete)
         self.commands = commands
-    
+
     def complete(self, text, state):
         idx = 0
         #print "text: '%s', state '%d'" % (text, state)
@@ -287,7 +278,7 @@ class CommandInput:
                 idx += 1
                 if idx > state:
                     return cmd
-    
+
     def loop(self):
         try:
             rawline = input(self.prompt)
@@ -304,23 +295,31 @@ class Tokens:
     "--confirm-quit",
     "--configfile",
     "--keymap",
+    "--country",
+    "--layout",
+    "--language",
     "--fast-forward",
+    "--auto",
     "--mono",
     "--monitor",
+    "--tos-res",
     "--fullscreen",
     "--window",
     "--grab",
+    "--resizable",
     "--frameskips",
+    "--slowdown",
+    "--mousewarp",
     "--statusbar",
     "--drive-led",
-    "--bpp",
-    "--borders",
-    "--desktop-st",
-    "--spec512",
-    "--zoom",
-    "--desktop",
     "--max-width",
     "--max-height",
+    "--zoom",
+    "--disable-video",
+    "--borders",
+    "--spec512",
+    "--video-timing",
+    "--desktop",
     "--force-max",
     "--aspect",
     "--vdi",
@@ -330,30 +329,51 @@ class Tokens:
     "--crop",
     "--avirecord",
     "--avi-vcodec",
+    "--png-level",
     "--avi-fps",
     "--avi-file",
+    "--screenshot-dir",
+    "--screenshot-format",
+    "--joystick",
     "--joy0",
     "--joy1",
     "--joy2",
     "--joy3",
     "--joy4",
     "--joy5",
-    "--joystick",
     "--printer",
     "--midi-in",
     "--midi-out",
+    "--midi",
     "--rs232-in",
     "--rs232-out",
+    "--scc-a-in",
+    "--scc-a-out",
+    "--scc-a-lan-in",
+    "--scc-a-lan-out",
+    "--scc-b-in",
+    "--scc-b-out",
+    "--drive-a",
+    "--drive-b",
+    "--drive-a-heads",
+    "--drive-b-heads",
     "--disk-a",
     "--disk-b",
     "--fastfdc",
     "--protect-floppy",
-    "--protect-hd",
     "--harddrive",
+    "--protect-hd",
+    "--gemdos-case",
+    "--gemdos-time",
+    "--gemdos-conv",
+    "--gemdos-drive",
     "--acsi",
+    "--scsi",
     "--ide-master",
     "--ide-slave",
+    "--ide-swap",
     "--memsize",
+    "--ttram",
     "--memstate",
     "--tos",
     "--patch-tos",
@@ -361,28 +381,41 @@ class Tokens:
     "--cpulevel",
     "--cpuclock",
     "--compatible",
+    "--data-cache",
+    "--cpu-exact",
+    "--addr24",
+    "--fpu",
+    "--fpu-softfloat",
+    "--mmu",
     "--machine",
     "--blitter",
     "--dsp",
+    "--rtc-year",
     "--timer-d",
     "--fast-boot",
-    "--rtc",
     "--mic",
     "--sound",
     "--sound-buffer-size",
+    "--sound-sync",
     "--ym-mixing",
     "--debug",
+    "--debug-except",
+    "--lilo",
     "--bios-intercept",
     "--conout",
+    "--disasm",
+    "--natfeats",
     "--trace",
     "--trace-file",
+    "--msg-repeat",
     "--parse",
     "--saveconfig",
-    "--no-parachute",
+    "--cmd-fifo",
     "--log-file",
     "--log-level",
     "--alert-level",
-    "--run-vbls"
+    "--run-vbls",
+    "--benchmark"
     ]
     shortcut_tokens = [
     "mousegrab",
@@ -406,16 +439,26 @@ class Tokens:
     device_tokens = [
     "printer",
     "rs232",
+    "scca",
+    "sccalan",
+    "sccb",
     "midi",
     ]
     path_tokens = [
     "memauto",
     "memsave",
+    "midiin",
     "midiout",
     "printout",
     "soundout",
     "rs232in",
-    "rs232out"
+    "rs232out",
+    "sccain",
+    "sccaout",
+    "sccalanin",
+    "sccalanout",
+    "sccbin",
+    "sccbout"
     ]
     # use the long variants of the commands for clarity
     debugger_tokens = [
@@ -432,6 +475,7 @@ class Tokens:
     "dspmemdump",
     "dspreg",
     "dspsymbols",
+    "echo",
     "evaluate",
     "help",
     "history",
@@ -448,6 +492,7 @@ class Tokens:
     "setopt",
     "stateload",
     "statesave",
+    "struct",
     "symbols",
     "trace"
     ]
@@ -519,11 +564,11 @@ give help when you give them invalid input.
     def do_sleep(self, line):
         items = line.split()[1:]
         try:
-            secs = int(items[0])
+            secs = float(items[0])
         except:
-            secs = 0
-        if secs > 0:
-            print("Sleeping for %d secs..." % secs)
+            secs = -1.0
+        if secs > 0.0:
+            print("Sleeping for %g secs..." % secs)
             time.sleep(secs)
         else:
             print("usage: sleep <seconds>")
@@ -594,7 +639,7 @@ class Main:
         exit = False
         if "--" not in args:
             return (args[1:], file, exit)
-        
+
         for arg in args:
             if arg == "--":
                 return (args[args.index("--")+1:], file, exit)
